@@ -35,6 +35,13 @@ my ($TRUE, $FALSE) = (1,0);
 # in Makefile.config.in.
 
 
+# $testCc is the command we use to do test compiles.  Note that test
+# compiles are never more than heuristics, because we may be configuring
+# a build that will happen on a whole different system, which will build
+# programs to run on a third system.
+
+my $testCc;
+
 #******************************************************************************
 #
 #  SUBROUTINES
@@ -66,6 +73,146 @@ sub prompt($$) {
 
 
 
+sub tmpdir() {
+# This is our approximation of File::Spec->tmpdir(), which became part of
+# basic Perl some time after Perl 5.005_03.
+
+    my $retval;
+    
+    if ($ENV{"TMPDIR"}) {
+        $retval = $ENV{"TMPDIR"};
+    } else {
+        if ($Config{'osvers'} eq "djgpp") {
+            $retval = "/dev/env/DJDIR/tmp";
+        } else {
+            $retval =  "/tmp";
+        }
+    }
+    return $retval;
+}
+
+
+
+sub tempFile($) {
+
+# Here's what we'd do if we could expect Perl 5.6.1 or later, instead
+# of calling this subroutine:
+#    my ($cFile, $cFileName) = tempfile("netpbmXXXX", 
+#                                       SUFFIX=>".c", 
+#                                       DIR=>File::Spec->tmpdir(),
+#                                       UNLINK=>0);
+    my ($suffix) = @_;
+    my $fileName;
+    local *file;  # For some inexplicable reason, must be local, not my
+    my $i;
+    $i = 0;
+    do {
+        $fileName = tmpdir() . "/netpbm" . $i++ . $suffix;
+    } until sysopen(*file, $fileName, O_RDWR|O_CREAT|O_EXCL);
+
+    return(*file, $fileName);
+}
+
+
+
+sub commandExists($) {
+    my ($command) = @_;
+#-----------------------------------------------------------------------------
+#  Return TRUE iff a shell command $command exists.
+#-----------------------------------------------------------------------------
+
+# Note that it's significant that the redirection on the following
+# causes it to be executed in a shell.  That makes the return code
+# from system() a lot different than if system() were to try to
+# execute the program directly.
+
+    return(system("$command 1</dev/null 1>/dev/null 2>/dev/null")/256 != 127); 
+}
+
+
+
+sub chooseTestCompiler($$) {
+
+    my ($compiler, $testCcR) = @_;
+
+    my $cc;
+
+    if (!defined($compiler)) {
+        if ($ENV{"CC"}) {
+            $cc = $ENV{"CC"};
+        } else {
+            if (commandExists("cc")) {
+                $cc = "cc";
+            } elsif (commandExists("gcc")) {
+                $cc = "gcc";
+            }
+        }
+    } elsif ($compiler eq "cc") {
+        $cc = "cc";
+    } elsif ($compiler eq "gcc") {
+        $cc = "gcc";
+    } else {
+        die("Internal error: invalid value \"$compiler\" for \$compiler");
+    }
+    $$testCcR = $cc;
+}
+
+
+
+sub testCflags($) {
+    my ($needLocal) = @_;
+
+    my $cflags;
+
+    $cflags = "";  # initial value 
+    
+    if ($ENV{"CPPFLAGS"}) {
+        $cflags = $ENV{"CPPFLAGS"};
+    } else {
+        $cflags = "";
+    }
+    
+    if ($ENV{"CFLAGS"}) {
+        $cflags .= " " . $ENV{"CFLAGS"};
+    }
+    
+    if ($needLocal) {
+        $cflags .= " -I/usr/local/include";
+    }
+    return $cflags;
+}    
+
+
+
+sub testCompile($$$) {
+    my ($cflags, $cSourceCodeR, $successR) = @_;
+#-----------------------------------------------------------------------------
+#  Do a test compile of the program in @{$cSourceCodeR}.
+#  
+#  Return $$successR == $TRUE iff the compile succeeds (exit code 0).
+#-----------------------------------------------------------------------------
+    my ($cFile, $cFileName) = tempFile(".c");
+
+    print $cFile @{$cSourceCodeR};
+    
+    my ($oFile, $oFileName) = tempFile(".o");
+    # Note: we tried using /dev/null for the output file and got complaints
+    # from the Sun compiler that it has the wrong suffix.  2002.08.09.
+    
+    my $compileCommand = "$testCc -c -o $oFileName $cflags $cFileName";
+    print ("Doing test compile: $compileCommand\n");
+    my $rc = system($compileCommand);
+    
+    unlink($oFileName);
+    close($oFile);
+    unlink($cFileName);
+    close($cFile);
+
+    $$successR = ($rc == 0);
+}
+
+
+
 sub displayIntroduction() {
     print("This is the Netpbm configurator.  It is an interactive dialog " .
           "that\n");
@@ -438,20 +585,214 @@ sub libSuffix($) {
 
 
 
+sub getLibTypes($$$$$$$$) {
+    my ($platform, $subplatform, $default_target,
+        $netpbmlibtypeR, $netpbmlibsuffixR, $shlibprefixlistR,
+        $willBuildSharedR, $staticlib_tooR) = @_;
+
+    print("Do you want libnetpbm to be statically linked or shared?\n");
+    print("\n");
+
+    my $default = ($default_target eq "merge") ? "static" : "shared";
+
+    my ($netpbmlibtype, $netpbmlibsuffix, $shlibprefixlist, 
+        $willBuildShared, $staticlib_too);
+    
+    my $response = prompt("static or shared", $default);
+    
+    if ($response eq "shared") {
+        $willBuildShared = $TRUE;
+        if ($platform eq "WINDOWS") {
+            $netpbmlibtype = "dll";
+            $netpbmlibsuffix = "dll";
+            if ($subplatform eq "cygwin") {
+                $shlibprefixlist = "cyg lib";
+            } 
+        } elsif ($platform eq "DARWIN") {
+            $netpbmlibtype = "dylib";
+            $netpbmlibsuffix = "dylib";
+        } else {
+	    if ($platform eq "IRIX") {
+		$netpbmlibtype = "irixshared";
+	    } else {
+		$netpbmlibtype = "unixshared";
+	    }
+            if ($platform eq "AIX") {
+                $netpbmlibsuffix = "a";
+            } elsif ($platform eq "HP-UX") {
+                $netpbmlibsuffix = "sl";
+            } else {
+                $netpbmlibsuffix = "so";
+            }
+        }
+    } elsif ($response eq "static") {
+        $willBuildShared = $FALSE;
+        $netpbmlibtype = "unixstatic";
+        $netpbmlibsuffix = "a";
+        # targets, but needed for building
+        # libopt 
+    } else {
+        print("'$response' isn't one of the choices.  \n" .
+              "You must choose 'static' or 'shared'.\n");
+        exit 12;
+    }
+
+    print("\n");
+
+    # Note that we can't do both a static and shared library for AIX, because
+    # they both have the same name: libnetpbm.a.
+    
+    if (($netpbmlibtype eq "unixshared" or 
+         $netpbmlibtype eq "irixshared" or 
+         $netpbmlibtype eq "dll") and $netpbmlibsuffix ne "a") {
+        print("Do you want to build static libraries too (for linking to \n");
+        print("programs not in the Netpbm package?\n");
+        print("\n");
+        
+        my $default = "y";
+        
+        my $response = prompt("(y)es or (n)o", $default);
+        
+        if (uc($response) =~ /^(Y|YES)$/)  {
+            $staticlib_too = "y";
+        } elsif (uc($response) =~ /^(N|NO)$/)  {
+            $staticlib_too = "n";
+        } else {
+            print("'$response' isn't one of the choices.  \n" .
+              "You must choose 'yes' or 'no' (or 'y' or 'n').\n");
+            exit 12;
+        }
+    } else {
+        $staticlib_too = "n";
+    }
+    print("\n");
+
+    $$netpbmlibtypeR   = $netpbmlibtype;
+    $$netpbmlibsuffixR = $netpbmlibsuffix;
+    $$shlibprefixlistR = $shlibprefixlist;
+    $$willBuildSharedR = $willBuildShared;
+    $$staticlib_tooR   = $staticlib_too;
+}
+
+
+
+sub inttypesDefault() {
+
+    my $retval;
+
+    if (defined($testCc)) {
+
+        print("(Doing test compiles to choose a default for you -- " .
+              "ignore errors)\n");
+
+        my $cflags = testCflags($FALSE);
+
+        my $works;
+
+        my @candidateList = ("<inttypes.h>", "<types.h>", 
+                             "<sys/inttypes.h>", "<sys/types.h>");
+        
+        for (my $i = 0; $i < @candidateList && !$works; ++$i) {
+            my $candidate = $candidateList[$i];
+            my @cSourceCode = (
+                               "#include $candidate\n",
+                               "uint32_t testvar;\n"
+                               );
+            
+            testCompile($cflags, \@cSourceCode, \my $success);
+            
+            if ($success) {
+                $works = $candidate;
+            }
+        }
+        if ($works) {
+            $retval = $works;
+        } else {
+            testCompile($cflags, ["uint32_t testvar;"], \my $success);
+            if ($success) {
+                $retval = "NONE";
+            } else {
+                $retval = '"inttypes_netpbm.h"';
+            }
+        }
+        print("\n");
+    } else {
+        $retval = '<inttypes.h>';
+    }
+    return $retval;
+}
+
+
+
+sub getInttypes($) {
+    my ($inttypesHeaderFileR) = @_;
+
+    my $gotit;
+
+    print("What header file defines uint32_t, etc.?\n");
+    print("\n");
+
+    my $default = inttypesDefault();
+    
+    while (!$gotit) {
+        my $response = prompt("'#include' argument or NONE", $default);
+
+        if ($response eq "NONE") {
+            $$inttypesHeaderFileR = '';
+            $gotit = $TRUE;
+        } else {
+            if ($response !~ m{<.+>} &&
+                $response !~ m{".+"}) {
+                print("'$response' is not a legal argument of a C #include " .
+                      "statement.  It must be something in <> or \"\".\n");
+            } else {
+                $gotit = $TRUE;
+                $$inttypesHeaderFileR = $response;
+            }
+        }
+    }
+}
+
+
+sub getInt64($$) {
+
+    my ($inttypes_h, $haveInt64R) = @_;
+
+    if (defined($testCc)) {
+
+        print("(Doing test compiles to determine if you have int64 type -- " .
+              "ignore errors)\n");
+
+        my $cflags = testCflags($FALSE);
+
+        my $works;
+
+        my @cSourceCode = (
+                           "#include $inttypes_h\n",
+                           "int64_t testvar;\n"
+                           );
+            
+        testCompile($cflags, \@cSourceCode, \my $success);
+            
+        if ($success) {
+            print("You do.\n");
+            $$haveInt64R = 'Y';
+        } else {
+            print("You do not.  64-bit code won't be built.\n");
+            $$haveInt64R = 'N';
+        }
+        print("\n");
+    } else {
+        $$haveInt64R = "N";
+    }
+}
+
+
+
 # TODO: These should do test compiles to see if the headers are in the
 # default search path, both to create a default to offer and to issue a
 # warning after user has chosen.  Also test links to test the link library.
 
-# Before we had prompts for library locations, we set values based on 
-# the platform type, as follows:
-#
-#    TRU64: /usr/local1/DEC/include, /usr/local1/DEC/lib
-#    IRIX:  default                  /usr/lib32
-#                                        this goes with our -n32 option
-#    NETBSD: $(LOCALBASE)/include    $(LOCALBASE)/lib
-#            where LOCALBASE is a build-time environment variable.
-#    DARWIN: same as NETBSD
-#
 # It looks like these should all be in the default search paths and were there
 # just to override defaults in Makefile.config.in.  Since Configure now
 # creates a default of "standard search path," I'm guessing we don't need
@@ -480,7 +821,9 @@ sub getTiffLibrary($@) {
     if (defined($tifflib)) {
         my $default;
 
-        if (-d("/usr/include/libtiff")) {
+        if (-d("/usr/include/tiff")) {
+            $default = "/usr/include/tiff";
+        } elsif (-d("/usr/include/libtiff")) {
             $default = "/usr/include/libtiff";
         } else {
             $default = "default";
@@ -620,21 +963,6 @@ sub getZLibrary($@) {
 
 
 
-sub commandExists($) {
-    my ($command) = @_;
-#-----------------------------------------------------------------------------
-#  Return TRUE iff a shell command $command exists.
-#-----------------------------------------------------------------------------
-
-# Note that it's significant that the redirection on the following
-# causes it to be executed in a shell.  That makes the return code
-# from system() a lot different than if system() were to try to
-# execute the program directly.
-
-    return(system("$command 1</dev/null 1>/dev/null 2>/dev/null")/256 != 127); 
-}
-
-
 sub symlink_command() {
 
     my $retval;
@@ -735,7 +1063,8 @@ sub help() {
 
 
 
-sub gnuOptimizeOpt() {
+sub gnuOptimizeOpt($) {
+    my ($gccCommandName) = @_;
 #-----------------------------------------------------------------------------
 #  Compute the -O compiler flag appropriate for a GNU system.  Ordinarily,
 #  this is just -O3.  But many popular GNU systems have a broken compiler
@@ -752,7 +1081,7 @@ sub gnuOptimizeOpt() {
 # subroutines in Netpbm marked for explicit inlining, and we need to disable
 # that inlining too, so we must go all the way down to -O0.
 
-    my @gccVerboseResp = `gcc --verbose 2>&1`;
+    my @gccVerboseResp = `$gccCommandName --verbose 2>&1`;
 
     my $brokenCompiler;
     
@@ -797,12 +1126,26 @@ sub needLocal($) {
 #  the compiler and linker default search paths, e.g. the compiler
 #  should search /usr/local/include and then /usr/include without any
 #  -I options needed.  But we've seen some systems where it isn't.
+#
+#  Actually, I have doubts now as to whether these misconfigured systems
+#  really exist.  This subroutine was apparently always broken, because
+#  before 04.03.15, it had "netbsd", etc. in lower case.  So it always
+#  returned false.  I never had a complaint.  Plus, we had a bug in 
+#  Makefile.config.in wherein it wiped out the user's setting of the LDFLAGS
+#  environment variable.  This could explain /usr/local/lib not being in
+#  the path when it should have been.
+#
+#  So I've disabled this function; we'll see if we encounter any truly
+#  misconfigured systems.  04.03.15.
 #-----------------------------------------------------------------------------
     my ($platform) = @_;
 
+    return $FALSE;  # See comments above.
+
     my $needLocal;
     
-    if ($platform eq "netbsd" || $platform eq "openbsd") {
+    if ($platform eq "NETBSD" || $platform eq "OPENBSD" || 
+        $platform eq "FREEBSD") {
         $needLocal = $TRUE;
     } else {
         $needLocal = $FALSE;
@@ -850,132 +1193,8 @@ sub warnJpegTiffDependency($$) {
 
 
 
-sub tmpdir() {
-# This is our approximation of File::Spec->tmpdir(), which became part of
-# basic Perl some time after Perl 5.005_03.
-
-    my $retval;
-    
-    if ($ENV{"TMPDIR"}) {
-        $retval = $ENV{"TMPDIR"};
-    } else {
-        if ($Config{'osvers'} eq "djgpp") {
-            $retval = "/dev/env/DJDIR/tmp";
-        } else {
-            $retval =  "/tmp";
-        }
-    }
-    return $retval;
-}
-
-
-
-sub tempFile($) {
-
-# Here's what we'd do if we could expect Perl 5.6.1 or later, instead
-# of calling this subroutine:
-#    my ($cFile, $cFileName) = tempfile("netpbmXXXX", 
-#                                       SUFFIX=>".c", 
-#                                       DIR=>File::Spec->tmpdir(),
-#                                       UNLINK=>1);
-    my ($suffix) = @_;
-    my $fileName;
-    local *file;  # For some inexplicable reason, must be local, not my
-    my $i;
-    $i = 0;
-    do {
-        $fileName = tmpdir() . "/netpbm" . $i++ . $suffix;
-    } until sysopen(*file, $fileName, O_RDWR|O_CREAT|O_EXCL);
-
-    return(*file, $fileName);
-}
-
-
-
-sub chooseTestCompiler($) {
-
-    my ($compiler) = @_;
-
-    my $cc;
-
-    if (!defined($compiler)) {
-        if ($ENV{"CC"}) {
-            $cc = $ENV{"CC"};
-        } else {
-            if (commandExists("cc")) {
-                $cc = "cc";
-            } elsif (commandExists("gcc")) {
-                $cc = "gcc";
-            }
-        }
-    } elsif ($compiler eq "cc") {
-        $cc = "cc";
-    } elsif ($compiler eq "gcc") {
-        $cc = "gcc";
-    } else {
-        die("Internal error: invalid value \"$compiler\" for \$compiler");
-    }
-    return $cc;
-}
-
-
-
-sub testCflags($) {
-    my ($needLocal) = @_;
-
-    my $cflags;
-
-    $cflags = "";  # initial value 
-    
-    if ($ENV{"CPPFLAGS"}) {
-        $cflags = $ENV{"CPPFLAGS"};
-    } else {
-        $cflags = "";
-    }
-    
-    if ($ENV{"CFLAGS"}) {
-        $cflags .= " " . $ENV{"CFLAGS"};
-    }
-    
-    if ($needLocal) {
-        $cflags .= " -I/usr/local/include";
-    }
-    return $cflags;
-}    
-
-
-
-sub testCompile($$$$) {
-    my ($cc, $cflags, $cSourceCodeR, $successR) = @_;
-#-----------------------------------------------------------------------------
-#  Do a test compile of the program in @{$cSourceCodeR}.
-#  
-#  Return $$successR == $TRUE iff the compile succeeds (exit code 0).
-#-----------------------------------------------------------------------------
-    my ($cFile, $cFileName) = tempFile(".c");
-
-    print $cFile @{$cSourceCodeR};
-    
-    my ($oFile, $oFileName) = tempFile(".o");
-    # Note: we tried using /dev/null for the output file and got complaints
-    # from the Sun compiler that it has the wrong suffix.  2002.08.09.
-    
-    my $compileCommand = "$cc -c -o $oFileName $cflags $cFileName";
-    print ("Doing test compile: $compileCommand\n");
-    my $rc = system($compileCommand);
-    
-    unlink($oFileName);
-    close($oFile);
-    unlink($cFileName);
-    close($cFile);
-
-    $$successR = ($rc == 0);
-}
-
-
-
-sub testCompileJpeglibH($$$) {
-    my ($cc, $cflags, $successR) = @_;
+sub testCompileJpeglibH($$) {
+    my ($cflags, $successR) = @_;
 #-----------------------------------------------------------------------------
 #  Do a test compile to see if we can see jpeglib.h.
 #-----------------------------------------------------------------------------
@@ -985,13 +1204,13 @@ sub testCompileJpeglibH($$$) {
                        "#include <jpeglib.h>\n",
                        );
     
-    testCompile($cc, $cflags, \@cSourceCode, $successR);
+    testCompile($cflags, \@cSourceCode, $successR);
 }
 
 
 
-sub testCompileJpegMarkerStruct($$$) {
-    my ($cc, $cflags, $successR) = @_;
+sub testCompileJpegMarkerStruct($$) {
+    my ($cflags, $successR) = @_;
 #-----------------------------------------------------------------------------
 #  Do a test compile to see if struct jpeg_marker_struct is defined in 
 #  jpeglib.h.  Assume it is already established that the compiler works
@@ -1004,7 +1223,7 @@ sub testCompileJpegMarkerStruct($$$) {
                        "struct jpeg_marker_struct test;\n",
                        );
 
-    testCompile($cc, $cflags, \@cSourceCode, $successR);
+    testCompile($cflags, \@cSourceCode, $successR);
 }
 
 
@@ -1052,19 +1271,17 @@ sub printOldJpegWarning() {
 
 
 
-sub testJpegHdr($$$) {
+sub testJpegHdr($$) {
 
-    my ($compiler, $needLocal, $jpeghdr_dir) = @_;
+    my ($needLocal, $jpeghdr_dir) = @_;
 
-    my $cc = chooseTestCompiler($compiler);
-
-    if (defined($cc)) {
+    if (defined($testCc)) {
 
         my $generalCflags = testCflags($needLocal);
 
         my $jpegIOpt = $jpeghdr_dir ? "-I$jpeghdr_dir" : "";
 
-        testCompileJpeglibH($cc, "$generalCflags $jpegIOpt", \my $success);
+        testCompileJpeglibH("$generalCflags $jpegIOpt", \my $success);
 
         if (!$success) {
             print("\n");
@@ -1072,7 +1289,7 @@ sub testJpegHdr($$$) {
         } else {
             # We can get to something named jpeglib.h, but maybe it's an old
             # version we can't use.  Check it out.
-            testCompileJpegMarkerStruct($cc, "$generalCflags $jpegIOpt", 
+            testCompileJpegMarkerStruct("$generalCflags $jpegIOpt", 
                                         \my $success);
             if (!$success) {
                 print("\n");
@@ -1084,8 +1301,8 @@ sub testJpegHdr($$$) {
 
 
 
-sub testCompileZlibH($$$) {
-    my ($cc, $cflags, $successR) = @_;
+sub testCompileZlibH($$) {
+    my ($cflags, $successR) = @_;
 #-----------------------------------------------------------------------------
 #  Do a test compile to see if we can see zlib.h.
 #-----------------------------------------------------------------------------
@@ -1093,13 +1310,13 @@ sub testCompileZlibH($$$) {
                        "#include <zlib.h>\n",
                        );
     
-    testCompile($cc, $cflags, \@cSourceCode, $successR);
+    testCompile($cflags, \@cSourceCode, $successR);
 }
 
 
 
-sub testCompilePngH($$$) {
-    my ($cc, $cflags, $successR) = @_;
+sub testCompilePngH($$) {
+    my ($cflags, $successR) = @_;
 #-----------------------------------------------------------------------------
 #  Do a test compile to see if we can see png.h, assuming we can see
 #  png.h, which png.h #includes.
@@ -1108,34 +1325,31 @@ sub testCompilePngH($$$) {
                        "#include <png.h>\n",
                        );
     
-    testCompile($cc, $cflags, \@cSourceCode, $successR);
+    testCompile($cflags, \@cSourceCode, $successR);
 }
 
 
 
-sub testPngHdr($$$$) {
+sub testPngHdr($$$) {
 #-----------------------------------------------------------------------------
 #  Issue a warning if the compiler can't find png.h.
 #-----------------------------------------------------------------------------
-    my ($compiler, $needLocal, $pnghdr_dir, $zhdr_dir) = @_;
+    my ($needLocal, $pnghdr_dir, $zhdr_dir) = @_;
 
-    my $cc = chooseTestCompiler($compiler);
-
-    if (defined($cc)) {
+    if (defined($testCc)) {
 
         my $generalCflags = testCflags($needLocal);
 
         my $zlibIOpt = $zhdr_dir ? "-I$zhdr_dir" : "";
 
-        testCompileZlibH($cc, "$generalCflags $zlibIOpt", \my $success);
+        testCompileZlibH("$generalCflags $zlibIOpt", \my $success);
         if (!$success) {
             print("\n");
             printMissingHdrWarning("Zlib", $zhdr_dir);
         } else {
             my $pngIOpt = $pnghdr_dir ? "-I$pnghdr_dir" : "";
 
-            testCompilePngH($cc, 
-                            "$generalCflags $zlibIOpt $pngIOpt", 
+            testCompilePngH("$generalCflags $zlibIOpt $pngIOpt", 
                             \my $success);
 
             if (!$success) {
@@ -1148,16 +1362,16 @@ sub testPngHdr($$$$) {
 
 
 
-sub testConfiguration($$$$$$$$) {
+sub testConfiguration($$$$$$$) {
 
-    my ($compiler, $needLocal, $jpeglib, $jpeghdr_dir,
+    my ($needLocal, $jpeglib, $jpeghdr_dir,
         $pnglib, $pnghdr_dir, $zlib, $zhdr_dir) = @_;
 
     if (defined($jpeglib)) {
-        testJpegHdr($compiler, $needLocal, $jpeghdr_dir);
+        testJpegHdr($needLocal, $jpeghdr_dir);
     }
     if (defined($pnglib) && defined($zlib)) {
-        testPngHdr($compiler, $needLocal, $pnghdr_dir, $zhdr_dir);
+        testPngHdr($needLocal, $pnghdr_dir, $zhdr_dir);
     }
 
     # TODO: We ought to validate other libraries too.  But it's not
@@ -1225,6 +1439,8 @@ if ($platform eq "NONE") {
 getCompiler($platform, \my $compiler);
 getLinker($platform, $compiler, \my $baseLinker, \my $linkViaCompiler);
 
+chooseTestCompiler($compiler, \$testCc);
+
 my $netpbmlib_runtime_path;
     # Undefined if the default from Makefile.config.in is acceptable.
 
@@ -1271,87 +1487,14 @@ print("\n");
 
 print("\n");
 
-my ($netpbmlibtype, $netpbmlibsuffix, $shlibprefixlist);
-# Undefined means the default from Makefile.config.in is acceptable.
+getLibTypes($platform, $subplatform, $default_target,
+            \my $netpbmlibtype, \my $netpbmlibsuffix, \my $shlibprefixlist,
+            \my $willBuildShared, \my $staticlib_too);
 
-my $willBuildShared;
 
-print("Do you want libnetpbm to be statically linked or shared?\n");
-print("\n");
+getInttypes(\my $inttypesHeaderFile);
 
-{
-    my $default = ($default_target eq "merge") ? "static" : "shared";
-
-    my $response = prompt("static or shared", $default);
-
-    if ($response eq "shared") {
-        $willBuildShared = $TRUE;
-        if ($platform eq "WINDOWS") {
-            $netpbmlibtype = "dll";
-            $netpbmlibsuffix = "dll";
-            if ($subplatform eq "cygwin") {
-                $shlibprefixlist = "cyg lib";
-            } 
-        } elsif ($platform eq "DARWIN") {
-            $netpbmlibtype = "dylib";
-            $netpbmlibsuffix = "dylib";
-        } else {
-	    if ($platform eq "IRIX") {
-		$netpbmlibtype = "irixshared";
-	    } else {
-		$netpbmlibtype = "unixshared";
-	    }
-            if ($platform eq "AIX") {
-                $netpbmlibsuffix = "a";
-            } elsif ($platform eq "HP-UX") {
-                $netpbmlibsuffix = "sl";
-            } else {
-                $netpbmlibsuffix = "so";
-            }
-        }
-    } elsif ($response eq "static") {
-        $willBuildShared = $FALSE;
-        $netpbmlibtype = "unixstatic";
-        $netpbmlibsuffix = "a";
-        # targets, but needed for building
-        # libopt 
-    } else {
-        print("'$response' isn't one of the choices.  \n" .
-              "You must choose 'static' or 'shared'.\n");
-        exit 12;
-    }
-}
-print("\n");
-
-my $staticlib_too;
-
-# Note that we can't do both a static and shared library for AIX, because
-# they both have the same name: libnetpbm.a.
-
-if (($netpbmlibtype eq "unixshared" or 
-     $netpbmlibtype eq "irixshared" or 
-     $netpbmlibtype eq "dll") and $netpbmlibsuffix ne "a") {
-    print("Do you want to build static libraries too (for linking to \n");
-    print("programs not in the Netpbm package?\n");
-    print("\n");
-    
-    my $default = "y";
-
-    my $response = prompt("(y)es or (n)o", $default);
-
-    if (uc($response) =~ /^(Y|YES)$/)  {
-        $staticlib_too = "y";
-    } elsif (uc($response) =~ /^(N|NO)$/)  {
-        $staticlib_too = "n";
-    } else {
-        print("'$response' isn't one of the choices.  \n" .
-              "You must choose 'yes' or 'no' (or 'y' or 'n').\n");
-        exit 12;
-    }
-} else {
-    $staticlib_too = "n";
-}
-print("\n");
+getInt64($inttypesHeaderFile, \my $haveInt64);
 
 #******************************************************************************
 #
@@ -1437,7 +1580,7 @@ validateLibraries($jpeglib, $tifflib, $pnglib, $zlib);
 
 warnJpegTiffDependency($jpeglib, $tifflib);
 
-testConfiguration($compiler, needLocal($platform), 
+testConfiguration(needLocal($platform), 
                   $jpeglib, $jpeghdr_dir,
                   $pnglib, $pnghdr_dir,
                   $zlib, $zhdr_dir,
@@ -1486,17 +1629,27 @@ if (-f("GNUmakefile")) {
     $defaultConfigInPath = "$srcdir/Makefile.config.in";
 }
 
+sub makeCompilerGcc($) {
+    my ($Makefile_configR) = @_;
+    my $compileCommand = 'gcc';
+    push(@{$Makefile_configR}, "CC = $compileCommand\n");
+    push(@{$Makefile_configR}, gnuCflags($compileCommand));
+}
+
+
 #******************************************************************************
 #
 #  BUILD Makefile.config
 #
 #*****************************************************************************
 
-sub gnuCflags() {
-    return("CFLAGS = " . gnuOptimizeOpt() . " -ffast-math " .
-           " -pedantic " . 
+sub gnuCflags($) {
+    my ($gccCommandName) = @_;
+
+    return("CFLAGS = " . gnuOptimizeOpt($gccCommandName) . " -ffast-math " .
+           " -pedantic -fno-common " . 
            "-Wall -Wno-uninitialized -Wmissing-declarations -Wimplicit " .
-           "-Wwrite-strings\n");
+           "-Wwrite-strings -Wmissing-prototypes\n");
 }
 
 my @Makefile_config;
@@ -1550,14 +1703,20 @@ if (defined($netpbmlib_runtime_path)) {
 }
 
 if ($platform eq "GNU") {
+    my $compileCommand;
     if (!commandExists("cc") && commandExists("gcc")) {
-        push(@Makefile_config, "CC = gcc\n");
+        $compileCommand = "gcc";
+        push(@Makefile_config, "CC = $compileCommand\n");
+    } else {
+        $compileCommand = "cc";
     }
-    push(@Makefile_config, gnuCflags());
+    push(@Makefile_config, gnuCflags($compileCommand));
 # The merged programs have a main_XXX subroutine instead of main(),
-# which would cause a warning with -Wmissing-declarations
-    push(@Makefile_config, "CFLAGS_MERGE = -Wno-missing-declarations\n");
-    push(@Makefile_config, "LDRELOC = ld --relocateable\n");
+# which would cause a warning with -Wmissing-declarations or 
+# -Wmissing-prototypes.
+    push(@Makefile_config, "CFLAGS_MERGE = " .
+         "-Wno-missing-declarations -Wno-missing-prototypes\n");
+    push(@Makefile_config, "LDRELOC = ld --reloc\n");
     push(@Makefile_config, "LINKER_CAN_DO_EXPLICIT_LIBRARY=Y\n");
 } elsif ($platform eq "SOLARIS") {
     push(@Makefile_config, 'LDSHLIB = -Wl,-Bdynamic,-G,-h,$(SONAME)', "\n");
@@ -1567,8 +1726,7 @@ if ($platform eq "GNU") {
         push(@Makefile_config, "CFLAGS = -O\n");
         push(@Makefile_config, "CFLAGS_SHLIB = -Kpic\n");
     } else {
-        push(@Makefile_config, "CC = gcc\n");
-        push(@Makefile_config, gnuCflags());
+        makeCompilerGcc(\@Makefile_config);
     }
     # Before Netpbm 10.20 (January 2004), we set this to -R for 
     # $compiler == cc and -rpath otherwise.  But now we know that the GNU
@@ -1581,17 +1739,14 @@ if ($platform eq "GNU") {
     push(@Makefile_config, "NETWORKLD = -lsocket -lnsl\n");
 } elsif ($platform eq "HP-UX") {
     if ($compiler eq "gcc") {
-        push(@Makefile_config, "CC = gcc\n");
-        push(@Makefile_config, gnuCflags());
+        makeCompilerGcc(\@Makefile_config);
         push(@Makefile_config, "CFLAGS += -fPIC\n");
         push(@Makefile_config, "LDSHLIB = -shared -fPIC\n");
-        push(@Makefile_config, 'LDFLAGS = -Wl,+b,/usr/pubsw/lib', "\n");
+        push(@Makefile_config, 'LDFLAGS += -Wl,+b,/usr/pubsw/lib', "\n");
     } else {
         # We don't know what to do here.  We used to (before 10.20) just
         # just assume the compiler was gcc.  We know that the gcc stuff
         # above does NOT work for HP native compiler.
-        push(@Makefile_config, "CFLAGS = \n");
-        push(@Makefile_config, "LDFLAGS = \n");
     }
 } elsif ($platform eq "AIX") {
     push(@Makefile_config, 'LDFLAGS = -L /usr/pubsw/lib', "\n");
@@ -1601,14 +1756,13 @@ if ($platform eq "GNU") {
         push(@Makefile_config, "RPATHOPTNAME = -L\n");
         push(@Makefile_config, "LDSHLIB = -qmkshrobj\n");
     } else {
-        push(@Makefile_config, "CC = gcc\n");
-        push(@Makefile_config, gnuCflags());
+        makeCompilerGcc(\@Makefile_config);
         push(@Makefile_config, "LDSHLIB = -shared\n");
     }
 } elsif ($platform eq "TRU64") {
 #    push(@Makefile_config, "INSTALL = installbsd\n");
     if ($compiler eq "cc") {
-        push(@Makefile_config, 'CFLAGS = -O2 -std1 $(CADD)', "\n");
+        push(@Makefile_config, 'CFLAGS = -O2 -std1', "\n");
         push(@Makefile_config, "LDFLAGS = -call_shared -oldstyle_liblookup " .
              "-L/usr/local/lib\n");
         push(@Makefile_config, "LDSHLIB = -shared -expect_unresolved \"*\"\n");
@@ -1616,7 +1770,7 @@ if ($platform eq "GNU") {
         # We've never tested this.  This is just here to give a user a 
         # headstart on submitting to us the necessary information.  2002.07.04.
         push(@Makefile_config, "CC = gcc\n");
-        push(@Makefile_config, 'CFLAGS = -O3 $(CADD)', "\n");
+        push(@Makefile_config, 'CFLAGS = -O3', "\n");
         push(@Makefile_config, "LDSHLIB = -shared\n");
     }
     # Between May 2000 and July 2003, we had -DLONG_32 in these options.
@@ -1631,13 +1785,12 @@ if ($platform eq "GNU") {
 #    push(@Makefile_config, "INSTALL = install\n");
     push(@Makefile_config, "MANPAGE_FORMAT = cat\n");
     push(@Makefile_config, "RANLIB = true\n");
-    push(@Makefile_config, "CFLAGS = -n32 -O3\n");
+    push(@Makefile_config, "CFLAGS = -n32 -O3 -fullwarn\n");
     push(@Makefile_config, "LDFLAGS = -n32\n");
     push(@Makefile_config, "LDSHLIB = -shared -n32\n");
 } elsif ($platform eq "WINDOWS") {
     if ($subplatform eq "cygwin") {
-        push(@Makefile_config, "CC = gcc\n");
-        push(@Makefile_config, gnuCflags());
+        makeCompilerGcc(\@Makefile_config);
     }
     push(@Makefile_config, "EXE = .exe\n");
     push(@Makefile_config, "OMIT_NETWORK = y\n");
@@ -1662,11 +1815,6 @@ if ($platform eq "GNU") {
     # cut it out of the build until someone feels like fixing it.
     push(@Makefile_config, "BUILD_FIASCO = N\n");
 } elsif ($platform eq "FREEBSD") {
-    # Information from "alexus" <alexus@alexus.org> on 02.10.14 that
-    # /usr/local/lib is not in FreeBSD's linker's default search path,
-    # but that's where e.g. the JPEG library normally resides.  This is
-    # probably a FreeBSD bug.
-    push(@Makefile_config, "LDFLAGS = -L/usr/local/lib\n");
 } elsif ($platform eq "AMIGA") {
     push(@Makefile_config, "CFLAGS = -m68020-60 -ffast-math -mstackextend\n");
 } elsif ($platform eq "UNIXWARE") {
@@ -1680,8 +1828,7 @@ if ($platform eq "GNU") {
         push(@Makefile_config, "LD_SHLIB = -G\n");
         push(@Makefile_config, "SHLIB_CLIB =\n");
     } else {
-        push(@Makefile_config, "CC = gcc\n");
-        push(@Makefile_config, gnuCflags());
+        makeCompilerGcc(\@Makefile_config);
     }
     push(@Makefile_config, "CFLAGS_SHLIB = -fPIC\n");
     push(@Makefile_config, "LDSHLIB = -shared\n"); 
@@ -1770,6 +1917,13 @@ if (defined($netpbm_docurl)) {
     push(@Makefile_config, "NETPBM_DOCURL = $netpbm_docurl\n");
 }
 
+if ($inttypesHeaderFile ne '<inttypes.h>') {
+    push(@Makefile_config, "INTTYPES_H = $inttypesHeaderFile\n");
+}
+
+if ($haveInt64 ne 'Y') {
+    push(@Makefile_config, "HAVE_INT64 = $haveInt64\n");
+}
 
 #******************************************************************************
 #
