@@ -18,7 +18,11 @@
    describe the Lempel-Ziv base.
 */
 
+#define _BSD_SOURCE   /* Make sure strcasecmp() is in string.h */
+
 #include <string.h>
+#include <assert.h>
+
 #include "pnm.h"
 #include "shhopt.h"
 #include "mallocvar.h"
@@ -58,6 +62,7 @@ struct cmdline_info {
 };
 
 
+
 static void
 parse_command_line(int argc, char ** argv,
                    struct cmdline_info *cmdlineP) {
@@ -65,16 +70,19 @@ parse_command_line(int argc, char ** argv,
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc(100*sizeof(optEntry));
+    optEntry *option_def;
         /* Instructions to optParseOptions3 on how to parse our options.
          */
-    optStruct3 opt;
     
+    optStruct3 opt;
+
     unsigned int alphaSpec, imageSpec;
     const char * image;
 
     unsigned int option_def_index;
 
+    MALLOCARRAY_NOFAIL(option_def, 100);
+    
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3(0, "verbose",     OPT_FLAG, NULL, 
             &cmdlineP->verbose,         0);
@@ -138,11 +146,7 @@ struct gifScreen {
     unsigned int    Height;
     gifColorMap     ColorMap;
     unsigned int    ColorMapSize;
-        /* Base 2 logarithm of number of colors in the color map.
-           Note that the color map portion must be a power of two slots in 
-           size.  Each color takes 3 bytes (1 R, 1G, 1B), so 
-           e.g. 3 means 8 colors = 24 bytes of color map.
-        */
+        /* Number of colors in the color map. */
     unsigned int    ColorResolution;
     unsigned int    Background;
     unsigned int    AspectRatio;
@@ -188,6 +192,8 @@ readColorMap(FILE *ifP, const int colormapsize,
     int             i;
     unsigned char   rgb[3];
 
+    assert(colormapsize <= MAXCOLORMAPSIZE);
+
     *hasGrayP = FALSE;  /* initial assumption */
     *hasColorP = FALSE;  /* initial assumption */
 
@@ -268,6 +274,13 @@ getDataBlock(FILE *          const ifP,
 
 static void
 readThroughEod(FILE * const ifP) {
+/*----------------------------------------------------------------------------
+  Read the file 'ifP' through the next EOD marker.  An EOD marker is a
+  a zero length data block.
+
+  If there is no EOD marker between the present file position and EOF,
+  we read to EOF and issue warning message about a missing EOD marker.
+-----------------------------------------------------------------------------*/
     unsigned char buf[260];
     bool eod;
 
@@ -701,6 +714,8 @@ lzwInit(struct decompressor * const decompP,
     decompP->ifP = ifP;
     decompP->init_codeSize = init_codeSize;
 
+    assert(decompP->init_codeSize < sizeof(decompP->max_dataVal) * 8);
+
     decompP->max_dataVal = (1 << init_codeSize) - 1;
     decompP->clear_code = decompP->max_dataVal + 1;
     decompP->end_code = decompP->max_dataVal + 2;
@@ -824,7 +839,9 @@ lzwReadByte(struct decompressor * const decompP) {
     forgive the case that the "end" code is followed by EOF instead of
     an EOD marker (zero length DataBlock)).
 
-    Return -2 if there are no more bytes in the image.
+    Return -2 if there are no more bytes in the image.  In that case,
+    make sure the file is positioned immediately after the image (i.e.
+    after the EOD marker that marks the end of the image or EOF).
 
     Return -3 if we encounter errors in the LZW stream.
 -----------------------------------------------------------------------------*/
@@ -843,18 +860,14 @@ lzwReadByte(struct decompressor * const decompP) {
                 getCode(decompP->ifP, decompP->codeSize, FALSE);
             decompP->prevcode = decompP->firstcode;
         } while (decompP->firstcode == decompP->clear_code);
-        if (decompP->firstcode == decompP->end_code) {
-            if (!zeroDataBlock)
-                readThroughEod(decompP->ifP);
-            retval = -2;
-        } else
-            retval = decompP->firstcode;
+        retval = decompP->firstcode;
     } else {
         int code;
         code = getCode(decompP->ifP, decompP->codeSize, FALSE);
-        if (code < 0)
-            retval = code;
+        if (code == -1)
+            retval = -1;
         else {
+            assert(code >= 0);  /* -1 is only possible error return */
             if (code == decompP->clear_code) {
                 resetDecompressor(decompP);
                 retval = lzwReadByte( decompP );
@@ -877,20 +890,16 @@ lzwReadByte(struct decompressor * const decompP) {
 
 
 
-
-
-
-
 static void
-readImageData(FILE * const ifP, 
-              xel ** const xels, 
-              int    const len, 
-              int    const height, 
-              gifColorMap  cmap, 
+readImageData(FILE *       const ifP, 
+              xel **       const xels, 
+              unsigned int const cols,
+              unsigned int const rows,
+              gifColorMap        cmap, 
               unsigned int const cmapSize,
-              bool   const interlace,
-              int    const transparentIndex,
-              bit ** const alphabits) {
+              bool         const interlace,
+              int          const transparentIndex,
+              bit **       const alphabits) {
 
     unsigned char lzwMinCodeSize;      
     int v;
@@ -917,15 +926,13 @@ readImageData(FILE * const ifP,
 
     lzwInit(&decomp, ifP, lzwMinCodeSize);
 
-    if (verbose)
-        pm_message("reading %d by %d%s GIF image",
-                   len, height, interlace ? " interlaced" : "" );
-
     while ((v = lzwReadByte(&decomp)) >= 0 ) {
         if (v >= cmapSize)
             pm_error("Invalid color index %u in an image that has only "
                      "%u colors in the color map.", v, cmapSize);
 
+        assert(v < MAXCOLORMAPSIZE);
+
         PPM_ASSIGN(xels[ypos][xpos], 
                    cmap[CM_RED][v], cmap[CM_GRN][v], cmap[CM_BLU][v]);
 
@@ -934,7 +941,7 @@ readImageData(FILE * const ifP,
                 PBM_BLACK : PBM_WHITE;
 
         ++xpos;
-        if (xpos == len) {
+        if (xpos == cols) {
             xpos = 0;
             if (interlace) {
                 switch (pass) {
@@ -947,7 +954,7 @@ readImageData(FILE * const ifP,
                     ypos += 2; break;
                 }
                 
-                while (ypos >= height) {
+                if (ypos >= rows) {
                     ++pass;
                     switch (pass) {
                     case 1:
@@ -960,19 +967,19 @@ readImageData(FILE * const ifP,
                         goto fini;
                     }
                 }
-            } else {
+            } else
                 ++ypos;
-            }
         }
-        if (ypos >= height)
+        if (ypos >= rows)
             break;
     }
 fini:
     if (v == -3)
         pm_error("Error in GIF input stream");
     else {
-        if (lzwReadByte(&decomp)>=0)
-            pm_message("too much input data, ignoring extra...");
+        if (lzwReadByte(&decomp) >= 0)
+            pm_message("Extraneous data at end of image.  "
+                       "Skipped to end of image");
     }
     lzwTerm(&decomp);
 }
@@ -1068,8 +1075,8 @@ readGifHeader(FILE * const gifFile, struct gifScreen * const gifScreenP) {
     
     gifScreenP->Width           = LM_to_uint(buf[0],buf[1]);
     gifScreenP->Height          = LM_to_uint(buf[2],buf[3]);
-    gifScreenP->ColorMapSize    = 2<<(buf[4]&0x07);
-    gifScreenP->ColorResolution = (((buf[4]&0x70)>>3)+1);
+    gifScreenP->ColorMapSize    = 1 << ((buf[4] & 0x07) + 1);
+    gifScreenP->ColorResolution = (buf[4] & 0x70 >> 3) + 1;
     gifScreenP->Background      = buf[5];
     gifScreenP->AspectRatio     = buf[6];
 
@@ -1153,6 +1160,24 @@ readExtensions(FILE*          const ifP,
 
 
 
+static void
+reportImageInfo(unsigned int const cols,
+                unsigned int const rows,
+                bool         const useGlobalColormap,
+                unsigned int const localColorMapSize,
+                bool         const interlaced) {
+
+
+    pm_message("reading %u by %u%s GIF image",
+               cols, rows, interlaced ? " interlaced" : "" );
+    if (useGlobalColormap)
+        pm_message("  Uses global colormap");
+    else
+        pm_message("  Uses local colormap of %u colors", localColorMapSize);
+}
+
+
+
 static void
 convertImage(FILE *           const ifP, 
              bool             const skipIt, 
@@ -1174,19 +1199,23 @@ convertImage(FILE *           const ifP,
         /* The image alpha mask, in libpbm format.  NULL if we aren't computing
            an alpha mask.
         */
-    int cols, rows;  /* Dimensions of the image */
+    unsigned int cols, rows;  /* Dimensions of the image */
     gifColorMap localColorMap;
-    int localColorMapSize;
+    unsigned int localColorMapSize;
+    bool interlaced;
 
     if (! ReadOK(ifP,buf,9))
         pm_error("couldn't read left/top/width/height");
 
     useGlobalColormap = ! BitSet(buf[8], LOCALCOLORMAP);
-        
-    localColorMapSize = 1<<((buf[8]&0x07)+1);
+    localColorMapSize = 1u << ((buf[8] & 0x07) + 1);
+    cols = LM_to_uint(buf[4], buf[5]);
+    rows = LM_to_uint(buf[6], buf[7]);
+    interlaced = !!BitSet(buf[8], INTERLACE);
 
-    cols = LM_to_uint(buf[4],buf[5]);
-    rows = LM_to_uint(buf[6],buf[7]);
+    if (verbose)
+        reportImageInfo(cols, rows, useGlobalColormap, localColorMapSize,
+                        interlaced);
         
     xels = pnm_allocarray(cols, rows);
     if (!xels)
@@ -1206,18 +1235,16 @@ convertImage(FILE *           const ifP,
                      &hasGray, &hasColor);
         transparencyMessage(gif89.transparent, localColorMap);
         readImageData(ifP, xels, cols, rows, localColorMap, localColorMapSize,
-                      BitSet(buf[8], INTERLACE), gif89.transparent,
-                      alphabits);
+                      interlaced, gif89.transparent, alphabits);
         if (!skipIt) {
             writePnm(imageout_file, xels, cols, rows,
                      hasGray, hasColor);
         }
     } else {
         transparencyMessage(gif89.transparent, gifScreen.ColorMap);
-        readImageData(ifP, xels, cols, rows,
+        readImageData(ifP, xels, cols, rows, 
                       gifScreen.ColorMap, gifScreen.ColorMapSize,
-                      BitSet(buf[8], INTERLACE), gif89.transparent,
-                      alphabits);
+                      interlaced, gif89.transparent, alphabits);
         if (!skipIt) {
             writePnm(imageout_file, xels, cols, rows,
                      gifScreen.hasGray, gifScreen.hasColor);
@@ -1319,5 +1346,5 @@ main(int argc, char **argv) {
     if (alpha_file != NULL)
         pm_close( alpha_file );
 
-    exit(0);
+    return 0;
 }
