@@ -43,15 +43,15 @@ parseCommandLine(int argc, char ** argv,
    sometimes, one of these strings is actually just a suffix of an entry
    in argv!
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc(100*sizeof(optEntry));
-        /* Instructions to OptParseOptions3 on how to parse our options.
-         */
+    optEntry *option_def;
     optStruct3 opt;
 
     unsigned int debuglevelSpec;
 
     unsigned int option_def_index;
 
+    MALLOCARRAY_NOFAIL(option_def, 100);
+
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3(0, "verbose",      OPT_FLAG,   NULL, 
             &cmdlineP->verbose,   0);
@@ -293,9 +293,7 @@ createMatrices(struct pam * const outpamP, jas_matrix_t *** matrixP) {
     jas_matrix_t ** matrix; 
     unsigned int plane;
 
-    matrix = malloc(outpamP->depth * sizeof(jas_matrix_t *));
-    if (matrix == NULL)
-        pm_error("Out of memory");
+    MALLOCARRAY_NOFAIL(matrix, outpamP->depth);
 
     for (plane = 0; plane < outpamP->depth; ++plane) {
         matrix[plane] = jas_matrix_create(1, outpamP->width);
@@ -332,9 +330,7 @@ computeComponentMaxval(struct pam *  const outpamP,
     sample * jasperMaxval;
     unsigned int plane;
 
-    jasperMaxval = malloc(outpamP->depth * sizeof(int *));
-    if (jasperMaxval == NULL)
-        pm_error("Out of memory");
+    MALLOCARRAY(jasperMaxval, outpamP->depth);
 
     *singleMaxvalP = TRUE;  /* initial assumption */
     for (plane = 0; plane < outpamP->depth; ++plane) {
