@@ -15,6 +15,7 @@
 #include "pam.h"
 #include "shhopt.h"
 #include "nstring.h"
+#include "mallocvar.h"
 
 #include <jasper/jasper.h>
 #include "libjasper_compat.h"
@@ -68,7 +69,7 @@ parseCommandLine(int argc, char ** argv,
    sometimes, one of these strings is actually just a suffix of an entry
    in argv!
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc(100*sizeof(optEntry));
+    optEntry *option_def;
         /* Instructions to OptParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
@@ -86,6 +87,8 @@ parseCommandLine(int argc, char ** argv,
     char * modeOpt;
 
     unsigned int option_def_index;
+    
+    MALLOCARRAY_NOFAIL(option_def, 100);
 
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3(0, "imgareatlx",   OPT_UINT,   &cmdlineP->imgareatlx,
@@ -234,9 +237,7 @@ createJasperRaster(struct pam *  const inpamP,
     bool oddMaxval;
     sample jasperMaxval;
 
-    matrix = malloc(inpamP->depth * sizeof(jas_matrix_t *));
-    if (matrix == NULL)
-        pm_error("Out of memory");
+    MALLOCARRAY_NOFAIL(matrix, inpamP->depth);
 
     for (plane = 0; plane < inpamP->depth; ++plane) {
         matrix[plane] = jas_matrix_create(1, inpamP->width);
@@ -300,10 +301,7 @@ createJasperImage(struct pam *   const inpamP,
 	jas_image_cmptparm_t * cmptparms;
     unsigned int plane;
 
-    cmptparms = malloc(sizeof(jas_image_cmptparm_t) * inpamP->depth);
-
-    if (cmptparms == NULL)
-        pm_error("Unable to allocate storage for cmptparms");
+    MALLOCARRAY_NOFAIL(cmptparms, inpamP->depth);
 
     for (plane = 0; plane < inpamP->depth; ++plane) {
         cmptparms[plane].tlx = 0;
@@ -365,7 +363,8 @@ convertToJasperImage(struct pam *   const inpamP,
 
 static void
 writeJpc(jas_image_t *      const jasperP, 
-         struct cmdlineInfo const cmdline) {
+         struct cmdlineInfo const cmdline,
+         FILE *             const ofP) {
 
     jas_stream_t * outStreamP;
     const char * options;
@@ -373,11 +372,11 @@ writeJpc(jas_image_t *      const jasperP,
     const char * prgValue;
     char rateOpt[20+1];
 
-/* Note: ilyrrates is a hack because we're too lazy to properly parse
-   command line options to get the information and then compose
-   a proper input to Jasper.  So the user can screw things up by 
-   specifying garbage for the -ilyrrates option 
-*/
+    /* Note: ilyrrates is a hack because we're too lazy to properly parse
+       command line options to get the information and then compose
+       a proper input to Jasper.  So the user can screw things up by 
+       specifying garbage for the -ilyrrates option 
+    */
     if (strlen(cmdline.ilyrrates) > 0)
         asprintfN(&ilyrratesOpt, "ilyrrates=%s", cmdline.ilyrrates);
     else
@@ -444,11 +443,10 @@ writeJpc(jas_image_t *      const jasperP,
 
 
     /* Open the output image file (Standard Output) */
-    outStreamP = jas_stream_fdopen(fileno(stdout), "w+b");
+    outStreamP = jas_stream_fdopen(fileno(ofP), "w+b");
     if (outStreamP == NULL)
         pm_error("Unable to open output stream.  jas_stream_fdopen() "
-                 "of stdout failed");
-
+                 "failed");
 
     {
         int rc;
@@ -512,7 +510,7 @@ main(int argc, char **argv)
     
     convertToJasperImage(&inpam, &jasperP);
     
-    writeJpc(jasperP, cmdline);
+    writeJpc(jasperP, cmdline, stdout);
     
 	jas_image_destroy(jasperP);
 
@@ -520,5 +518,5 @@ main(int argc, char **argv)
 
     pm_close(stdout);
     
-    exit(0);
+    return 0;
 }
