@@ -51,7 +51,7 @@ typedef struct {    /* character-pixel mapping */
 
 static void
 parseCommandLine(int argc, char ** argv,
-                 struct cmdlineInfo *cmdlineP) {
+                 struct cmdlineInfo * const cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.  The outname array is in newly
@@ -379,7 +379,7 @@ freeCmap(cixel_map cmap[], unsigned int const ncolors) {
         if (cmapEntry.uilname)
             freeString(cmapEntry.uilname);
         if (cmapEntry.rgbname)
-            freeString(cmapEntry.rgbname);
+            freeString(cmapEntry.uilname);
     }
 }
 
@@ -433,5 +433,5 @@ main(int argc, char *argv[]) {
     free(cmdline.outname);
     freeCmap(cmap, ncolors);
 
-    exit(0);
+    return 0;
 }
