@@ -33,59 +33,8 @@
 #include "shhopt.h"
 #include "mallocvar.h"
 
-enum MediaSize {
-    eLetterPaper,
-    eLegalPaper,
-    eA4Paper,
-    eExecPaper,
-    eLedgerPaper,
-    eA3Paper,
-    eCOM10Envelope,
-    eMonarchEnvelope,
-    eC5Envelope,
-    eDLEnvelope,
-    eJB4Paper,
-    eJB5Paper,
-    etralala,
-    eB5Envelope,
-    eJPostcard,
-    eJDoublePostcard,
-    eA5Paper,
-    eA6Paper,
-    eJB6Paper
-};
+#include "pclxl.h"
 
-enum DuplexPageMode {
-    eDuplexHorizontalBinding,
-    eDuplexVerticalBinding
-}; 
-
-struct sPaperFormat {
-    const char *name;
-    int xl_nr;
-    float width,height; /* inch */
-} xlPaperFormats[] = {
-    {"letter",eLetterPaper,8.5,11.0},
-    {"legal",eLegalPaper,8.5,14},
-    {"a4",eA4Paper,8.26389,11.6944},
-    {"exec",eExecPaper,190/2.54,254/2.54},
-    {"ledger",eLedgerPaper,279/2.54,432/2.54},
-    {"a3",eA3Paper,11.6944,16.5278},
-    {"com10envelope",eCOM10Envelope,105/2.54,241/2.54},
-    {"monarchenvelope",eMonarchEnvelope,98/2.54,191/2.54},
-    {"c5envelope",eC5Envelope,162/2.54,229/2.54},
-    {"dlenvelope",eDLEnvelope,110/2.54,220/2.54},
-    {"jb4",eJB4Paper,257/2.54,364/2.54},
-    {"jb5",eJB5Paper,182/2.54,257/2.54},
-    {"b5envelope",eB5Envelope,176/2.54,250/2.54},
-    {"",-1,0,0},
-    {"jpostcard",eJPostcard,148/2.54,200/2.54},
-    {"jdoublepostcard",eJDoublePostcard,100/2.54,148/2.54},
-    {"a5",eA5Paper, 5.84722,8.26389},
-    {"a6",eA6Paper,4.125,5.84722},
-    {"jb6",eJB6Paper,0,0},
-    {NULL,0,0,0}
-};
 #define PAPERWIDTH(format) (xlPaperFormats[format].width)
 #define PAPERHEIGHT(format) (xlPaperFormats[format].height)
 
@@ -118,14 +67,15 @@ struct cmdlineInfo {
     float yoffs;
     unsigned int colorok;
     unsigned int verbose;
+    const char * jobsetup;  /* -jobsetup option value.  NULL if none */
+    unsigned int rendergray;
 };
 
 
 
 static void
-parseCommandLine ( int argc, char ** argv,
-                   struct cmdlineInfo *cmdlineP )
-{
+parseCommandLine(int argc, char ** argv,
+                 struct cmdlineInfo * const cmdlineP) {
 /*----------------------------------------------------------------------------
    parse program command line described in Unix standard form by argc
    and argv.  Return the information in the options as *cmdlineP.  
@@ -145,7 +95,7 @@ parseCommandLine ( int argc, char ** argv,
 
     char *formatOpt;
     char *duplexOpt;
-    unsigned int dpiSpec, xoffsSpec, yoffsSpec, formatSpec;
+    unsigned int dpiSpec, xoffsSpec, yoffsSpec, formatSpec, jobsetupSpec;
 
     option_def_index = 0;   /* incremented by OPTENT3 */
     OPTENT3(0, "dpi",       OPT_UINT,    &cmdlineP->dpi,
@@ -164,12 +114,16 @@ parseCommandLine ( int argc, char ** argv,
             &cmdlineP->colorok, 0);
     OPTENT3(0, "center",    OPT_FLAG,    NULL,                  
             &cmdlineP->center, 0 );
-    OPTENT3(0, "feeder",    OPT_UINT,    &cmdlineP->feeder,
+    OPTENT3(0, "feeder",    OPT_STRING,  &cmdlineP->feeder,
             &cmdlineP->feederSpec,        0);
-    OPTENT3(0, "outtray",   OPT_UINT,    &cmdlineP->outtray,
+    OPTENT3(0, "outtray",   OPT_STRING,  &cmdlineP->outtray,
             &cmdlineP->outtraySpec,       0);
     OPTENT3(0, "verbose",   OPT_FLAG,    NULL,                  
             &cmdlineP->verbose, 0);
+    OPTENT3(0, "jobsetup",  OPT_STRING,  &cmdlineP->jobsetup,
+            &jobsetupSpec,      0);
+    OPTENT3(0, "rendergray", OPT_FLAG,    NULL,
+            &cmdlineP->rendergray, 0 );
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
@@ -215,10 +169,12 @@ parseCommandLine ( int argc, char ** argv,
     } else
         cmdlineP->format = eLetterPaper;
 
+    if (!jobsetupSpec)
+        cmdlineP->jobsetup = NULL;
+
     if (argc-1 < 1) {
         MALLOCVAR(cmdlineP->sourceP);
         cmdlineP->sourceP->name = "-";
-        cmdlineP->sourceP->next = NULL;
     } else {
         int i;
         InputSource ** nextLinkP;
@@ -250,12 +206,6 @@ typedef struct XY_rle {
 } XY_rle;
 
 
-enum {
-    eSTART,
-    eRLE,
-    eLIT 
-} RLEstates;
-
 static void 
 XY_RLEreset(XY_rle *rle) 
 {   
@@ -378,331 +328,9 @@ XY_Write(int fd, const void *buf,int cnt) {
 }
 #define XY_Puts(fd,str)  XY_Write(fd,str,strlen(str))
 
-enum ArcDirection {
-    eClockWise,
-    eCounterClockWise
-};
-
-enum CharSubModeArray {
-    eNoSubstitution,
-    eVerticalSubstitution
-};
-
-enum ClipRegion {
-    eInterior,
-    eExterior
-};
-
-enum ColorDepth {
-    e1Bit,
-    e4Bit,
-    e8Bit
-};
-
-enum ColrMapping {
-    eDirectPixel,
-    eIndexedPixel
-};
-
-enum Colorspace {
-    eGray=1,
-    eRGB=2
-};
-
-enum Compression {
-    eNoCompression,
-    eRLECompression,
-    eJPEGCompression
-};
-
-enum DataOrg {
-    eBinaryHighByteFirst,
-    eBinaryLowByteFirst
-};
-
-enum DataSource {
-    eDefault
-};
-
-enum DataType {
-    eUByte,
-    eSByte,
-    eUint16,
-    eSint16
-};
-
-enum DitherMatrix {  
-    eDeviceBest
-}; 
-
-
-enum DuplexPageSide {
-    eFrontMediaSide,
-    eBackMediaSide
-};
-
-enum ErrorReport {
-    eNoReporting,   
-    eBackChannel,
-    eErrorPage,
-    eBackChAndErrPage,
-    eNWBackChannel,
-    eNWErrorPage,
-    eNWBackChAndErrPage
-};
-
-enum FillMode {
-    eNonZeroWinding,
-    eEvenOdd
-};
-
-enum LineCap {
-    eButtCap,
-    eRoundCap,
-    eSquareCap,
-    eTriangleCap
-};
-
-enum LineJoin {
-    eMiterJoin,
-    eRoundJoin,
-    eBevelJoin,
-    eNoJoin
-};
-
-enum Measure {
-    eInch,
-    eMillimeter,
-    eTenthsOfAMillimeter    
-};
-
-enum MediaSource {
-    eDefaultSource,
-    eAutoSelect,
-    eManualFeed,
-    eMultiPurposeTray,
-    eUpperCassette,
-    eEnvelopeTray,
-    eThirdCassette
-};
-
-enum MediaDestination {
-    eDefaultDestination,
-    eFaceDownBin,
-    eFaceUpBin,
-    eJobOffsetBin
-};
-
-enum Orientation {
-    ePortraitOrientation,
-    eLandscapeOrientation,
-    eReversePortrait,
-    eReverseLandscape   
-};
-
-enum PatternPersistence {
-    eTempPatter,
-    ePagePatter,
-    eSessionPattern
-};
-
-enum SimplexPageMode {
-    eSimplexFrontSide
-};
-
-enum TxMode {
-    eOpaque,
-    eTransparent
-};
-
-enum WritingMode{
-    eHorizontal,
-    eVertical
-};
-
-enum Attribute {
-    aPaletteDepth        =   2,
-    aColorSpace          =   3,
-    aNullBrush           =   4,
-    aNullPen             =   5,
-    aPaletteData         =   6,
-    aPatternSelectID     =   8,
-    aGrayLevel           =   9,
-    aRGBColor            =  11,
-    aPatternOrigin       =  12,
-    aNewDestinationSize  =  13,
-    aPrimaryArray        =  14,
-    aPrimaryDepth        =  15,
-    aDeviceMatrix        =  33,
-    aDitherMatrixDataType=  34,
-    aDitherOrigin        =  35,
-    aMediaDestination    =  36,
-    aMediaSize           =  37,
-    aMediaSource         =  38,
-    aMediaType           =  39,
-    aOrientation         =  40,
-    aPageAngle           =  41,
-    aPageOrigin          =  42,
-    aPageScale           =  43,
-    aROP3                =  44,
-    aTxMode              =  45,
-    aCustomMediaSize     =  47,
-    aCustomMediaUnits    =  48,
-    aPageCopies          =  49,
-    aDitherMatrixSize    =  50,
-    aDitherMatrixDepth   =  51,
-    aSimplexPageMode     =  52,
-    aDuplexPageMode      =  53,
-    aDuplexPageSide      =  54,
-    aArcDirection        =  65,
-    aBoundingBox         =  66,
-    aDashOffset          =  67,
-    aEllipseDimension    =  68,
-    aEndPoint            =  69,
-    aFillMode            =  70,
-    aLineCapStyle        =  71,
-    aLineJoinStyle       =  72,
-    aMiterLength         =  73,
-    aLineDashStyle       =  74,
-    aPenWidth            =  75,
-    aPoint               =  76,
-    aNumberOfPoints      =  77,
-    aSolidLine           =  78,
-    aStartPoint          =  79,
-    aPointType           =  80,
-    aControlPoint1       =  81,
-    aControlPoint2       =  82,
-    aClipRegion          =  83,
-    aClipMode            =  84,
-    aColorDepth          =  98,
-    aBlockHeight         =  99,
-    aColorMapping        = 100,
-    aCompressMode        = 101,
-    aDestinationBox      = 102,
-    aDestinationSize     = 103,
-    aPatternPersistence  = 104,
-    aPatternDefineID     = 105,
-    aSourceHeight        = 107,
-    aSourceWidth         = 108,
-    aStartLine           = 109,
-    aPadBytesMultiple    = 110,
-    aBlockByteLength     = 111,
-    aNumberOfScanLines   = 115,
-    aColorTreatment      = 120,
-    aCommentData         = 129,
-    aDataOrg             = 130,
-    aMeasure             = 134,
-    aSourceType          = 136,
-    aUnitsPerMeasure     = 137,
-    aStreamName          = 139,
-    aStreamDataLength    = 140,
-    aErrorReport         = 143,
-    aCharAngle           = 161,
-    aCharCode            = 162,
-    aCharDataSize        = 163,
-    aCharScale           = 164,
-    aCharShear           = 165,
-    aCharSize            = 166,
-    aFontHeaderLength    = 167,
-    aFontName            = 168,
-    aFontFormat          = 169,
-    aSymbolSet           = 170,
-    aTextData            = 171,
-    aCharSubModeArray    = 172,
-    aXSpacingData        = 175,
-    aYSpacingData        = 176,
-    aCharBoldValue       = 177
-};
-                                          
-enum Operator {
-    oBeginSession        = 0x41,
-    oEndSession          = 0x42,
-    oBeginPage           = 0x43,
-    oEndPage             = 0x44,
-    oComment             = 0x47,
-    oOpenDataSource      = 0x48,
-    oCloseDataSource     = 0x49,
-    oBeginFontHeader     = 0x4f,
-    oReadFontHeader      = 0x50,
-    oEndFontHeader       = 0x51,
-    oBeginChar           = 0x52,
-    oReadChar            = 0x53,
-    oEndChar             = 0x54,
-    oRemoveFont          = 0x55,
-    oSetCharAttributes   = 0x56,
-    oBeginStream         = 0x5b,
-    oReadStream          = 0x5c,
-    oEndStream           = 0x5d,
-    oExecStream          = 0x5e,
-    oRemoveStream        = 0x5f,
-    oPopGS               = 0x60,
-    oPushGS              = 0x61,
-    oSetClipReplace      = 0x62,
-    oSetBrushSource      = 0x63,
-    oSetCharAngle        = 0x64,
-    oSetCharScale        = 0x65,
-    oSetCharShear        = 0x66,
-    oSetClipIntersect    = 0x67,
-    oSetClipRectangle    = 0x68,
-    oSetClipToPage       = 0x69,
-    oSetColorSpace       = 0x6a,
-    oSetCursor           = 0x6b,
-    oSetCursorRel        = 0x6c,
-    oSetHalftoneMethod   = 0x6d,
-    oSetFillMode         = 0x6e,
-    oSetFont             = 0x6f,
-    oSetLineDash         = 0x70,
-    oSetLineCap          = 0x71,
-    oSetLineJoin         = 0x72,
-    oSetMiterLimit       = 0x73,
-    oSetPageDefaultCTM   = 0x74,
-    oSetPageOrigin       = 0x75,
-    oSetPageRotation     = 0x76,
-    oSetPageScale        = 0x77,
-    oSetPatternTxMode    = 0x78,
-    oSetPenSource        = 0x79,
-    oSetPenWidth         = 0x7a,
-    oSetROP              = 0x7b,
-    oSetSourceTxMode     = 0x7c,
-    oSetCharBoldValue    = 0x7d,
-    oSetClipMode         = 0x7f,
-    oSetPathToClip       = 0x80,
-    oSetCharSubMode      = 0x81,
-    oCloseSubPath        = 0x84,
-    oNewPath             = 0x85,
-    oPaintPath           = 0x86,
-    oArcPath             = 0x91,
-    oBezierPath          = 0x93,
-    oBezierRelPath       = 0x95,
-    oChord               = 0x96,
-    oChordPath           = 0x97,
-    oEllipse             = 0x98,
-    oEllipsePath         = 0x99,
-    oLinePath            = 0x9b,
-    oLineRelPath         = 0x9d,
-    oPie                 = 0x9e,
-    oPiePath             = 0x9f,
-    oRectangle           = 0xa0,
-    oRectanglePath       = 0xa1,
-    oRoundRectangle      = 0xa2,  
-    oRoundRectanglePath  = 0xa3,
-    oText                = 0xa8,
-    oTextPath            = 0xa9,
-    oBeginImage          = 0xb0,
-    oReadImage           = 0xb1,
-    oEndImage            = 0xb2,
-    oBeginRastPattern    = 0xb3,
-    oReadRastPattern     = 0xb4,
-    oEndRastPattern      = 0xb5,
-    oBeginScan           = 0xb6,
-    oEndScan             = 0xb8,
-    oScanLineRel         = 0xb9
-};
-
-
 typedef struct pcl_img {
-    unsigned char colorDepth;
-    unsigned char colorSpace;
+    enum ColorDepth colorDepth;
+    enum Colorspace colorSpace;
     int width,height;
     int linelen; /* bytes per line */
     unsigned char *data;
@@ -788,17 +416,32 @@ xl_sint32(int fd,signed int data ) {
     return out_sint32(fd,data);
 }
 #endif
+
+
+
 static int
-xl_ubyte_array(int fd,unsigned char *data,int len) {
-    int i;
+xl_ubyte_array(int                   const fd,
+               const unsigned char * const data,
+               int                   const len) {
+
+    unsigned int i;
     unsigned char head[4];
-    head[0]=0xc8;head[1]=0xc1;head[2]=len&0xff;head[3]=(len>>8)&0xff;
-    XY_Write(fd,head,4);
-    for(i=0;i<len;i++) {
-        out_ubyte(fd,data[i]);
-    }
+    
+    head[0] = 0xc8;
+    head[1] = 0xc1;
+    head[2] = len&0xff;
+    head[3] = (len>>8)&0xff;
+
+    XY_Write(fd, head, 4);
+
+    for (i = 0; i < len; ++i)
+        out_ubyte(fd, data[i]);
+
     return 0;
 }
+
+
+
 #if 0
 static int
 xl_uint16_array(int fd,unsigned short *data,int len) {
@@ -916,30 +559,213 @@ xl_dataLengthbytes(int fd,unsigned char dataLengthBytes) {
     return out_ubyte(fd,dataLengthBytes);
 }
 #endif 
-/*
- * ---------------------------------------------------- 
- * job_head/job_end:  
- *  switch printer to PCL-XL mode / reset 
- * ---------------------------------------------------- 
- */
-static int 
-job_head(int fd) {
-    XY_Puts(fd,"\033%-12345X@PJL ENTER LANGUAGE=PCLXL\n");  
-    XY_Puts(fd,") HP-PCL XL;1;1;Generated by Netpbm Pnmtopclxl\n");  
-    return 0;
-}
-static int 
-job_end(int fd) {
-    XY_Puts(fd,"\033%-12345X");  
-    return 0;
-}
+
+
 
 static void
-XL_WriteRleBlock(int const s,
-                 XL_pcl_img * const img,
-                 int const firstLine,
-                 int const nlines,
-                 XY_rle * const rle) {
+copyFile(const char * const sourceFileName,
+         int          const destFd) {
+
+    FILE * sourceFileP;
+
+    sourceFileP = pm_openr(sourceFileName);
+
+    while (!feof(sourceFileP)) {
+        char buffer[1024];
+        size_t bytesRead;
+        size_t totalBytesWritten;
+
+        bytesRead = fread(buffer, 1, sizeof(buffer), sourceFileP);
+
+        if (ferror(sourceFileP))
+            pm_error("Read from file failed.  errno=%d (%s)",
+                     errno, strerror(errno));
+        
+        totalBytesWritten = 0;
+        
+        while (totalBytesWritten < bytesRead) {
+            ssize_t rc;
+
+            rc = write(destFd, buffer, bytesRead);
+
+            if (rc < 0)
+                pm_error("Write to file failed. errno=%d (%s)",
+                         errno, strerror(errno));
+            else
+                totalBytesWritten += rc;
+        }
+    }
+    pm_close(sourceFileP);
+}
+
+
+
+static void
+jobHead(int          const outFd,
+        bool         const renderGray,
+        const char * const userJobSetupFileName) {
+/*----------------------------------------------------------------------------
+   Start a PJL job.
+
+   Switch printer to PCL-XL mode.  This is called "entering a printer
+   language" in PCL terms.  In particular, we enter the PCL-XL language,
+   as opposed to e.g. Postscript.
+-----------------------------------------------------------------------------*/
+    /* Reset */
+    XY_Puts(outFd,"\033%-12345X");  
+
+    if (userJobSetupFileName)
+        copyFile(userJobSetupFileName, outFd);
+
+    if (renderGray)
+        XY_Puts(outFd,"@PJL SET RENDERMODE=GRAYSCALE\n");  
+
+    XY_Puts(outFd,"@PJL ENTER LANGUAGE=PCLXL\n");  
+    XY_Puts(outFd,") HP-PCL XL;1;1;Generated by Netpbm Pnmtopclxl\n");  
+}
+
+
+
+static void
+jobEnd(int const outFd) {
+/*----------------------------------------------------------------------------
+   End a PJL job.
+
+   Reset printer to quiescent mode.  Exit the printer language.
+-----------------------------------------------------------------------------*/
+    XY_Puts(outFd,"\033%-12345X");  
+}
+
+
+
+static void
+beginPage(int                 const outFd,
+          bool                const doDuplex,
+          enum DuplexPageMode const duplex,
+          bool                const doMediaSource,
+          int                 const mediaSource,
+          bool                const doMediaDestination,
+          int                 const mediaDestination,
+          enum MediaSize      const format) {
+/*----------------------------------------------------------------------------
+   Emit a BeginPage printer command.
+-----------------------------------------------------------------------------*/
+    if (doDuplex) {
+        xl_ubyte(outFd, duplex);  xl_attr_ubyte(outFd, aDuplexPageMode);
+    }
+
+    if (doMediaSource) {
+        /* if not included same as last time in same session is selected */
+        xl_ubyte(outFd, mediaSource);  xl_attr_ubyte(outFd, aMediaSource);
+    }
+
+    if (doMediaDestination) {
+        xl_ubyte(outFd, mediaDestination);  
+        xl_attr_ubyte(outFd, aMediaDestination);
+    }
+
+    xl_ubyte(outFd, ePortraitOrientation); xl_attr_ubyte(outFd, aOrientation);
+    xl_ubyte(outFd, format); xl_attr_ubyte(outFd, aMediaSize);
+
+    XL_Operator(outFd, oBeginPage);
+}
+
+
+
+static void
+openDataSource(int             const outFd,
+               enum DataOrg    const dataOrg,
+               enum DataSource const dataSource) {
+
+    xl_ubyte(outFd, dataOrg); xl_attr_ubyte(outFd,aDataOrg);
+    xl_ubyte(outFd, dataSource); xl_attr_ubyte(outFd,aSourceType);
+    XL_Operator(outFd,oOpenDataSource);
+}
+
+
+
+static void
+setColorSpace(int                   const outFd,
+              enum Colorspace       const colorSpace,
+              const unsigned char * const palette,
+              unsigned int          const paletteSize,
+              enum ColorDepth       const paletteDepth) {
+/*----------------------------------------------------------------------------
+   Emit printer control to set the color space.
+
+   'palette' == NULL means no palette (raster contains colors, not indexes
+   into a palette).
+
+   'paletteSize' is the number of bytes in the palette (undefined if
+   'palette' is NULL)
+
+   'paletteDepth' is the color depth of the entries in the palette.
+   e8Bit means the palette contains 8 bit values.
+
+   The palette is a "direct color" palette: A separate table for each
+   color component (i.e. one table for grayscale; three tables for
+   RGB).  Each table is indexed by a value from the raster and yields
+   a byte of color component value.
+
+   The palette has to be the right size to fit the number of color
+   components and raster color depth (bits per component in the raster).
+
+   E.g. with raster color depth of 1 bit (e1Bit) and RGB color (eRGB),
+   'paletteSize' would have to be 6 -- a table for each of R, G, and
+   B, of two elements each.
+
+   It is not clear from the documentation what the situation is when
+   paletteDepth is not e8Bit.  Is each palette entry still a byte and only
+   some of the byte gets used?  Or are there multiple entries per byte?
+-----------------------------------------------------------------------------*/
+    xl_ubyte(outFd, colorSpace); xl_attr_ubyte(outFd, aColorSpace);   
+    if (palette) {
+        xl_ubyte(outFd, paletteDepth); 
+        xl_attr_ubyte(outFd, aPaletteDepth);   
+        xl_ubyte_array(outFd, palette, paletteSize); 
+        xl_attr_ubyte(outFd, aPaletteData);
+    }
+    XL_Operator(outFd, oSetColorSpace);
+}
+
+
+
+static void
+positionCursor(int            const outFd,
+               bool           const center,
+               float          const xoffs,
+               float          const yoffs,
+               int            const imageWidth,
+               int            const imageHeight,
+               int            const dpi,
+               enum MediaSize const format) {
+/*----------------------------------------------------------------------------
+   Emit printer control to position the cursor to start the page.
+-----------------------------------------------------------------------------*/
+    float xpos, ypos;
+
+    if (center) {
+        float const width  = 1.0 * imageWidth/dpi;  
+        float const height = 1.0 * imageHeight/dpi;    
+        xpos = (PAPERWIDTH(format) - width)/2;
+        ypos = (PAPERHEIGHT(format) - height)/2;
+    } else {
+        xpos = xoffs;
+        ypos = yoffs;
+    }
+    /* cursor positioning */
+    xl_sint16_xy(outFd, xpos * dpi, ypos * dpi); xl_attr_ubyte(outFd, aPoint);
+    XL_Operator(outFd, oSetCursor);
+}
+
+
+
+static void
+XL_WriteRleBlock(int                const outFd,
+                 const XL_pcl_img * const img,
+                 int                const firstLine,
+                 int                const nlines,
+                 XY_rle *           const rle) {
 
     const unsigned char pad[4] = {0,0,0,0};
     unsigned int const paddedLinelen = ((img->linelen+3)/4)*4;
@@ -955,8 +781,9 @@ XL_WriteRleBlock(int const s,
     rlelen=XY_RLEfinish(rle);
     if(rlelen<0) 
         pm_error("Error on Making rle");
-    xl_dataLength(s,rlelen); 
-    XY_Write(s,rle->fbuf,rlelen);
+
+    xl_dataLength(outFd,rlelen); 
+    XY_Write(outFd,rle->fbuf,rlelen);
 }
 
 
@@ -967,18 +794,18 @@ XL_WriteRleBlock(int const s,
  * ------------------------------------------------------------
  */
 static void 
-XL_WriteImage(int s,XL_pcl_img *img) 
+XL_WriteImage(int outFd,XL_pcl_img *img) 
 {
     int blockStartLine;
     XY_rle *rle;
 
-    xl_ubyte(s,eDirectPixel); xl_attr_ubyte(s,aColorMapping);
-    xl_ubyte(s,img->colorDepth); xl_attr_ubyte(s,aColorDepth);
-    xl_uint16(s,img->width); xl_attr_ubyte(s,aSourceWidth);  
-    xl_uint16(s,img->height); xl_attr_ubyte(s,aSourceHeight);    
-    xl_uint16_xy(s,img->width*1,img->height*1); 
-    xl_attr_ubyte(s,aDestinationSize);   
-    XL_Operator(s,oBeginImage);
+    xl_ubyte(outFd,eDirectPixel); xl_attr_ubyte(outFd,aColorMapping);
+    xl_ubyte(outFd,img->colorDepth); xl_attr_ubyte(outFd,aColorDepth);
+    xl_uint16(outFd,img->width); xl_attr_ubyte(outFd,aSourceWidth);  
+    xl_uint16(outFd,img->height); xl_attr_ubyte(outFd,aSourceHeight);    
+    xl_uint16_xy(outFd,img->width*1,img->height*1); 
+    xl_attr_ubyte(outFd,aDestinationSize);   
+    XL_Operator(outFd,oBeginImage);
 
     rle = XY_RLEnew(img->linelen*20);
     if(!rle) 
@@ -988,99 +815,103 @@ XL_WriteImage(int s,XL_pcl_img *img)
     blockStartLine=0;
     while (blockStartLine<img->height) {
         int const blockHeight=MIN(20,img->height-blockStartLine);
-        xl_uint16(s,blockStartLine); xl_attr_ubyte(s,aStartLine); 
-        xl_uint16(s,blockHeight); xl_attr_ubyte(s,aBlockHeight);
-        xl_ubyte(s,eRLECompression); xl_attr_ubyte(s,aCompressMode);
+        xl_uint16(outFd,blockStartLine); xl_attr_ubyte(outFd,aStartLine); 
+        xl_uint16(outFd,blockHeight); xl_attr_ubyte(outFd,aBlockHeight);
+        xl_ubyte(outFd,eRLECompression); xl_attr_ubyte(outFd,aCompressMode);
         /* In modern PCL-XL, we could use a PadBytesMultiple attribute
            here to avoid having to pad the data to a multiple of 4
            bytes.  But PCL-XL 1.1 didn't have PadBytesMultiple.
-        xl_ubyte(s,1); xl_attr_ubyte(s,aPadBytesMultiple); 
+        xl_ubyte(outFd,1); xl_attr_ubyte(outFd,aPadBytesMultiple); 
         */
-        XL_Operator(s,oReadImage);
-        XL_WriteRleBlock(s,img,blockStartLine,blockHeight,rle);
+        XL_Operator(outFd,oReadImage);
+        XL_WriteRleBlock(outFd,img,blockStartLine,blockHeight,rle);
         blockStartLine+=blockHeight;
     }
     XY_RLEdelete(rle);
-    XL_Operator(s,oEndImage);
+    XL_Operator(outFd,oEndImage);
 }
 
 
 
 static void
-print_page(int                 const s,
-           XL_pcl_img *        const img,
-           enum MediaSize      const format,
-           int                 const dpi,
-           bool                const center,
-           float               const xoffs,
-           float               const yoffs,
-           bool                const doDuplex,
-           enum DuplexPageMode const duplex,
-           bool                const doCopies,
-           unsigned int        const copies,
-           bool                const doMediaSource,
-           int                 const mediaSource,
-           bool                const doMediaDestination,
-           int                 const mediaDestination) 
-{
-    float xpos=xoffs,ypos=yoffs;
-    if(doDuplex) {
-        xl_ubyte(s,duplex);  xl_attr_ubyte(s,aDuplexPageMode);
-    }
-    if(doMediaSource) { 
-        /* if not included same as last time in same session is selected */
-        xl_ubyte(s,mediaSource);  xl_attr_ubyte(s,aMediaSource);
-    }
-    if(doMediaDestination) {
-        xl_ubyte(s,mediaDestination);  xl_attr_ubyte(s,aMediaDestination);
-    }
-    xl_ubyte(s,ePortraitOrientation); xl_attr_ubyte(s,aOrientation);
-    xl_ubyte(s,format); xl_attr_ubyte(s,aMediaSize);
-    XL_Operator(s,oBeginPage);
-        xl_ubyte(s,img->colorSpace); xl_attr_ubyte(s,aColorSpace);   
-        XL_Operator(s,oSetColorSpace);
-        
-        if(center) {
-            float width = 1.0*img->width/dpi;  
-            float height = 1.0*img->height/dpi;    
-            xpos = (PAPERWIDTH(format) - width)/2;
-            ypos = (PAPERHEIGHT(format) - height)/2;
-        } 
-        /* cursor positioning */
-        xl_sint16_xy(s,xpos*dpi,ypos*dpi); xl_attr_ubyte(s,aPoint);
-        XL_Operator(s,oSetCursor);
-        XL_WriteImage(s,img);   
-    if(doCopies) {
+endPage(int          const outFd,
+        bool         const doCopies,
+        unsigned int const copies) {
+/*----------------------------------------------------------------------------
+   Emit an EndPage printer command.
+-----------------------------------------------------------------------------*/
+    if (doCopies) {
         /* wrong in example in PCL-XL manual. Type is uint16 ! */
-        xl_uint16(s,copies); xl_attr_ubyte(s,aPageCopies);
+        xl_uint16(outFd, copies); xl_attr_ubyte(outFd, aPageCopies);
     }
-    XL_Operator(s,oEndPage);
+    XL_Operator(outFd, oEndPage);
 }
 
 
-/*
- * ---------------------------------------------- 
- * StartSession:  
- *  Prepare for print
- * ---------------------------------------------- 
- */
+
 static void
-start_session(int s,int dpi)   { 
-    xl_uint16_xy(s,dpi,dpi); 
-    xl_attr_ubyte(s,aUnitsPerMeasure); 
-    xl_ubyte(s,eInch); 
-    xl_attr_ubyte(s,aMeasure);
-    /* xl_ubyte(s,eNoReporting); xl_attr_ubyte(s,aErrorReport); */
-    xl_ubyte(s,eBackChAndErrPage); 
-    xl_attr_ubyte(s,aErrorReport);
-    XL_Operator(s,oBeginSession);
-}        
+printPage(int                 const outFd,
+          XL_pcl_img *        const img,
+          enum MediaSize      const format,
+          int                 const dpi,
+          bool                const center,
+          float               const xoffs,
+          float               const yoffs,
+          bool                const doDuplex,
+          enum DuplexPageMode const duplex,
+          bool                const doCopies,
+          unsigned int        const copies,
+          bool                const doMediaSource,
+          int                 const mediaSource,
+          bool                const doMediaDestination,
+          int                 const mediaDestination) {
+
+
+    beginPage(outFd, doDuplex, duplex, doMediaSource, mediaSource,
+              doMediaDestination, mediaDestination, format);
+
+    /* Before Netpbm 10.27 (March 2005), we always set up a two-byte 8
+       bit deep palette: {0, 255}.  I don't know why, because this
+       works only for e1Bit color depth an eGray color space, and in
+       that case does the same thing as having no palette at all.  But
+       in other cases, it doesn't work.  E.g. with eRGB, e8Bit, we got
+       an IllegalArraySize error from the printer on the SetColorSpace
+       command.
+
+       So we don't use a palette at all now.  
+    */
+    setColorSpace(outFd, img->colorSpace, NULL, 0, 0);
+
+    positionCursor(outFd, center, xoffs, yoffs, 
+                   img->width, img->height, dpi, format);
+
+    XL_WriteImage(outFd,img);   
+
+    endPage(outFd, doCopies, copies);
+}
 
 
 
+static void
+beginSession(int              const outFd,
+             unsigned int     const xdpi,
+             unsigned int     const ydpi,
+             enum Measure     const measure,
+             bool             const noReporting,
+             enum ErrorReport const errorReport) {
+
+    xl_uint16_xy(outFd, xdpi, ydpi); xl_attr_ubyte(outFd, aUnitsPerMeasure); 
+    xl_ubyte(outFd, measure);  xl_attr_ubyte(outFd, aMeasure);
+    /* xl_ubyte(outFd,eNoReporting); xl_attr_ubyte(outFd,aErrorReport); */
+    xl_ubyte(outFd,errorReport); xl_attr_ubyte(outFd,aErrorReport);
+    XL_Operator(outFd,oBeginSession);
+}
+
+
+             
 static void 
-end_session(int s) {
-    XL_Operator(s,oEndSession);
+endSession(int outFd) {
+    XL_Operator(outFd,oEndSession);
 }
 
 
@@ -1139,28 +970,32 @@ pnm_to_pclimg_wholebytes(struct pam * pamP,XL_pcl_img *img) {
     
     tuple * tuplerow;
         
-    if (pamP->depth==1)
-        img->colorSpace=eGray;
+    if (pamP->depth <  3)
+        img->colorSpace = eGray;
     else
-        img->colorSpace=eRGB;
+        img->colorSpace = eRGB;
 
-    img->colorDepth=e8Bit;
-    img->height=pamP->height;
-    img->width=pamP->width;
-    img->linelen=pamP->width*pamP->depth;
+    img->colorDepth = e8Bit;
+    img->height     = pamP->height;
+    img->width      = pamP->width;
+    img->linelen    = pamP->width * pamP->depth;
+
+    if (UINT_MAX / pamP->height / pamP->width < pamP->depth)
+        pm_error("Image to big to process");
+    else
+        img->data = malloc(pamP->width * pamP->height * pamP->depth);
 
-    img->data=(unsigned char *)malloc(pamP->width*pamP->height*pamP->depth);
     tuplerow=pnm_allocpamrow(pamP);
     {
-        int row;
+        unsigned int row;
         unsigned int pcl_cursor;
 
         pcl_cursor=0;
         for (row=0;row<pamP->height;++row) {
-            int col;
+            unsigned int col;
             pnm_readpamrow(pamP, tuplerow);
             for (col=0;col<pamP->width;++col) {
-                int plane;
+                unsigned int plane;
                 for (plane=0;plane<pamP->depth;++plane)
                     img->data[pcl_cursor++] = 
                         pnm_scalesample(tuplerow[col][plane], 
@@ -1193,43 +1028,38 @@ pnm_to_pclimg(struct pam *        const pamP,
 
 
 
-/*
- * -------------------------------------------------------------------
- * print_pages
- *  Loop over all input files, and each file, all images.
- * -------------------------------------------------------------------
- */
 
-static int
-print_pages(int                 const s,
-            InputSource *       const firstSourceP,
-            enum MediaSize      const format,
-            int                 const dpi,
-            bool                const center,
-            float               const xoffs,
-            float               const yoffs,
-            bool                const doDuplex,
-            enum DuplexPageMode const duplex,
-            bool                const doCopies,
-            unsigned int        const copies,
-            bool                const doMediaSource,
-            int                 const mediaSource,
-            bool                const doMediaDestination,
-            int                 const mediaDestination,
-            bool                const colorok) {
-
-    InputSource *sourceP;
+static void
+printPages(int                 const outFd,
+           InputSource *       const firstSourceP,
+           enum MediaSize      const format,
+           int                 const dpi,
+           bool                const center,
+           float               const xoffs,
+           float               const yoffs,
+           bool                const doDuplex,
+           enum DuplexPageMode const duplex,
+           bool                const doCopies,
+           unsigned int        const copies,
+           bool                const doMediaSource,
+           int                 const mediaSource,
+           bool                const doMediaDestination,
+           int                 const mediaDestination,
+           bool                const colorok) {
+/*----------------------------------------------------------------------------
+  Loop over all input files, and each file, all images.
+-----------------------------------------------------------------------------*/
+    InputSource * sourceP;
     unsigned int sourceNum;
 
     sourceP = firstSourceP;    
-    xl_ubyte(s, eBinaryLowByteFirst); xl_attr_ubyte(s,aDataOrg);
-    xl_ubyte(s, eDefaultSource); xl_attr_ubyte(s,aSourceType);
-    XL_Operator(s,oOpenDataSource);
+
+    openDataSource(outFd, eBinaryLowByteFirst, eDefaultSource);
 
     sourceNum = 0;   /* initial value */
 
-    while(sourceP) {
-        FILE *in_file;
+    while (sourceP) {
+        FILE * in_file;
         struct pam pam;
         bool eof;
         unsigned int pageNum;
@@ -1253,10 +1083,10 @@ print_pages(int                 const s,
                 
                 pnm_to_pclimg(&pam, &pcl_img, colorok);
 
-                print_page(s, &pcl_img,
-                           format, dpi, center, xoffs, yoffs, doDuplex,duplex,
-                           doCopies, copies, doMediaSource, mediaSource,
-                           doMediaDestination, mediaDestination);
+                printPage(outFd, &pcl_img,
+                          format, dpi, center, xoffs, yoffs, doDuplex, duplex,
+                          doCopies, copies, doMediaSource, mediaSource,
+                          doMediaDestination, mediaDestination);
 
                 free(pcl_img.data);
             }
@@ -1264,8 +1094,7 @@ print_pages(int                 const s,
         pm_close(in_file);
         sourceP = sourceP->next; 
     }
-    XL_Operator(s,oCloseDataSource);
-    return 0;
+    XL_Operator(outFd,oCloseDataSource);
 }
 
 
@@ -1286,29 +1115,41 @@ freeSource(InputSource * const firstSourceP) {
 
 
 int
-main(int argc,char *argv[]) {
-    struct cmdlineInfo cmdline;
-    int const out_fd = 1;
+main(int argc, char *argv[]) {
 
+    int const outFd = STDOUT_FILENO;
+
+    struct cmdlineInfo cmdline;
+    
+    /* In case you're wondering why we do direct file descriptor I/O
+       instead of stream (FILE *), it's because Jochen originally 
+       wrote this code for an embedded system with diet-libc.  Without
+       the stream library, the statically linked binary was only about
+       5K big.
+    */
     pnm_init(&argc, argv);
 
     parseCommandLine(argc, argv, &cmdline);
 
-    job_head(out_fd);
-    start_session(out_fd,cmdline.dpi);
-    print_pages(out_fd,cmdline.sourceP,
-                cmdline.format, cmdline.dpi, cmdline.center,
-                cmdline.xoffs, cmdline.yoffs,
-                cmdline.duplexSpec, cmdline.duplex,
-                cmdline.copiesSpec, cmdline.copies,
-                cmdline.feederSpec, cmdline.feeder,
-                cmdline.outtraySpec, cmdline.outtray,
-                cmdline.colorok
+    jobHead(outFd, cmdline.rendergray, cmdline.jobsetup);
+
+    beginSession(outFd, cmdline.dpi, cmdline.dpi, eInch, 
+                 FALSE, eBackChAndErrPage);
+
+    printPages(outFd,cmdline.sourceP,
+               cmdline.format, cmdline.dpi, cmdline.center,
+               cmdline.xoffs, cmdline.yoffs,
+               cmdline.duplexSpec, cmdline.duplex,
+               cmdline.copiesSpec, cmdline.copies,
+               cmdline.feederSpec, cmdline.feeder,
+               cmdline.outtraySpec, cmdline.outtray,
+               cmdline.colorok
         );
-    end_session(out_fd);
-    job_end(out_fd);
+    endSession(outFd);
+
+    jobEnd(outFd);
 
     freeSource(cmdline.sourceP);
 
-    exit(0);
+    return 0;
 }
