@@ -17,6 +17,10 @@
 ** implied warranty.
 */
 
+/* This Netpbm version of Pnmtopng was derived from the independently
+   distributed program of the same name, Version 2.37.6 (21 July 2001).
+*/
+
 /* A performance note: This program reads one row at a time because
    the whole image won't fit in memory always.  When you realize that
    in a Netpbm xel array a one bit pixel can take 96 bits of memory,
@@ -57,11 +61,11 @@
 #include <string.h>	/* strcat() */
 #include <limits.h>
 #include <png.h>	/* includes zlib.h and setjmp.h */
-#define VERSION "2.37.6 (21 July 2001) +netpbm"
 #include "pnm.h"
 #include "pngtxt.h"
 #include "mallocvar.h"
 #include "nstring.h"
+#include "version.h"
 
 struct zlib_compression {
     /* These are parameters that describe a form of zlib compression. 
@@ -246,8 +250,8 @@ addtocoloralphahash(coloralphahash_table const caht,
 
 static int
 lookupColorAlpha(coloralphahash_table const caht,
-                 pixel const * const colorP,
-                 gray * const alphaP) {
+                 const pixel *        const colorP,
+                 const gray *         const alphaP) {
 
     int hash;
     cahitem * p;
@@ -262,14 +266,10 @@ lookupColorAlpha(coloralphahash_table const caht,
 
 
 
-#ifdef __STDC__
-static void pnmtopng_error_handler (png_structp png_ptr, png_const_charp msg)
-#else
-static void pnmtopng_error_handler (png_ptr, msg)
-png_structp png_ptr;
-png_const_charp msg;
-#endif
-{
+static void
+pnmtopng_error_handler(png_structp     const png_ptr,
+                       png_const_charp const msg) {
+
   jmpbuf_wrapper  *jmpbuf_ptr;
 
   /* this function, aside from the extra step of retrieving the "error
@@ -407,7 +407,7 @@ meaningful_bits_pgm(FILE *         const ifp,
             pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
             for (x = 0 ; x < cols && mayscale ; x++) {
                 if ((PNM_GET1 (xelrow[x]) & 0xf) * 0x11 
-                    != (PNM_GET1 (xelrow[x]) & 0xff))
+                    != PNM_GET1 (xelrow[x]))
                     mayscale = FALSE;
             }
         }
@@ -421,8 +421,7 @@ meaningful_bits_pgm(FILE *         const ifp,
         for (y = 0 ; y < rows && mayscale ; y++) {
             pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
             for (x = 0 ; x < cols && mayscale ; x++) {
-	         if ((PNM_GET1 (xelrow[x])&3) * 0x5 != 
-		     (PNM_GET1 (xelrow[x]) & 0xf))
+                if ((PNM_GET1 (xelrow[x])&3) * 0x5 != PNM_GET1 (xelrow[x]))
                     mayscale = FALSE;
             }
         }
@@ -437,8 +436,7 @@ meaningful_bits_pgm(FILE *         const ifp,
         for (y = 0 ; y < rows && mayscale ; y++) {
             pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
             for (x = 0 ; x < cols && mayscale ; x++) {
-	         if ((PNM_GET1 (xelrow[x])&1) * 0x3 !=
-		     (PNM_GET1 (xelrow[x]) & 3))
+                if ((PNM_GET1 (xelrow[x])&1) * 0x3 != PNM_GET1 (xelrow[x]))
                     mayscale = FALSE;
             }
         }
@@ -498,33 +496,102 @@ meaningful_bits_ppm(FILE *         const ifp,
 
 
 static void
-alpha_trans(FILE *     const ifp, 
-            pm_filepos const imagepos, 
-            int        const cols, 
-            int        const rows, 
-            xelval     const maxval, int const format, 
-            gray **    const alpha_mask, gray alpha_maxval,
-            bool *     const alpha_can_be_transparency_indexP, 
-            pixel*     const alpha_transcolorP) {
+tryTransparentColor(FILE *     const ifp, 
+                    pm_filepos const imagepos, 
+                    int        const cols, 
+                    int        const rows, 
+                    xelval     const maxval,
+                    int        const format, 
+                    gray **    const alphaMask,
+                    gray       const alphaMaxval,
+                    pixel      const transcolor,
+                    bool *     const singleColorIsTransP) {
+
+    int const pnm_type = PNM_FORMAT_TYPE(format);
+
+    xel * xelrow;
+    bool singleColorIsTrans;
+        /* So far, it looks like a single color is uniquely transparent */
+    int row;
+
+    xelrow = pnm_allocrow(cols);
+
+    pm_seek2(ifp, &imagepos, sizeof(imagepos));
+
+    singleColorIsTrans = TRUE;  /* initial assumption */
+        
+    for (row = 0; row < rows && singleColorIsTrans; ++row) {
+        int col;
+        pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
+        for (col = 0 ; col < cols && singleColorIsTrans; ++col) {
+            if (alphaMask[row][col] == 0) { /* transparent */
+                /* If we have a second transparent color, we're
+                   disqualified
+                */
+                if (pnm_type == PPM_TYPE) {
+                    if (!PPM_EQUAL(xelrow[col], transcolor))
+                        singleColorIsTrans = FALSE;
+                } else {
+                    if (PNM_GET1(xelrow[col]) != PNM_GET1(transcolor))
+                        singleColorIsTrans = FALSE;
+                }
+            } else if (alphaMask[row][col] != alphaMaxval) {
+                /* Here's an area of the mask that is translucent.  That
+                   disqualified us.
+                */
+                singleColorIsTrans = FALSE;
+            } else {
+                /* Here's an area of the mask that is opaque.  If it's
+                   the same color as our candidate transparent color,
+                   that disqualifies us.
+                */
+                if (pnm_type == PPM_TYPE) {
+                    if (PPM_EQUAL(xelrow[col], transcolor))
+                        singleColorIsTrans = FALSE;
+                } else {
+                    if (PNM_GET1(xelrow[col]) == PNM_GET1(transcolor))
+                        singleColorIsTrans = FALSE;
+                }
+            }
+        }
+    }  
+    pnm_freerow(xelrow);
+}
+
+
+
+static void
+analyzeAlpha(FILE *     const ifp, 
+             pm_filepos const imagepos, 
+             int        const cols, 
+             int        const rows, 
+             xelval     const maxval,
+             int        const format, 
+             gray **    const alphaMask,
+             gray       const alphaMaxval,
+             bool *     const allOpaqueP,
+             bool *     const singleColorIsTransP, 
+             pixel*     const alphaTranscolorP) {
 /*----------------------------------------------------------------------------
-  Check if the alpha mask can be represented by a single transparency
-  value (i.e. all colors fully opaque except one fully transparent;
-  the transparent color may not also occur as fully opaque.
-  we have to do this before any scaling occurs, since alpha is only
+  Get information about the alpha mask, in combination with the masked
+  image, that Caller can use to choose the most efficient way to
+  represent the information in the alpha mask in a PNG.  Simply
+  putting the alpha mask in the PNG is a last resort.  But if the mask
+  says all opaque, we can simply omit any mention of transparency
+  instead -- default is opaque.  And if the mask makes all the pixels
+  of a certain color fully transparent and every other pixel opaque,
+  we can simply identify that color in the PNG.
+
+  We have to do this before any scaling occurs, since alpha is only
   possible with 8 and 16-bit.
 -----------------------------------------------------------------------------*/
     xel * xelrow;
-    bool retval;
-        /* Our eventual return value -- alpha mask can be represented by
-           a simple transparency index.
-        */
-    bool found_transparent_pixel;
+    bool foundTransparentPixel;
         /* We found a pixel in the image where the alpha mask says it is
            transparent.
         */
     pixel transcolor;
         /* Color of the transparent pixel mentioned above. */
-    int const pnm_type = PNM_FORMAT_TYPE(format);
     
     xelrow = pnm_allocrow(cols);
 
@@ -533,66 +600,32 @@ alpha_trans(FILE *     const ifp,
         /* Find a candidate transparent color -- the color of any pixel in the
            image that the alpha mask says should be transparent.
         */
-        found_transparent_pixel = FALSE;  /* initial assumption */
+        foundTransparentPixel = FALSE;  /* initial assumption */
         pm_seek2(ifp, &imagepos, sizeof(imagepos));
-        for (row = 0 ; row < rows && !found_transparent_pixel ; ++row) {
+        for (row = 0 ; row < rows && !foundTransparentPixel ; ++row) {
             int col;
             pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
-            for (col = 0 ; col < cols && !found_transparent_pixel; ++col) {
-                if (alpha_mask[row][col] == 0) {
-                    found_transparent_pixel = TRUE;
-                    transcolor = pnm_xeltopixel(xelrow[col], format);
+            for (col = 0; col < cols && !foundTransparentPixel; ++col) {
+                if (alphaMask[row][col] == 0) {
+                    foundTransparentPixel = TRUE;
+                    transcolor = xeltopixel(xelrow[col]);
                 }
             }
         }
     }
 
-    if (found_transparent_pixel) {
-        int row;
-        pm_seek2(ifp, &imagepos, sizeof(imagepos));
-        retval = TRUE;  /* initial assumption */
-        
-        for (row = 0 ; row < rows && retval == TRUE; ++row) {
-            int col;
-            pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
-            for (col = 0 ; col < cols && retval == TRUE; ++col) {
-                if (alpha_mask[row][col] == 0) { /* transparent */
-                    /* If we have a second transparent color, we're
-                       disqualified
-                    */
-                    if (pnm_type == PPM_TYPE) {
-                        if (!PPM_EQUAL (xelrow[col], transcolor))
-                            retval = FALSE;
-                    } else {
-                        if (PNM_GET1 (xelrow[col]) != PNM_GET1 (transcolor))
-                            retval = FALSE;
-                    }
-                } else if (alpha_mask[row][col] != alpha_maxval) {
-                    /* Here's an area of the mask that is translucent.  That
-                       disqualified us.
-                    */
-                    retval = FALSE;
-                } else 
-                    /* Here's an area of the mask that is opaque.  If it's
-                       the same color as our candidate transparent color,
-                       that disqualifies us.
-                    */
-                    if (pnm_type == PPM_TYPE) {
-                        if (PPM_EQUAL (xelrow[col], transcolor))
-                            retval = FALSE;
-                    } else {
-                        if (PNM_GET1 (xelrow[col]) == PNM_GET1 (transcolor))
-                            retval = FALSE;
-                    }
-            }
-        }  
-    } else
-        retval = FALSE;
-
     pnm_freerow(xelrow);
 
-    *alpha_can_be_transparency_indexP = retval;
-    *alpha_transcolorP = transcolor;
+    if (foundTransparentPixel) {
+        *allOpaqueP = FALSE;
+        tryTransparentColor(ifp, imagepos, cols, rows, maxval, format,
+                            alphaMask, alphaMaxval, transcolor,
+                            singleColorIsTransP);
+        *alphaTranscolorP = transcolor;
+    } else {
+        *allOpaqueP   = TRUE;
+        *singleColorIsTransP = FALSE;
+    }
 }
 
 
@@ -780,7 +813,7 @@ computeUnsortedAlphaPalette(FILE *           const ifP,
 
     cht = ppm_colorhisttocolorhash (chv, colors);
 
-    for (color_index = 0 ; color_index < colors ; ++color_index) {
+    for (color_index = 0 ; color_index < colors + 1 ; ++color_index) {
         /* TODO: It sure would be nice if we didn't have to allocate
            256 words here for what is normally only 0 or 1 different
            alpha values!  Maybe we should do some sophisticated reallocation.
@@ -830,7 +863,6 @@ sortAlphaPalette(gray *alphas_of_color[],
                  unsigned int alphas_first_index[],
                  unsigned int alphas_of_color_cnt[],
                  unsigned int const colors,
-                 gray         const alphaMaxval,
                  unsigned int mapping[],
                  unsigned int * const transSizeP) {
 /*----------------------------------------------------------------------------
@@ -863,7 +895,7 @@ sortAlphaPalette(gray *alphas_of_color[],
         for (j = 0; j < alphas_of_color_cnt[colorIndex]; ++j) {
             unsigned int const paletteIndex = 
                 alphas_first_index[colorIndex] + j;
-            if (alphas_of_color[colorIndex][j] == alphaMaxval)
+            if (alphas_of_color[colorIndex][j] == PALETTEOPAQUE)
                 mapping[paletteIndex] = top_idx--;
                 else
                     mapping[paletteIndex] = bot_idx++;
@@ -888,7 +920,6 @@ compute_alpha_palette(FILE *         const ifP,
                       int            const format,
                       pm_filepos     const imagepos,
                       gray **        const alpha_mask,
-                      gray           const alphaMaxval,
                       pixel                palette_pnm[],
                       gray                 trans_pnm[],
                       unsigned int * const paletteSizeP,
@@ -948,7 +979,7 @@ compute_alpha_palette(FILE *         const ifP,
 
         /* Make the opaque palette entries last */
         sortAlphaPalette(alphas_of_color, alphas_first_index,
-                         alphas_of_color_cnt, colors, alphaMaxval,
+                         alphas_of_color_cnt, colors,
                          mapping, transSizeP);
 
         {
@@ -968,9 +999,10 @@ compute_alpha_palette(FILE *         const ifP,
     }
     { 
         unsigned int colorIndex;
-        for (colorIndex = 0; colorIndex < colors; ++colorIndex)
+        for (colorIndex = 0; colorIndex < colors + 1; ++colorIndex)
             free(alphas_of_color[colorIndex]);
     }
+pm_message("returning alpha palette of size %u", *transSizeP);
 } 
 
 
@@ -1166,14 +1198,19 @@ tryAlphaPalette(FILE *         const ifP,
                 int            const format,
                 pm_filepos     const imagepos,
                 gray **        const alpha_mask,
-                gray           const alphaMaxval,
                 FILE *         const pfP,
                 pixel *        const palette_pnm,
                 unsigned int * const paletteSizeP,
                 gray *         const trans_pnm,
                 unsigned int * const transSizeP,
                 const char **  const impossibleReasonP) {
-    
+/*----------------------------------------------------------------------------
+   Try to make an alpha palette as 'trans_pnm', size *transSizeP.
+
+   If it's impossible, return as *impossibleReasonP newly malloced storage
+   containing text that tells why.  But if we succeed, return
+   *impossibleReasonP == NULL.
+-----------------------------------------------------------------------------*/
     bool tooBig;
     if (pfP)
         pm_error("This program is not capable of generating "
@@ -1181,8 +1218,7 @@ tryAlphaPalette(FILE *         const ifP,
                  "the palette with -palette.");
 
     compute_alpha_palette(ifP, cols, rows, maxval, format, 
-                          imagepos,  alpha_mask, alphaMaxval,
-                          palette_pnm, trans_pnm, 
+                          imagepos,  alpha_mask, palette_pnm, trans_pnm, 
                           paletteSizeP, transSizeP, &tooBig);
     if (tooBig) {
         asprintfN(impossibleReasonP,
@@ -1291,7 +1327,6 @@ computeColorMap(FILE *         const ifP,
                 int            const background,
                 pixel          const backcolor,
                 gray **        const alpha_mask,
-                gray           const alphaMaxval,
                 unsigned int   const pnm_meaningful_bits,
                 /* Outputs */
                 pixel *        const palette_pnm,
@@ -1360,8 +1395,7 @@ computeColorMap(FILE *         const ifP,
                     unsigned int transSize;
                     if (alpha)
                         tryAlphaPalette(ifP, cols, rows, maxval, format,
-                                        imagepos, alpha_mask, alphaMaxval,
-                                        pfP,
+                                        imagepos, alpha_mask, pfP,
                                         palette_pnm, &paletteSize, 
                                         trans_pnm, &transSize,
                                         noColormapReasonP);
@@ -1373,7 +1407,7 @@ computeColorMap(FILE *         const ifP,
                                                  palette_pnm, &paletteSize, 
                                                  trans_pnm, &transSize);
     
-                        if (transparent)
+                        if (transparent != -1)
                             makeOneColorTransparentInPalette(
                                 transcolor, transexact, 
                                 palette_pnm, paletteSize, trans_pnm, 
@@ -1514,6 +1548,159 @@ createPngPalette(pixel              palette_pnm[],
 }
 
 
+
+static void
+setCompressionSize(png_struct * const png_ptr,
+                   int const    buffer_size) {
+
+#if PNG_LIBPNG_VER >= 10009
+    png_set_compression_buffer_size(png_ptr, buffer_size);
+#else
+    pm_error("Your PNG library cannot set the compression buffer size.  "
+             "You need at least Version 1.0.9 of Libpng; you have Version %s",
+             PNG_LIBPNG_VER_STRING);
+#endif
+}
+
+
+
+static void
+setZlibCompression(png_struct *            const png_ptr,
+                   struct zlib_compression const zlib_compression) {
+
+    if (zlib_compression.level >= 0)
+        png_set_compression_level(png_ptr, zlib_compression.level);
+
+    if (zlib_compression.mem_level >= 0)
+        png_set_compression_mem_level(png_ptr, zlib_compression.mem_level);
+
+    if (zlib_compression.strategy >= 0)
+        png_set_compression_strategy(png_ptr, zlib_compression.strategy);
+
+    if (zlib_compression.window_bits >= 0)
+        png_set_compression_window_bits(png_ptr, zlib_compression.window_bits);
+
+    if (zlib_compression.method >= 0)
+        png_set_compression_method(png_ptr, zlib_compression.method);
+
+    if (zlib_compression.buffer_size >= 0) {
+        setCompressionSize(png_ptr, zlib_compression.buffer_size);
+    }
+}
+                  
+
+
+static void
+makePngLine(png_byte *           const line,
+            const xel *          const xelrow,
+            unsigned int         const cols,
+            xelval               const maxval,
+            bool                 const alpha,
+            gray *               const alpha_mask,
+            colorhash_table      const cht,
+            coloralphahash_table const caht,
+            png_info *           const info_ptr,
+            xelval               const png_maxval,
+            unsigned int         const depth) {
+            
+    unsigned int col;
+    png_byte *pp;
+
+    pp = line;  /* start at beginning of line */
+    for (col = 0; col < cols; ++col) {
+        xel p_png;
+        xel const p = xelrow[col];
+        PPM_DEPTH(p_png, p, maxval, png_maxval);
+        if (info_ptr->color_type == PNG_COLOR_TYPE_GRAY ||
+            info_ptr->color_type == PNG_COLOR_TYPE_GRAY_ALPHA) {
+            if (depth == 16)
+                *pp++ = PNM_GET1(p_png) >> 8;
+            *pp++ = PNM_GET1(p_png) & 0xff;
+        } else if (info_ptr->color_type == PNG_COLOR_TYPE_PALETTE) {
+            unsigned int paletteIndex;
+            if (alpha)
+                paletteIndex = lookupColorAlpha(caht, &p, &alpha_mask[col]);
+            else
+                paletteIndex = ppm_lookupcolor(cht, &p);
+            *pp++ = paletteIndex;
+        } else if (info_ptr->color_type == PNG_COLOR_TYPE_RGB ||
+                   info_ptr->color_type == PNG_COLOR_TYPE_RGB_ALPHA) {
+            if (depth == 16)
+                *pp++ = PPM_GETR(p_png) >> 8;
+            *pp++ = PPM_GETR(p_png) & 0xff;
+            if (depth == 16)
+                *pp++ = PPM_GETG(p_png) >> 8;
+            *pp++ = PPM_GETG(p_png) & 0xff;
+            if (depth == 16)
+                *pp++ = PPM_GETB(p_png) >> 8;
+            *pp++ = PPM_GETB(p_png) & 0xff;
+        } else
+            pm_error("INTERNAL ERROR: undefined color_type");
+                
+        if (info_ptr->color_type & PNG_COLOR_MASK_ALPHA) {
+            int const png_alphaval = (int)
+                alpha_mask[col] * (float) png_maxval / maxval + 0.5;
+            if (depth == 16)
+                *pp++ = png_alphaval >> 8;
+            *pp++ = png_alphaval & 0xff;
+        }
+    }
+}
+
+
+
+static void
+writeRaster(png_struct *         const png_ptr,
+            png_info *           const info_ptr,
+            FILE *               const ifP,
+            pm_filepos           const imagepos,
+            unsigned int         const cols,
+            unsigned int         const rows,
+            xelval               const maxval,
+            int                  const format,
+            xelval               const png_maxval,
+            unsigned             const int depth,
+            bool                 const alpha,
+            gray **              const alpha_mask,
+            colorhash_table      const cht,
+            coloralphahash_table const caht
+            ) {
+/*----------------------------------------------------------------------------
+   Write the PNG raster via compressor *png_ptr, reading the PNM raster
+   from file *ifP, position 'imagepos'.
+
+   The PNG raster consists of IDAT chunks.
+-----------------------------------------------------------------------------*/
+    xel * xelrow;
+    png_byte *line;
+    unsigned int pass;
+
+    xelrow = pnm_allocrow(cols);
+
+    /* max: 3 color channels, one alpha channel, 16-bit */
+    MALLOCARRAY(line, cols * 8);
+    if (line == NULL)
+        pm_error("out of memory allocating PNG row buffer");
+
+    for (pass = 0; pass < png_set_interlace_handling(png_ptr); ++pass) {
+        unsigned int row;
+        pm_seek2(ifP, &imagepos, sizeof(imagepos));
+        for (row = 0; row < rows; ++row) {
+            pnm_readpnmrow(ifP, xelrow, cols, maxval, format);
+            pnm_promoteformatrow(xelrow, cols, maxval, format, maxval,
+                                 PPM_TYPE);
+            
+            makePngLine(line, xelrow, cols, maxval, alpha, alpha_mask[row],
+                        cht, caht, info_ptr, png_maxval, depth);
+
+            png_write_row(png_ptr, line);
+        }
+    }
+    pnm_freerow(xelrow);
+}
+
+
+
 static int 
 convertpnm(FILE * const ifp, 
            FILE * const afp, 
@@ -1593,13 +1780,9 @@ convertpnm(FILE * const ifp,
       /* Index into palette[] of the background color. */
 
   png_uint_16 histogram[MAXCOLORS];
-  png_byte *line;
-  png_byte *pp;
-  int pass;
   gray alpha_maxval;
   int alpha_rows;
   int alpha_cols;
-  int alpha_can_be_transparency_index;
   const char * noColormapReason;
       /* The reason that we shouldn't make a colormapped PNG, or NULL if
          we should.  malloc'ed null-terminated string.
@@ -1695,6 +1878,8 @@ convertpnm(FILE * const ifp,
   }
   if (alpha_opt) {
     pixel alpha_transcolor;
+    bool alpha_can_be_transparency_index;
+    bool all_opaque;
 
     if (verbose)
       pm_message ("reading alpha-channel image...");
@@ -1705,9 +1890,9 @@ convertpnm(FILE * const ifp,
       pm_closer (ifp);
       pm_error ("dimensions for image and alpha mask do not agree");
     }
-    alpha_trans(ifp, imagepos, cols, rows, maxval, format, 
-                alpha_mask, alpha_maxval,
-                &alpha_can_be_transparency_index, &alpha_transcolor);
+    analyzeAlpha(ifp, imagepos, cols, rows, maxval, format, 
+                 alpha_mask, alpha_maxval, &all_opaque,
+                 &alpha_can_be_transparency_index, &alpha_transcolor);
 
     if (alpha_can_be_transparency_index && !force) {
       if (verbose)
@@ -1715,6 +1900,9 @@ convertpnm(FILE * const ifp,
       alpha = FALSE;
       transparent = 2;
       transcolor = alpha_transcolor;
+    } else if (all_opaque) {
+        alpha = FALSE;
+        transparent = -1;
     } else {
       alpha = TRUE;
       transparent = -1;
@@ -1764,9 +1952,9 @@ convertpnm(FILE * const ifp,
                     force, &pnm_meaningful_bits);
   
   computeColorMap(ifp, imagepos, cols, rows, maxval, format,
-                  force, pfp, alpha, transparent >= 0, transcolor, transexact, 
+                  force, pfp, alpha, transparent, transcolor, transexact, 
                   background, backcolor,
-                  alpha_mask, alpha_maxval, pnm_meaningful_bits,
+                  alpha_mask, pnm_meaningful_bits,
                   palette_pnm, &palette_size, trans_pnm, &trans_size,
                   &background_index, &noColormapReason);
 
@@ -1991,17 +2179,10 @@ convertpnm(FILE * const ifp,
     png_convert_from_struct_tm (&info_ptr->mod_time, &time_struct);
   }
 
-  /* explicit filter-type (or none) required */
   if (filterSet != 0)
-  {
-    png_set_filter (png_ptr, 0, filterSet);
-  }
+      png_set_filter(png_ptr, 0, filterSet);
 
-  /* zlib compression-level (or none) required */
-  if (zlib_compression.level >= 0)
-  {
-    png_set_compression_level (png_ptr, zlib_compression.level);
-  }
+  setZlibCompression(png_ptr, zlib_compression);
 
   /* write the png-info struct */
   png_write_info (png_ptr, info_ptr);
@@ -2017,69 +2198,9 @@ convertpnm(FILE * const ifp,
   /* let libpng take care of, e.g., bit-depth conversions */
   png_set_packing (png_ptr);
 
-  /* Write the raster */
- 
-  /* max: 3 color channels, one alpha channel, 16-bit */
-  MALLOCARRAY(line, cols * 8);
-  if (line == NULL)
-  {
-    png_destroy_write_struct (&png_ptr, &info_ptr);
-    pm_closer (ifp);
-    pm_error ("out of memory allocating PNG row buffer");
-  }
+  writeRaster(png_ptr, info_ptr, ifp, imagepos, cols, rows, maxval, format,
+              png_maxval, depth, alpha, alpha_mask, cht, caht);
 
-  for (pass = 0 ; pass < png_set_interlace_handling (png_ptr) ; pass++) {
-      unsigned int row;
-      pm_seek2(ifp, &imagepos, sizeof(imagepos));
-      for (row = 0; row < rows; ++row) {
-          unsigned int col;
-          pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
-          pnm_promoteformatrow(xelrow, cols, maxval, format, maxval, PPM_TYPE);
-          pp = line;
-          for (col = 0 ; col < cols ; ++col) {
-              xel p_png;
-              xel const p = xelrow[col];
-              PPM_DEPTH(p_png, p, maxval, png_maxval);
-              if (info_ptr->color_type == PNG_COLOR_TYPE_GRAY ||
-                  info_ptr->color_type == PNG_COLOR_TYPE_GRAY_ALPHA) {
-                  if (depth == 16)
-                      *pp++ = PNM_GET1 (p_png) >> 8;
-                  *pp++ = PNM_GET1 (p_png)&0xff;
-              } else if (info_ptr->color_type == PNG_COLOR_TYPE_PALETTE) {
-                  unsigned int paletteIndex;
-                  if (alpha)
-                      paletteIndex = lookupColorAlpha(caht, &p, 
-                                                      &alpha_mask[row][col]);
-                  else
-                      paletteIndex = ppm_lookupcolor(cht, &p);
-                  *pp++ = paletteIndex;
-              } else if (info_ptr->color_type == PNG_COLOR_TYPE_RGB ||
-                         info_ptr->color_type == PNG_COLOR_TYPE_RGB_ALPHA) {
-                  if (depth == 16)
-                      *pp++ = PPM_GETR (p_png) >> 8;
-                  *pp++ = PPM_GETR (p_png)&0xff;
-                  if (depth == 16)
-                      *pp++ = PPM_GETG (p_png) >> 8;
-                  *pp++ = PPM_GETG (p_png)&0xff;
-                  if (depth == 16)
-                      *pp++ = PPM_GETB (p_png) >> 8;
-                  *pp++ = PPM_GETB (p_png)&0xff;
-              } else {
-                  png_destroy_write_struct (&png_ptr, &info_ptr);
-                  pm_closer (ifp);
-                  pm_error (" (can't happen) undefined color_type");
-              }
-              if (info_ptr->color_type & PNG_COLOR_MASK_ALPHA) {
-                  int const png_alphaval = (int)
-                      alpha_mask[row][col] * (float) png_maxval / maxval +0.5;
-                  if (depth == 16)
-                      *pp++ = png_alphaval >> 8;
-                  *pp++ = png_alphaval & 0xff;
-              }
-          }
-          png_write_row (png_ptr, line);
-      }
-  }
   png_write_end (png_ptr, info_ptr);
 
 
@@ -2118,7 +2239,7 @@ convertpnm(FILE * const ifp,
 static void
 displayVersion() {
 
-    fprintf(stderr,"pnmtopng version %s.\n", VERSION);
+    fprintf(stderr,"Pnmtopng version %s.\n", NETPBM_VERSION);
 
     /* We'd like to display the version of libpng with which we're
        linked, as we do for zlib, but it isn't practical.
@@ -2137,10 +2258,6 @@ displayVersion() {
             PNG_LIBPNG_VER_STRING);
     fprintf(stderr, "   Compiled with zlib %s; using zlib %s.\n",
             ZLIB_VERSION, zlib_version);
-    fprintf(stderr,    
-            "   Compiled with %d-bit netpbm support "
-            "(PPM_OVERALLMAXVAL = %d).\n",
-            pm_maxvaltobits (PPM_OVERALLMAXVAL), PPM_OVERALLMAXVAL);
     fprintf(stderr, "\n");
 }
 
@@ -2182,6 +2299,11 @@ main(int argc, char *argv[]) {
   argn = 1;
 
   zlib_compression.level = -1;  /* initial value */
+  zlib_compression.mem_level = -1;
+  zlib_compression.strategy = -1;
+  zlib_compression.window_bits = -1;
+  zlib_compression.method = -1;
+  zlib_compression.buffer_size = -1;
   /* TODO: switch this to shhopt, and make options for the rest of
      zlib_compression.
   */
