@@ -21,8 +21,10 @@
 ** other special, indirect and consequential damages.
 */
 
-#include "pnm.h"
-#include "shhopt.h"
+#define _XOPEN_SOURCE    /* Make sure stdio.h contains fileno() */
+
+#include <unistd.h>
+#include <stdio.h>
 #include <fcntl.h>
 #ifdef VMS
 #ifdef SYSV
@@ -34,7 +36,10 @@
 
 #include <string.h>
 
-#include "ppmcmap.h"
+#include "pnm.h"
+#include "shhopt.h"
+#include "mallocvar.h"
+
 #define MAXCOLORS 256
 
 /* 2001.09.03: Old tiff.h doesn't contain these values: */
@@ -46,7 +51,7 @@ struct sizeset {
     bool b1, b2, b4, b8;
 };
 
-struct cmdline_info {
+struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
@@ -62,33 +67,41 @@ struct cmdline_info {
     unsigned int color;        /* logical: assume not grayscale */
     float xresolution; /* XRESOLUTION Tiff tag value or -1 for none */
     float yresolution; /* YRESOLUTION Tiff tag value or -1 for none */
+    int resolutionUnit;  /* RESOLUTIONUNIT Tiff tag value */
     struct sizeset indexsizeAllowed;
         /* Which bit widths are allowable in a raster of palette indices */
+    unsigned int verbose;
+    unsigned int append;
 };
 
 
 
 static void
 parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdline_p) {
+                   struct cmdlineInfo * const cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc(100*sizeof(optEntry));
+    optEntry * option_def;
 
     optStruct3 opt;
 
     unsigned int none, packbits, lzw, g3, g4, msb2lsb, lsb2msb, opt_2d, fill;
     unsigned int flate, adobeflate;
     char * indexbits;
+    char * resolutionUnit;
 
     unsigned int predictorSpec, rowsperstripSpec, xresolutionSpec,
-                 yresolutionSpec, indexbitsSpec;
+                 yresolutionSpec, indexbitsSpec, resolutionUnitSpec;
 
     unsigned int option_def_index;
 
+    MALLOCARRAY_NOFAIL(option_def, 100);
+
     option_def_index = 0;   /* incremented by OPTENT3 */
+    OPTENT3(0, "verbose",      OPT_FLAG,   NULL, &cmdlineP->verbose,       0);
+    OPTENT3(0, "append",       OPT_FLAG,   NULL, &cmdlineP->append,        0);
     OPTENT3(0, "none",         OPT_FLAG,   NULL, &none,                    0);
     OPTENT3(0, "packbits",     OPT_FLAG,   NULL, &packbits,                0);
     OPTENT3(0, "lzw",          OPT_FLAG,   NULL, &lzw,                     0);
@@ -100,20 +113,22 @@ parse_command_line(int argc, char ** argv,
     OPTENT3(0, "lsb2msb",      OPT_FLAG,   NULL, &lsb2msb,                 0);
     OPTENT3(0, "2d",           OPT_FLAG,   NULL, &opt_2d,                  0);
     OPTENT3(0, "fill",         OPT_FLAG,   NULL, &fill,                    0);
-    OPTENT3(0, "minisblack",   OPT_FLAG,   NULL, &cmdline_p->minisblack,   0);
-    OPTENT3(0, "mb",           OPT_FLAG,   NULL, &cmdline_p->minisblack,   0);
-    OPTENT3(0, "miniswhite",   OPT_FLAG,   NULL, &cmdline_p->miniswhite,   0);
-    OPTENT3(0, "mw",           OPT_FLAG,   NULL, &cmdline_p->miniswhite,   0);
-    OPTENT3(0, "truecolor",    OPT_FLAG,   NULL, &cmdline_p->truecolor,    0);
-    OPTENT3(0, "color",        OPT_FLAG,   NULL, &cmdline_p->color,        0);
-    OPTENT3(0, "predictor",    OPT_UINT,   &cmdline_p->predictor,    
+    OPTENT3(0, "minisblack",   OPT_FLAG,   NULL, &cmdlineP->minisblack,    0);
+    OPTENT3(0, "mb",           OPT_FLAG,   NULL, &cmdlineP->minisblack,    0);
+    OPTENT3(0, "miniswhite",   OPT_FLAG,   NULL, &cmdlineP->miniswhite,    0);
+    OPTENT3(0, "mw",           OPT_FLAG,   NULL, &cmdlineP->miniswhite,    0);
+    OPTENT3(0, "truecolor",    OPT_FLAG,   NULL, &cmdlineP->truecolor,     0);
+    OPTENT3(0, "color",        OPT_FLAG,   NULL, &cmdlineP->color,         0);
+    OPTENT3(0, "predictor",    OPT_UINT,   &cmdlineP->predictor,    
             &predictorSpec,    0);
-    OPTENT3(0, "rowsperstrip", OPT_UINT,   &cmdline_p->rowsperstrip, 
+    OPTENT3(0, "rowsperstrip", OPT_UINT,   &cmdlineP->rowsperstrip, 
             &rowsperstripSpec, 0);
-    OPTENT3(0, "xresolution",  OPT_FLOAT,  &cmdline_p->xresolution,  
+    OPTENT3(0, "xresolution",  OPT_FLOAT,  &cmdlineP->xresolution,  
             &xresolutionSpec,  0);
-    OPTENT3(0, "yresolution",  OPT_FLOAT,  &cmdline_p->yresolution,  
+    OPTENT3(0, "yresolution",  OPT_FLOAT,  &cmdlineP->yresolution,  
             &yresolutionSpec,  0);
+    OPTENT3(0, "resolutionunit", OPT_STRING, &resolutionUnit,
+            &resolutionUnitSpec,    0);
     OPTENT3(0, "indexbits",    OPT_STRING, &indexbits, 
             &indexbitsSpec,    0);
 
@@ -122,7 +137,7 @@ parse_command_line(int argc, char ** argv,
     opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
 
     optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
-        /* Uses and sets argc, argv, and some of *cmdline_p and others. */
+        /* Uses and sets argc, argv, and some of *cmdlineP and others. */
 
     if (none + packbits + lzw + g3 + g4 + flate + adobeflate > 1)
         pm_error("You specified more than one compression option.  "
@@ -130,102 +145,122 @@ parse_command_line(int argc, char ** argv,
                  "is allowed.");
     
     if (none)
-        cmdline_p->compression = COMPRESSION_NONE;
+        cmdlineP->compression = COMPRESSION_NONE;
     else if (packbits)
-        cmdline_p->compression = COMPRESSION_PACKBITS;
+        cmdlineP->compression = COMPRESSION_PACKBITS;
     else if (lzw)
-        cmdline_p->compression = COMPRESSION_LZW;
+        cmdlineP->compression = COMPRESSION_LZW;
     else if (g3)
-        cmdline_p->compression = COMPRESSION_CCITTFAX3;
+        cmdlineP->compression = COMPRESSION_CCITTFAX3;
     else if (g4)
-        cmdline_p->compression = COMPRESSION_CCITTFAX4;
+        cmdlineP->compression = COMPRESSION_CCITTFAX4;
     else if (adobeflate)
-        cmdline_p->compression = COMPRESSION_ADOBE_DEFLATE;
+        cmdlineP->compression = COMPRESSION_ADOBE_DEFLATE;
     else if (flate)
-        cmdline_p->compression = COMPRESSION_DEFLATE;
+        cmdlineP->compression = COMPRESSION_DEFLATE;
     else
-        cmdline_p->compression = COMPRESSION_NONE;
+        cmdlineP->compression = COMPRESSION_NONE;
     
     if (msb2lsb + lsb2msb > 1)
         pm_error("You specified both -msb2lsb and -lsb2msb.  "
                  "These are conflicting options.");
 
     if (msb2lsb)
-        cmdline_p->fillorder = FILLORDER_MSB2LSB;
+        cmdlineP->fillorder = FILLORDER_MSB2LSB;
     else if (lsb2msb)
-        cmdline_p->fillorder = FILLORDER_LSB2MSB;
+        cmdlineP->fillorder = FILLORDER_LSB2MSB;
     else 
-        cmdline_p->fillorder = FILLORDER_MSB2LSB;
+        cmdlineP->fillorder = FILLORDER_MSB2LSB;
     
 
-    if (cmdline_p->miniswhite && cmdline_p->minisblack)
+    if (cmdlineP->miniswhite && cmdlineP->minisblack)
         pm_error("You cannot specify both -miniswhite and -minisblack");
 
-    cmdline_p->g3options = 0;  /* initial value */
+    cmdlineP->g3options = 0;  /* initial value */
     if (opt_2d)
-        cmdline_p->g3options |= GROUP3OPT_2DENCODING;
+        cmdlineP->g3options |= GROUP3OPT_2DENCODING;
     if (fill)
-        cmdline_p->g3options |= GROUP3OPT_FILLBITS;
+        cmdlineP->g3options |= GROUP3OPT_FILLBITS;
 
     if (predictorSpec) {
-        if (cmdline_p->predictor != 1 && cmdline_p->predictor != 2)
+        if (cmdlineP->predictor != 1 && cmdlineP->predictor != 2)
             pm_error("-predictor may be only 1 or 2.  You specified %d.", 
-                     cmdline_p->predictor);
+                     cmdlineP->predictor);
     } else
-        cmdline_p->predictor = -1;
+        cmdlineP->predictor = -1;
 
     if (rowsperstripSpec) {
-        if (cmdline_p->rowsperstrip < 1)
+        if (cmdlineP->rowsperstrip < 1)
             pm_error("-rowsperstrip must be positive.  You specified %d.",
-                     cmdline_p->rowsperstrip);
+                     cmdlineP->rowsperstrip);
     } else
-        cmdline_p->rowsperstrip = -1;
+        cmdlineP->rowsperstrip = -1;
 
     if (xresolutionSpec) {
-        if (cmdline_p->xresolution < 1)
+        if (cmdlineP->xresolution < 1)
             pm_error("-xresolution must be positive.  You specified %f.",
-                     cmdline_p->xresolution);
+                     cmdlineP->xresolution);
     } else
-        cmdline_p->xresolution = -1;
+        cmdlineP->xresolution = -1;
 
     if (yresolutionSpec) {
-        if (cmdline_p->yresolution < 1)
+        if (cmdlineP->yresolution < 1)
             pm_error("-yresolution must be positive.  You specified %f.",
-                     cmdline_p->yresolution);
+                     cmdlineP->yresolution);
     } else
-        cmdline_p->yresolution = -1;
-    
+        cmdlineP->yresolution = -1;
+
+    if (resolutionUnitSpec) {
+        if (strcmp(resolutionUnit, "inch") == 0)
+            cmdlineP->resolutionUnit = RESUNIT_INCH;
+        else if (strcmp(resolutionUnit, "in") == 0)
+            cmdlineP->resolutionUnit = RESUNIT_INCH;
+        else if (strcmp(resolutionUnit, "centimeter") == 0)
+            cmdlineP->resolutionUnit = RESUNIT_CENTIMETER;
+        else if (strcmp(resolutionUnit, "cm") == 0)
+            cmdlineP->resolutionUnit = RESUNIT_CENTIMETER;
+        else if (strcmp(resolutionUnit, "none") == 0)
+            cmdlineP->resolutionUnit = RESUNIT_NONE;
+        else if (strcmp(resolutionUnit, "no") == 0)
+            cmdlineP->resolutionUnit = RESUNIT_NONE;
+        else
+            pm_error("The only acceptable values for -resolutionunit are "
+                     "inch, centimeter, none, in, cm, and no.  "
+                     "You specified '%s'.", resolutionUnit);
+    } else
+        cmdlineP->resolutionUnit = RESUNIT_INCH;
+
     if (indexbitsSpec) {
         if (strstr(indexbits, "1"))
-            cmdline_p->indexsizeAllowed.b1 = TRUE;
+            cmdlineP->indexsizeAllowed.b1 = TRUE;
         else
-            cmdline_p->indexsizeAllowed.b1 = FALSE;
+            cmdlineP->indexsizeAllowed.b1 = FALSE;
         if (strstr(indexbits, "2"))
-            cmdline_p->indexsizeAllowed.b2 = TRUE;
+            cmdlineP->indexsizeAllowed.b2 = TRUE;
         else
-            cmdline_p->indexsizeAllowed.b2 = FALSE;
+            cmdlineP->indexsizeAllowed.b2 = FALSE;
         if (strstr(indexbits, "4"))
-            cmdline_p->indexsizeAllowed.b4 = TRUE;
+            cmdlineP->indexsizeAllowed.b4 = TRUE;
         else
-            cmdline_p->indexsizeAllowed.b4 = FALSE;
+            cmdlineP->indexsizeAllowed.b4 = FALSE;
         if (strstr(indexbits, "8"))
-            cmdline_p->indexsizeAllowed.b8 = TRUE;
+            cmdlineP->indexsizeAllowed.b8 = TRUE;
         else
-            cmdline_p->indexsizeAllowed.b8 = FALSE;
+            cmdlineP->indexsizeAllowed.b8 = FALSE;
     } else {
-        cmdline_p->indexsizeAllowed.b1 = FALSE;
-        cmdline_p->indexsizeAllowed.b2 = FALSE;
-        cmdline_p->indexsizeAllowed.b4 = FALSE;
-        cmdline_p->indexsizeAllowed.b8 = TRUE;
+        cmdlineP->indexsizeAllowed.b1 = FALSE;
+        cmdlineP->indexsizeAllowed.b2 = FALSE;
+        cmdlineP->indexsizeAllowed.b4 = FALSE;
+        cmdlineP->indexsizeAllowed.b8 = TRUE;
     }
 
     if (argc-1 == 0) 
-        cmdline_p->input_filespec = "-";
+        cmdlineP->input_filespec = "-";
     else if (argc-1 != 1)
         pm_error("Program takes zero or one argument (filename).  You "
                  "specified %d", argc-1);
     else
-        cmdline_p->input_filespec = argv[1];
+        cmdlineP->input_filespec = argv[1];
 }
 
 
@@ -424,7 +459,7 @@ writeScanLines(FILE *          const ifp,
 
 static void
 analyzeColors(FILE *              const ifp,
-              struct cmdline_info const cmdline,
+              struct cmdlineInfo  const cmdline,
               int                 const cols,
               int                 const rows,
               xelval              const maxval,
@@ -599,18 +634,96 @@ computeRasterParm(int              const cols,
 
 
 
-int
-main(int argc, char *argv[]) {
-    struct cmdline_info cmdline;
-    const char* inf;
-    FILE* ifp;
+static void
+validateSeekableOutputFile(int          const ofd,
+                           const char * const outFileName) {
+/*----------------------------------------------------------------------------
+   Validate that the file attached to file descriptor 'ofd' is capable
+   of seeking.  If not, fail the program.
+
+   This is useful because the TIFF library requires seekable output and
+   fails with an unhelpful error message about a file I/O error if it is
+   not.  We, on the other hand, give a helpful error message.
+
+   We leave the file positioned to the beginning.
+-----------------------------------------------------------------------------*/
+    int rc;
+
+    /* We'd like to get the current position and leave the file positioned
+       where we found it.  But that entails the mess with some systems
+       having 32 bit file offsets and some having 64 bit file offsets.
+    */
+
+    /* Some files can seek ahead, but not back.  So we test for the
+       more difficult backward seek.
+    */
+    lseek(ofd, 1, SEEK_SET);
+    rc = lseek(ofd, 0, SEEK_SET);
+            
+    if (rc < 0)
+        pm_error("Output file (%s) is not seekable.  lseek() returned "
+                 "errno %d (%s).  "
+                 "The TIFF library can write only to "
+                 "a seekable file.", 
+                 outFileName, errno, strerror(errno));
+}
+
+
+
+static void
+createTiffGenerator(int          const ofd, 
+                    const char * const outFileName,
+                    bool         const append,
+                    TIFF **      const tifPP) {
+
+    const char * option;
+
+    /* Before 10.12 (November 2002), we set O_NONBLOCK here:
+
+       fcntl( 1, F_SETFL, O_NONBLOCK ) ; 
+   
+       I have no idea why.  The comment attached said, 
+
+         acooke dec99 - otherwise blocks on read inside 
+         next line (Linux i386) 
+    */
+
+    validateSeekableOutputFile(ofd, outFileName);
+
+    if (append)
+        option = "a";
+    else
+        option = "w";
+
+    *tifPP = TIFFFdOpen(ofd, outFileName, option);
+    if (*tifPP == NULL)
+        pm_error("error opening standard output as TIFF file.  "
+                 "TIFFFdOpen() failed.");
+}
+
+
+
+static void
+destroyTiffGenerator(TIFF * const tifP) {
+
+    TIFFFlushData(tifP);
+    TIFFClose(tifP);
+}
+
+
+
+static void
+convertImage(FILE *             const ifP,
+             TIFF *             const tifP,
+             const char *       const inputFileDescription,
+             struct cmdlineInfo const cmdline) {
+
     colorhist_vector chv;
     colorhash_table cht;
     unsigned short red[MAXCOLORS], grn[MAXCOLORS], blu[MAXCOLORS];
     int cols, rows, format, colors, i;
     xelval maxval;  /* maxval of PNM input */
     int grayscale;
-    TIFF* tif;
     unsigned short photometric;
     unsigned short rowsperstrip;
     unsigned short samplesperpixel;
@@ -618,38 +731,19 @@ main(int argc, char *argv[]) {
     unsigned short tiff_maxval;
        /* This is the maxval of the samples in the tiff file.  It is 
           determined solely by the bits per sample ('bitspersample').
-          */
+       */
     int bytesperrow;
-    unsigned int rasterPos;
+    pm_filepos rasterPos;
 
-    pnm_init( &argc, argv );
+    pnm_readpnminit(ifP, &cols, &rows, &maxval, &format);
 
-    parse_command_line(argc, argv, &cmdline);
-    
-    ifp = pm_openr_seekable(cmdline.input_filespec);
+    pm_tell2(ifP, &rasterPos, sizeof(rasterPos));
 
-    if (strcmp(cmdline.input_filespec, "-") == 0)
-        inf = "Standard Input";
-    else 
-        inf = cmdline.input_filespec;
-
-    pnm_readpnminit(ifp, &cols, &rows, &maxval, &format);
-
-    rasterPos = pm_tell(ifp);
-
-    analyzeColors(ifp, cmdline, cols, rows, maxval, format, MAXCOLORS, 
+    analyzeColors(ifP, cmdline, cols, rows, maxval, format, MAXCOLORS, 
                   &chv, &colors, &grayscale);
 
-    pm_seek(ifp, rasterPos);  /* Go back to beginning of raster */
-
-    /* Open output file. */
-    fcntl( 1, F_SETFL, O_NONBLOCK ) ; 
-      /* acooke dec99 - otherwise blocks on read inside 
-         next line (Linux i386) 
-         */
-    tif = TIFFFdOpen( 1, "Standard Output", "w" );
-    if ( tif == NULL )
-        pm_error( "error opening standard output as TIFF file" );
+    /* Go back to beginning of raster */
+    pm_seek2(ifP, &rasterPos, sizeof(rasterPos));  
 
     /* Figure out TIFF parameters. */
 
@@ -664,58 +758,138 @@ main(int argc, char *argv[]) {
     tiff_maxval = pm_bitstomaxval( bitspersample );
 
     /* Set TIFF parameters. */
-    TIFFSetField( tif, TIFFTAG_IMAGEWIDTH, cols );
-    TIFFSetField( tif, TIFFTAG_IMAGELENGTH, rows );
-    TIFFSetField( tif, TIFFTAG_BITSPERSAMPLE, bitspersample );
-    TIFFSetField( tif, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT );
-    TIFFSetField( tif, TIFFTAG_COMPRESSION, cmdline.compression );
+    TIFFSetField(tifP, TIFFTAG_IMAGEWIDTH,    cols);
+    TIFFSetField(tifP, TIFFTAG_IMAGELENGTH,   rows);
+    TIFFSetField(tifP, TIFFTAG_BITSPERSAMPLE, bitspersample);
+    TIFFSetField(tifP, TIFFTAG_ORIENTATION,   ORIENTATION_TOPLEFT);
+    TIFFSetField(tifP, TIFFTAG_COMPRESSION,   cmdline.compression);
     if (cmdline.compression == COMPRESSION_CCITTFAX3 && cmdline.g3options != 0)
-        TIFFSetField( tif, TIFFTAG_GROUP3OPTIONS, cmdline.g3options );
-    if ( cmdline.compression == COMPRESSION_LZW && cmdline.predictor != -1 )
-        TIFFSetField( tif, TIFFTAG_PREDICTOR, cmdline.predictor );
-    TIFFSetField( tif, TIFFTAG_PHOTOMETRIC, photometric );
-    TIFFSetField( tif, TIFFTAG_FILLORDER, cmdline.fillorder );
-    TIFFSetField( tif, TIFFTAG_DOCUMENTNAME, inf );
-    TIFFSetField( tif, TIFFTAG_IMAGEDESCRIPTION, "converted PNM file" );
-    TIFFSetField( tif, TIFFTAG_SAMPLESPERPIXEL, samplesperpixel );
-    TIFFSetField( tif, TIFFTAG_ROWSPERSTRIP, rowsperstrip );
-    if ( cmdline.xresolution != -1 )
-        TIFFSetField( tif, TIFFTAG_XRESOLUTION, cmdline.xresolution );
-    if ( cmdline.yresolution != -1 )
-        TIFFSetField( tif, TIFFTAG_YRESOLUTION, cmdline.yresolution );
-    TIFFSetField( tif, TIFFTAG_RESOLUTIONUNIT, RESUNIT_INCH);
-    /* TIFFSetField( tif, TIFFTAG_STRIPBYTECOUNTS, rows / rowsperstrip ); */
-    TIFFSetField( tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG );
+        TIFFSetField( tifP, TIFFTAG_GROUP3OPTIONS, cmdline.g3options );
+    if (cmdline.compression == COMPRESSION_LZW && cmdline.predictor != -1)
+        TIFFSetField( tifP, TIFFTAG_PREDICTOR, cmdline.predictor);
+    TIFFSetField(tifP, TIFFTAG_PHOTOMETRIC,      photometric);
+    TIFFSetField(tifP, TIFFTAG_FILLORDER,        cmdline.fillorder);
+    TIFFSetField(tifP, TIFFTAG_DOCUMENTNAME,     inputFileDescription);
+    TIFFSetField(tifP, TIFFTAG_IMAGEDESCRIPTION, "converted PNM file");
+    TIFFSetField(tifP, TIFFTAG_SAMPLESPERPIXEL,  samplesperpixel);
+    TIFFSetField(tifP, TIFFTAG_ROWSPERSTRIP,     rowsperstrip);
+    if (cmdline.xresolution != -1)
+        TIFFSetField(tifP, TIFFTAG_XRESOLUTION, cmdline.xresolution);
+    if (cmdline.yresolution != -1)
+        TIFFSetField(tifP, TIFFTAG_YRESOLUTION, cmdline.yresolution);
+    TIFFSetField(tifP, TIFFTAG_RESOLUTIONUNIT,   cmdline.resolutionUnit);
+    /* TIFFSetField( tifP, TIFFTAG_STRIPBYTECOUNTS, rows / rowsperstrip ); */
+    TIFFSetField(tifP, TIFFTAG_PLANARCONFIG,     PLANARCONFIG_CONTIG);
 
     if (!chv)
         cht = NULL;
     else {
         /* Make TIFF colormap. */
         for (i = 0; i < colors; ++i) {
-            red[i] = (long) PPM_GETR( chv[i].color ) * 65535L / maxval;
-            grn[i] = (long) PPM_GETG( chv[i].color ) * 65535L / maxval;
-            blu[i] = (long) PPM_GETB( chv[i].color ) * 65535L / maxval;
+            red[i] = (long) PPM_GETR(chv[i].color) * 65535L / maxval;
+            grn[i] = (long) PPM_GETG(chv[i].color) * 65535L / maxval;
+            blu[i] = (long) PPM_GETB(chv[i].color) * 65535L / maxval;
         }
         /* Clear unused colormap entries. */
         for (i = colors; i < (1 << bitspersample); ++i) {
             red[i] = grn[i] = blu[i] = 0;
         }
-        TIFFSetField( tif, TIFFTAG_COLORMAP, red, grn, blu );
+        TIFFSetField(tifP, TIFFTAG_COLORMAP, red, grn, blu);
 
         /* Convert color vector to color hash table, for fast lookup. */
-        cht = ppm_colorhisttocolorhash( chv, colors );
-        ppm_freecolorhist( chv );
+        cht = ppm_colorhisttocolorhash(chv, colors);
+        ppm_freecolorhist(chv);
     }
 
-    writeScanLines(ifp, tif, cht, format, cols, rows, maxval, 
+    writeScanLines(ifP, tifP, cht, format, cols, rows, maxval, 
                    tiff_maxval, bitspersample, photometric, bytesperrow, 
                    grayscale, cmdline.fillorder);
-
-    TIFFFlushData(tif);
-    TIFFClose(tif);
-
-    pm_close(ifp);
-
-    exit(0);
 }
 
+
+
+static void
+validateReadableStdout(void) {
+/*----------------------------------------------------------------------------
+   We validate that Standard Output is readable and fail the program if
+   it isn't.
+
+   This is useful because there are situations in which the TIFF library
+   must read the output file and if it can't, it fails with an unhelpful
+   error message about a file I/O error.  We, on the other hand, produce
+   a helpful error message.
+-----------------------------------------------------------------------------*/
+    int flags;
+
+    flags = fcntl(STDOUT_FILENO, F_GETFL);
+
+    if (flags < 0) {
+        /* We couldn't get the flags.  So just assume the file's OK */
+    } else {
+        if ((flags & O_RDONLY) || (flags & O_RDWR)) {
+            /* File is readable.  All is well. */
+        } else
+            pm_error("Standard Output is not opened for reading.  "
+                     "In order to create a multi-image TIFF stream, "
+                     "Standard Output must be both readable and writable.");
+    }
+}
+
+
+
+int
+main(int argc, char *argv[]) {
+    struct cmdlineInfo cmdline;
+    const char * inputFileDescription;
+    FILE* ifP;
+    TIFF* tifP;
+    bool eof;
+    unsigned int imageSeq;
+
+    pnm_init(&argc, argv);
+
+    parse_command_line(argc, argv, &cmdline);
+    
+    ifP = pm_openr_seekable(cmdline.input_filespec);
+
+    if (strcmp(cmdline.input_filespec, "-") == 0)
+        inputFileDescription = "Standard Input";
+    else 
+        inputFileDescription = cmdline.input_filespec;
+
+    if (cmdline.append)
+        validateReadableStdout();
+
+    createTiffGenerator(STDOUT_FILENO, "Standard Output", cmdline.append,
+                        &tifP);
+
+    eof = FALSE;  /* initial assumption */
+    imageSeq = 0;
+
+    while (!eof) {
+        bool success;
+
+        if (cmdline.verbose)
+            pm_message("Converting Image %u", imageSeq);
+
+        pnm_nextimage(ifP, &eof);
+
+        if (!eof) {
+            if (imageSeq > 0)
+                validateReadableStdout();
+
+            convertImage(ifP, tifP, inputFileDescription, cmdline);
+            
+            success = TIFFWriteDirectory(tifP);
+            if (!success)
+                pm_error("Unable to write TIFF image %u to file.  "
+                         "tifWriteDirectory() failed.", imageSeq);
+            ++imageSeq;
+        }
+    }
+
+    destroyTiffGenerator(tifP);
+    pm_close(ifP);
+
+    return 0;
+}
