@@ -55,11 +55,7 @@ Software copyrights will soon need family trees... :-)
 
 */
 
-/* On AIX, _XOPEN_SOURCE keeps inttypes.h from conflicting with tiffio.h */
-#define _XOPEN_SOURCE
-
 #include <string.h>
-#include "pnm.h"
 #include <math.h>
 /* float.h used to be included only if __osf__ was defined.  On some
    platforms, float.h is also included automatically by math.h.  But at
@@ -70,6 +66,8 @@ Software copyrights will soon need family trees... :-)
 #include <fcntl.h>
 #include <tiffio.h>
 
+#include "pnm.h"
+
 /* first release after testing */
 /* second release after testing - small mods to netpbm package */
 #define VERSION 1.01
