@@ -43,7 +43,7 @@ struct box {
     int ury;  /* upper right Y coord */
 };
 
-static struct cmdline_info {
+struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
@@ -62,12 +62,12 @@ static struct cmdline_info {
     unsigned int dpi;    /* zero means unspecified */
     enum orientation orientation;
     unsigned int goto_stdout;
-} cmdline;
+};
 
 
 static void
-parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdlineP) {
+parseCommandLine(int argc, char ** argv,
+                 struct cmdlineInfo * const cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
@@ -288,13 +288,13 @@ computeSizeResBlind(unsigned int   const xmax,
 
 
 static void
-compute_size_res(struct cmdline_info const cmdline, 
-                 enum orientation    const orientation, 
-                 struct box          const bordered_box,
-                 unsigned int *      const xsizeP, 
-                 unsigned int *      const ysizeP,
-                 unsigned int *      const xresP, 
-                 unsigned int *      const yresP) {
+compute_size_res(struct cmdlineInfo const cmdline, 
+                 enum orientation   const orientation, 
+                 struct box         const bordered_box,
+                 unsigned int *     const xsizeP, 
+                 unsigned int *     const ysizeP,
+                 unsigned int *     const xresP, 
+                 unsigned int *     const yresP) {
 /*----------------------------------------------------------------------------
   Figure out how big the output image should be (return as
   *xsizeP and *ysizeP) and what output device resolution Ghostscript
@@ -395,13 +395,13 @@ language_declaration(const char input_filespec[], int const verbose) {
 
 
 static struct box
-compute_box_to_extract(const struct box cmdline_extract_box,
-                       const char input_filespec[],
-                       const int verbose) {
+compute_box_to_extract(struct box const cmdline_extract_box,
+                       char       const input_filespec[],
+                       bool       const verbose) {
 
     struct box retval;
 
-    if (cmdline.extract_box.llx != -1)
+    if (cmdline_extract_box.llx != -1)
         /* User told us what box to extract, so that's what we'll do */
         retval = cmdline_extract_box;
     else {
@@ -466,8 +466,8 @@ compute_box_to_extract(const struct box cmdline_extract_box,
 
 
 static enum orientation
-compute_orientation(struct cmdline_info const cmdline, 
-                    struct box          const extract_box) {
+compute_orientation(struct cmdlineInfo const cmdline, 
+                    struct box         const extract_box) {
 
     enum orientation retval;
     unsigned int const input_width  = extract_box.urx - extract_box.llx;
@@ -570,7 +570,7 @@ compute_pstrans(const struct box box, const enum orientation orientation,
 
 
 static const char *
-compute_outfile_arg(const struct cmdline_info cmdline) {
+compute_outfile_arg(const struct cmdlineInfo cmdline) {
 
     const char *retval;  /* malloc'ed */
 
@@ -610,7 +610,7 @@ compute_gs_device(const int format_type, const int forceplain) {
     const char * basetype;
     const char * retval;
 
-    switch (cmdline.format_type) {
+    switch (format_type) {
     case PBM_TYPE: basetype = "pbm"; break;
     case PGM_TYPE: basetype = "pgm"; break;
     case PPM_TYPE: basetype = "ppm"; break;
@@ -700,15 +700,22 @@ execGhostscript(int const inputPipeFd,
     asprintfN(&gopt, "-g%dx%d", xsize, ysize);
     asprintfN(&ropt, "-r%dx%d", xres, yres);
 
+    /* -dSAFER causes Postscript to disable %pipe and file operations,
+       which are almost certainly not needed here.  This prevents our
+       Postscript program from doing crazy unexpected things, possibly
+       as a result of a malicious booby trapping of our Postscript file.
+    */
+
     if (verbose) {
         pm_message("execing '%s' with args '%s' (arg 0), "
-                   "'%s', '%s', '%s', '%s', '%s', '%s', '%s'",
+                   "'%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s'",
                    ghostscriptProg, arg0,
-                   deviceopt, outfileopt, gopt, ropt, "-q", "-dNOPAUSE", "-");
+                   deviceopt, outfileopt, gopt, ropt, "-q", "-dNOPAUSE", 
+                   "-dSAFER", "-");
     }
 
     execl(ghostscriptProg, arg0, deviceopt, outfileopt, gopt, ropt, "-q",
-          "-dNOPAUSE", "-", NULL);
+          "-dNOPAUSE", "-dSAFER", "-", NULL);
     
     pm_error("execl() of Ghostscript ('%s') failed, errno=%d (%s)",
              ghostscriptProg, errno, strerror(errno));
@@ -830,6 +837,7 @@ execute_ghostscript(const char pstrans[], const char ghostscript_device[],
 int
 main(int argc, char **argv) {
 
+    struct cmdlineInfo cmdline;
     char *input_filespec;  /* malloc'ed */
         /* The file specification of our Postscript input file */
     unsigned int xres, yres;    /* Resolution in pixels per inch */
@@ -849,7 +857,7 @@ main(int argc, char **argv) {
 
     pnm_init(&argc, argv);
 
-    parse_command_line(argc, argv, &cmdline);
+    parseCommandLine(argc, argv, &cmdline);
 
     add_ps_to_filespec(cmdline.input_filespec, &input_filespec,
                        cmdline.verbose);
