@@ -53,6 +53,7 @@
 #include "pnm.h"
 #include "shhopt.h"
 #include "mallocvar.h"
+#include "nstring.h"
 
 #ifdef VMS
 #ifdef SYSV
@@ -78,7 +79,7 @@
 #define PHOTOMETRIC_DEPTH 32768
 #endif
 
-static struct cmdlineInfo {
+struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
@@ -88,7 +89,8 @@ static struct cmdlineInfo {
     int alphaStdout;
     unsigned int respectfillorder;   /* -respectfillorder option */
     unsigned int byrow;
-} cmdline;
+    unsigned int verbose;
+};
 
 
 
@@ -102,33 +104,37 @@ parseCommandLine(int argc, char ** argv,
    in argv!
 -----------------------------------------------------------------------------*/
     optStruct3 opt;
-    optEntry *option_def = malloc(100*sizeof(optEntry));
+    optEntry *option_def;
     unsigned int option_def_index;
     unsigned int alpha_spec;
 
+    MALLOCARRAY_NOFAIL(option_def, 100);
+
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;
     opt.allowNegNum = FALSE;
 
-    option_def_index = 0;   /* incremented by OPTENTRY */
+    option_def_index = 0;   /* incremented by OPTENT3 */
+    OPTENT3(0, "verbose", 
+            OPT_FLAG,   NULL, &cmdlineP->verbose,              0);
     OPTENT3(0, "respectfillorder", 
-            OPT_FLAG,   NULL, &cmdlineP->respectfillorder, 0);
+            OPT_FLAG,   NULL, &cmdlineP->respectfillorder,     0);
     OPTENT3(0,   "byrow",   
-            OPT_FLAG,   NULL, &cmdlineP->byrow, 0);
+            OPT_FLAG,   NULL, &cmdlineP->byrow,                0);
     OPTENT3('h', "headerdump", 
-            OPT_FLAG,   NULL, &cmdlineP->headerdump, 0);
+            OPT_FLAG,   NULL, &cmdlineP->headerdump,           0);
     OPTENT3(0,   "alphaout",   
             OPT_STRING, &cmdlineP->alphaFilename, &alpha_spec, 0);
 
     optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
 
     if (argc - 1 == 0)
-        cmdlineP->inputFilename = NULL;  /* he wants stdin */
+        cmdlineP->inputFilename = strdup("-");  /* he wants stdin */
     else if (argc - 1 == 1)
         cmdlineP->inputFilename = strdup(argv[1]);
     else 
-        pm_error("Too many arguments.  The only argument accepted\n"
-                 "is the input file specificaton");
+        pm_error("Too many arguments.  The only argument accepted "
+                 "is the input file name");
 
     if (alpha_spec && strcmp(cmdlineP->alphaFilename, "-") == 0 )
         cmdlineP->alphaStdout = 1;
@@ -138,7 +144,6 @@ parseCommandLine(int argc, char ** argv,
 
 
 
-
 static void 
 read_directory(TIFF * const tif,
                unsigned short * const bps_p,
@@ -393,14 +398,15 @@ computeFillorder(unsigned short   const fillorderTag,
 
 
 static void
-analyze_image_type(TIFF *         const tif, 
-                   unsigned short const bps, 
-                   unsigned short const spp, 
-                   unsigned short const photomet,
-                   xelval *       const maxvalP, 
-                   int *          const formatP, 
-                   xel                  colormap[],
-                   bool           const headerdump) {
+analyzeImageType(TIFF *             const tif, 
+                 unsigned short     const bps, 
+                 unsigned short     const spp, 
+                 unsigned short     const photomet,
+                 xelval *           const maxvalP, 
+                 int *              const formatP, 
+                 xel                      colormap[],
+                 bool               const headerdump,
+                 struct cmdlineInfo const cmdline) {
 
     bool grayscale; 
 
@@ -913,12 +919,11 @@ convertRasterInMemory(FILE *         const imageoutFile,
 
 
 
-
 static void
-convertIt(TIFF*              const tif,
-          FILE *             const alphaFile, 
-          FILE *             const imageoutFile,
-          struct cmdlineInfo const cmdline) {
+convertImage(TIFF *             const tifP,
+             FILE *             const alphaFile, 
+             FILE *             const imageoutFile,
+             struct cmdlineInfo const cmdline) {
 
     unsigned int cols, rows;
     int format;
@@ -929,15 +934,15 @@ convertIt(TIFF*              const tif,
     unsigned short photomet, planarconfig, fillorderTag;
     unsigned short fillorder;
 
-    read_directory(tif, &bps, &spp, &photomet, &planarconfig, &fillorderTag,
+    read_directory(tifP, &bps, &spp, &photomet, &planarconfig, &fillorderTag,
                    &cols, &rows, 
                    cmdline.headerdump);
 
 
     computeFillorder(fillorderTag, &fillorder, cmdline.respectfillorder);
 
-    analyze_image_type(tif, bps, spp, photomet, 
-                       &maxval, &format, colormap, cmdline.headerdump);
+    analyzeImageType(tifP, bps, spp, photomet, 
+                     &maxval, &format, colormap, cmdline.headerdump, cmdline);
 
     if (imageoutFile != NULL) 
         pnm_writepnminit( imageoutFile, 
@@ -952,7 +957,7 @@ convertIt(TIFF*              const tif,
         else {
             convertRasterInMemory(
                 imageoutFile, alphaFile, cols, rows, maxval, format, 
-                tif, photomet, planarconfig, bps, spp, fillorder,
+                tifP, photomet, planarconfig, bps, spp, fillorder,
                 colormap, &status);
         }
         if (status == CONV_DONE) {
@@ -968,16 +973,43 @@ convertIt(TIFF*              const tif,
             
             convertRasterByRows(
                 imageoutFile, alphaFile, cols, rows, maxval, format, 
-                tif, photomet, planarconfig, bps, spp, fillorder, colormap);
+                tifP, photomet, planarconfig, bps, spp, fillorder, colormap);
         }
     }
 }
 
 
 
+static void
+convertIt(TIFF *             const tifP,
+          FILE *             const alphaFile, 
+          FILE *             const imageoutFile,
+          struct cmdlineInfo const cmdline) {
+
+    unsigned int imageSeq;
+    bool eof;
+
+    imageSeq = 0;
+    eof = FALSE;
+
+    while (!eof) {
+        bool success;
+
+        if (cmdline.verbose)
+            pm_message("Converting Image %u", imageSeq);
+        convertImage(tifP, alphaFile, imageoutFile, cmdline);
+        success = TIFFReadDirectory(tifP);
+        eof = !success;
+        ++imageSeq;
+    }
+}
+
+
+
 int
 main(int argc, char * argv[]) {
 
+    struct cmdlineInfo cmdline;
     TIFF* tif;
     FILE *alphaFile, *imageoutFile;
 
@@ -985,23 +1017,22 @@ main(int argc, char * argv[]) {
 
     parseCommandLine(argc, argv, &cmdline);
 
-    if ( cmdline.inputFilename != NULL ) {
-        tif = TIFFOpen( cmdline.inputFilename, "r" );
-        if ( tif == NULL )
-            pm_error( "error opening TIFF file %s", cmdline.inputFilename );
+    if (strcmp(cmdline.inputFilename, "-") != 0) {
+        tif = TIFFOpen(cmdline.inputFilename, "r");
+        if (tif == NULL)
+            pm_error("error opening TIFF file %s", cmdline.inputFilename);
     } else {
-        tif = TIFFFdOpen( 0, "Standard Input", "r" );
-        if ( tif == NULL )
-            pm_error( "error opening standard input as TIFF file" );
+        tif = TIFFFdOpen(0, "Standard Input", "r");
+        if (tif == NULL)
+            pm_error("error opening standard input as TIFF file");
     }
 
     if (cmdline.alphaStdout)
         alphaFile = stdout;
     else if (cmdline.alphaFilename == NULL) 
         alphaFile = NULL;
-    else {
+    else
         alphaFile = pm_openw(cmdline.alphaFilename);
-    }
 
     if (cmdline.alphaStdout) 
         imageoutFile = NULL;
@@ -1015,9 +1046,10 @@ main(int argc, char * argv[]) {
     if (alphaFile != NULL)
         pm_close( alphaFile );
 
+    strfree(cmdline.inputFilename);
+
     /* If the program failed, it previously aborted with nonzero completion
        code, via various function calls.
     */
     return 0;
 }
-
