@@ -42,6 +42,10 @@ static struct cmdline_info {
 static bool debug;
 static bool verbose;
 
+#ifdef DEBUG_PIXEL
+static unsigned int pixel_count = 0;
+#endif
+
 struct row_control {
     /* This structure contains the state of the getpix() reader as it
        reads across a row in the input image.
@@ -79,8 +83,45 @@ struct row_control {
       /* This is the number of carryover bits in 'carryover_bits' */
 };
 
-static short bs_short ARGS(( short s ));
-static long bs_long ARGS(( long l ));
+/* Byte-swapping junk. */
+
+union cheat {
+    uint32n l;
+    short s;
+    unsigned char c[4];
+};
+
+
+
+static short
+bs_short(short const s) {
+    union cheat u;
+    unsigned char t;
+
+    u.s = s;
+    t = u.c[0];
+    u.c[0] = u.c[1];
+    u.c[1] = t;
+    return u.s;
+}
+
+
+
+static long
+bs_long(long const l) {
+    union cheat u;
+    unsigned char t;
+
+    u.l = l;
+    t = u.c[0];
+    u.c[0] = u.c[3];
+    u.c[3] = t;
+    t = u.c[1];
+    u.c[1] = u.c[2];
+    u.c[2] = t;
+    return u.l;
+}
+
 
 
 static int
@@ -418,7 +459,7 @@ dumpX11Header(X11WDFileHeader * const h11P) {
     case GrayScale:   visualClassDesc = "GrayScale";   break;
     case StaticColor: visualClassDesc = "StaticColor"; break;
     case PseudoColor: visualClassDesc = "PseudoColor"; break;
-    case TrueColor:   visualClassDesc = "TrueColor";   break;
+    case TrueColor:   visualClassDesc = "TrueColor"; break;
     case DirectColor: visualClassDesc = "DirectColor"; break;
     default:          visualClassDesc = "???";
     }
@@ -696,6 +737,44 @@ getinit(FILE *             const file,
 
 
 
+#ifdef DEBUG_PIXEL
+#define DEBUG_PIXEL_1 \
+   if (pixel_count < 4) \
+       pm_message("getting pixel %d", pixel_count);
+
+#define DEBUG_PIXEL_2 \
+    if (pixel_count < 4) \
+        pm_message("item: %.8lx", row_controlP->item.l); 
+
+
+#define DEBUG_PIXEL_3 \
+    if (pixel_count < 4) \
+        pm_message("  bits_taken: %lx(%d), carryover_bits: %lx(%d), " \
+                     "pixel: %lx", \
+                     bits_taken, bits_to_take, row_controlP->carryover_bits, \
+                     row_controlP->bits_carried_over, pixel);
+
+#define DEBUG_PIXEL_4 \
+    if (pixel_count < 4) { \
+        pm_message("  row_control.bits_carried_over = %d" \
+                   "  carryover_bits= %.8lx", \
+                   row_controlP->bits_carried_over, \
+                   row_controlP->carryover_bits); \
+        pm_message("  row_control.bits_used = %d", \
+                   row_controlP->bits_used); \
+        pm_message("  row_control.bits_left = %d", \
+                   row_controlP->bits_left); \
+                   } \
+    \
+    pixel_count++;
+#else
+#define DEBUG_PIXEL_1 do {} while(0)
+#define DEBUG_PIXEL_2 do {} while(0)
+#define DEBUG_PIXEL_3 do {} while(0)
+#define DEBUG_PIXEL_4 do {} while(0)
+#endif
+
+
 static unsigned long
 getpix(FILE *file, struct row_control * const row_controlP,
        const int bits_per_pixel, const int bits_per_item, 
@@ -714,7 +793,7 @@ getpix(FILE *file, struct row_control * const row_controlP,
 
    Each item can contain one or more pixels, and may contain
    fractional pixels.  'bits_per_pixel' tells us how many bits each
-   pixel has, and 'bits_per_pixel' is always less than
+   pixel has, and 'bits_per_pixel' is always less than or equal to
    'bits_per_item', but not necessarily a factor of it.  Within an item,
    after taking care of the endianness of its storage format, the pixels
    may be arranged from left to right or right to left.  'bit_order' tells
@@ -743,30 +822,24 @@ getpix(FILE *file, struct row_control * const row_controlP,
       /* This is the number of bits we need to take from the current
          item to combine with carryover bits to form a pixel.
          */
-    /*
-      if (pixel_count < 4)
-      pm_message("getting pixel %d", pixel_count);
-    */
+    DEBUG_PIXEL_1;
 
-    if ( row_controlP->bits_left <= 0) {
+    if (row_controlP->bits_left <= 0) {
         /* We've used up the most recently read item.  Get the next one
            from the input file.
-           */
-        switch ( bits_per_item )
-        {
+        */
+        switch (bits_per_item) {
         case 8:
-            row_controlP->item.c = getc( file );
+            row_controlP->item.c = getc(file);
             break;
 
         case 16:
             switch (byte_order) {
             case MSBFirst:
-                if (pm_readbigshort(file, &row_controlP->item.s) == -1)
-                    pm_error( "error reading image" );
+                pm_readbigshort(file, &row_controlP->item.s);
                 break;
             case LSBFirst:
-                if (pm_readlittleshort(file, &row_controlP->item.s) == -1)
-                    pm_error( "error reading image" );
+                pm_readlittleshort(file, &row_controlP->item.s);
                 break;
             }
             break;
@@ -774,23 +847,18 @@ getpix(FILE *file, struct row_control * const row_controlP,
         case 32:
             switch (byte_order) {
             case MSBFirst:
-                if (pm_readbiglong(file, &row_controlP->item.l) == -1)
-                    pm_error( "error reading image" );
+                pm_readbiglong(file, &row_controlP->item.l);
                 break;
             case LSBFirst:
-                if (pm_readlittlelong(file, &row_controlP->item.l) == -1) 
-                    pm_error( "error reading image");
+                pm_readlittlelong(file, &row_controlP->item.l);
                 break;
             }
             break;
 
         default:
-            pm_error( "can't happen" );
+            pm_error("INTERNAL ERROR: impossible bits_per_item");
         }
-        /* 
-        if (pixel_count < 4) 
-            pm_message("item: %.8lx", row_controlP->item.l); 
-        */
+        DEBUG_PIXEL_2;
 
         row_controlP->bits_used = 0;  
           /* Fresh item from file; none of it used yet. */
@@ -816,9 +884,9 @@ getpix(FILE *file, struct row_control * const row_controlP,
             bit_shift = row_controlP->bits_used;
 
         if ( bits_to_take == sizeof(bits_to_take_mask) * 8 )
-            bits_to_take_mask = -1L;
+            bits_to_take_mask = -1;
         else
-            bits_to_take_mask = ( 1L << bits_to_take ) - 1;
+            bits_to_take_mask = ( 1 << bits_to_take ) - 1;
         
         switch ( bits_per_item ) {
         case 8:
@@ -842,13 +910,7 @@ getpix(FILE *file, struct row_control * const row_controlP,
             pixel = bits_taken << row_controlP->bits_carried_over |
                 row_controlP->carryover_bits;
 
-        /*
-        if (pixel_count < 4)
-        pm_message("  bits_taken: %lx(%d), carryover_bits: %lx(%d), "
-                     "pixel: %lx",
-                     bits_taken, bits_to_take, row_controlP->carryover_bits, 
-                     row_controlP->bits_carried_over, pixel);
-        */
+        DEBUG_PIXEL_3;
     }
 
     row_controlP->bits_used += bits_to_take;
@@ -895,20 +957,7 @@ getpix(FILE *file, struct row_control * const row_controlP,
         row_controlP->carryover_bits = 0;
         row_controlP->bits_carried_over = 0;
     }
-    /*
-    if (pixel_count < 4) {
-        pm_message("  row_control.bits_carried_over = %d"
-                   "  carryover_bits= %.8lx", 
-                   row_controlP->bits_carried_over,
-                   row_controlP->carryover_bits);
-        pm_message("  row_control.bits_used = %d", 
-                   row_controlP->bits_used);
-        pm_message("  row_control.bits_left = %d",
-                   row_controlP->bits_left);
-                   }
- 
-    pixel_count++;
-    */
+    DEBUG_PIXEL_4;
     return pixel;
 }
 
@@ -1143,6 +1192,29 @@ convertRow(FILE*            const ifp,
 
 
 
+static void
+reportOutputType(int const format) {
+
+    switch (PNM_FORMAT_TYPE(format)) {
+    case PBM_TYPE:
+        pm_message("writing PBM file");
+    break;
+
+    case PGM_TYPE:
+        pm_message("writing PGM file");
+    break;
+    
+    case PPM_TYPE:
+        pm_message("writing PPM file");
+    break;
+    
+    default:
+        pm_error("shouldn't happen");
+    }
+}
+
+
+
 int
 main( int argc, char *argv[] ) {
     FILE* ifp;
@@ -1179,23 +1251,8 @@ main( int argc, char *argv[] ) {
                    byte_order, bit_order);
 
     pnm_writepnminit( stdout, cols, rows, maxval, format, 0 );
-    switch ( PNM_FORMAT_TYPE(format) )
-    {
-    case PBM_TYPE:
-    pm_message( "writing PBM file" );
-    break;
 
-    case PGM_TYPE:
-    pm_message( "writing PGM file" );
-    break;
-
-    case PPM_TYPE:
-    pm_message( "writing PPM file" );
-    break;
-
-    default:
-    pm_error( "shouldn't happen" );
-    }
+    reportOutputType(format);
 
     for ( row = 0; row < rows; ++row ) {
         convertRow(ifp, stdout, bits_per_pixel, bits_per_item,
@@ -1210,47 +1267,9 @@ main( int argc, char *argv[] ) {
     }
 
 
-/* Byte-swapping junk. */
-
-union cheat {
-    uint32n l;
-    short s;
-    unsigned char c[4];
-    };
-
-static short
-bs_short( short s )
-    {
-    union cheat u;
-    unsigned char t;
-
-    u.s = s;
-    t = u.c[0];
-    u.c[0] = u.c[1];
-    u.c[1] = t;
-    return u.s;
-    }
-
-static long
-bs_long( long l )
-    {
-    union cheat u;
-    unsigned char t;
-
-    u.l = l;
-    t = u.c[0];
-    u.c[0] = u.c[3];
-    u.c[3] = t;
-    t = u.c[1];
-    u.c[1] = u.c[2];
-    u.c[2] = t;
-    return u.l;
-    }
-
-
 /* 
    This used to be the way we parsed a direct/true color pixel.  I'm 
-   keeping it here in case we find out some applications needs it this way.
+   keeping it here in case we find out some application needs it this way.
 
    There doesn't seem to be any reason to do this hard-coded stuff when
    the header contains 32 bit masks that tell exactly how to extract the
