@@ -23,8 +23,7 @@
  * $Id$
  */
 
-#define _BSD_SOURCE 1
-    /* This makes sure mkstemp() is in unistd.h */
+#define _XOPEN_SOURCE
 
 #include <unistd.h>
 #include <stdio.h>
@@ -32,14 +31,10 @@
 #include <errno.h>
 #include <assert.h>
 
-#if defined(__MINGW32__) && !defined(__CYGWIN__)
-/* use libiberty replacement function */
-#define mkstemp mkstemps
-#endif
-
 #include "ppm.h"
 #include "pbmfont.h"
 #include "mallocvar.h"
+#include "nstring.h"
 
 static bool const dump = FALSE;
 
@@ -602,22 +597,22 @@ dump_rect(const char * const s, struct Rect * const r) {
 
 
 static int
-rectwidth(struct Rect * const r) {
+rectwidth(const struct Rect * const r) {
     return r->right - r->left;
 }
 
 
 
 static int
-rectheight(struct Rect * const r) {
+rectheight(const struct Rect * const r) {
     return r->bottom - r->top;
 }
 
 
 
-static int
-rectsamesize(struct Rect * const r1, 
-             struct Rect * const r2) {
+static bool
+rectsamesize(const struct Rect * const r1, 
+             const struct Rect * const r2) {
     return r1->right - r1->left == r2->right - r2->left &&
            r1->bottom - r1->top == r2->bottom - r2->top ;
 }
@@ -903,6 +898,181 @@ unlink(p)
 
 
 
+static void
+doDiffSize(struct Rect        const clipsrc,
+           struct Rect        const clipdst,
+           int                const pixSize,
+           int                const xsize,
+           int                const ysize,
+           struct RGBColour * const ctArg,
+           int                const srcaddArg,
+           int                const dstadd,
+           int                const srcwid, 
+           transfer_func      const trf,
+           struct RGBColour * const colour_map, 
+           byte *             const srcArg,
+           word *             const reddstArg,
+           word *             const greendstArg,
+           word *             const bluedstArg
+
+) {
+    FILE * pnmscalePipeP;
+    const char * command;
+    byte * redsrc;
+    byte * greensrc;
+    byte * bluesrc;
+    int greenpix;
+    FILE *   scaled;
+    int cols, rows, format;
+    pixval maxval;
+    pixel * row;
+    pixel * rowp;
+    FILE * tempFileP;
+    const char * tempFilename;
+    struct RGBColour * ct;
+    byte * src;
+    int srcadd;
+    word * reddst;
+    word * greendst;
+    word * bluedst;
+    struct RGBColour src_c;
+    struct RGBColour dst_c;
+
+    ct = ctArg; /* initial value */
+    src = srcArg; /* initial value */
+    srcadd = srcaddArg;  /* intial value */
+    reddst = reddstArg;
+    greendst = greendstArg;
+    bluedst = bluedstArg;
+
+    pm_make_tmpfile(&tempFileP, &tempFilename);
+
+    pm_close(tempFileP);
+
+    asprintfN(&command, "pnmscale -xsize %d -ysize %d > %s",
+              rectwidth(&clipdst), rectheight(&clipdst), tempFilename);
+
+    pm_message("running command '%s'", command);
+
+    pnmscalePipeP = popen(command, "w");
+    if (pnmscalePipeP == NULL)
+        pm_error("cannot execute command '%s'  popen() errno = %s (%d)",
+                 command, strerror(errno), errno);
+
+    strfree(command);
+
+    fprintf(pnmscalePipeP, "P6\n%d %d\n%d\n",
+            rectwidth(&clipsrc), rectheight(&clipsrc), PPM_MAXMAXVAL);
+
+#define REDEPTH(c, oldmax)  ((c) * ((PPM_MAXMAXVAL) + 1) / (oldmax + 1))
+
+    switch (pixSize) {
+    case 8: {
+        unsigned int i;
+        for (i = 0; i < ysize; ++i) {
+            unsigned int j;
+            for (j = 0; j < xsize; ++j) {
+                ct = colour_map + *src++;
+                fputc(REDEPTH(ct->red, 65535L), pnmscalePipeP);
+                fputc(REDEPTH(ct->green, 65535L), pnmscalePipeP);
+                fputc(REDEPTH(ct->blue, 65535L), pnmscalePipeP);
+            }
+            src += srcadd;
+        }
+    }
+    break;
+    case 16: {
+        unsigned int i;
+        for (i = 0; i < ysize; ++i) {
+            unsigned int j;
+            for (j = 0; j < xsize; ++j) {
+                fputc(REDEPTH((*src & 0x7c) >> 2, 32), pnmscalePipeP);
+                greenpix = (*src++ & 3) << 3;
+                greenpix |= (*src & 0xe0) >> 5;
+                fputc(REDEPTH(greenpix, 32), pnmscalePipeP);
+                fputc(REDEPTH((*src++ & 0x1f) << 11, 32), pnmscalePipeP);
+            }
+            src += srcadd;
+        }
+    }
+    break;
+    case 32: {
+        unsigned int i;
+
+        srcadd = srcwid - xsize;
+        redsrc = src;
+        greensrc = src + (srcwid / 3);
+        bluesrc = greensrc + (srcwid / 3);
+
+        for (i = 0; i < ysize; ++i) {
+            unsigned int j;
+            for (j = 0; j < xsize; ++j) {
+                fputc(REDEPTH(*redsrc++, 256), pnmscalePipeP);
+                fputc(REDEPTH(*greensrc++, 256), pnmscalePipeP);
+                fputc(REDEPTH(*bluesrc++, 256), pnmscalePipeP);
+            }
+            redsrc += srcadd;
+            greensrc += srcadd;
+            bluesrc += srcadd;
+        }
+    }
+    break;
+    }
+
+    if (pclose(pnmscalePipeP)) {
+        pm_error("pnmscale failed.  pclose() returned Errno %s (%d)",
+                 strerror(errno), errno);
+    }
+
+    ppm_readppminit(scaled = pm_openr(tempFilename), &cols, &rows,
+                    &maxval, &format);
+    row = ppm_allocrow(cols);
+    /* couldn't hurt to assert cols, rows and maxval... */  
+
+    if (trf == 0) {
+        while (rows-- > 0) {
+            unsigned int i;
+            ppm_readppmrow(scaled, row, cols, maxval, format);
+            for (i = 0, rowp = row; i < cols; ++i, ++rowp) {
+                *reddst++ = PPM_GETR(*rowp) * 65536L / (maxval + 1); 
+                *greendst++ = PPM_GETG(*rowp) * 65536L / (maxval + 1); 
+                *bluedst++ = PPM_GETB(*rowp) * 65536L / (maxval + 1); 
+            }
+            reddst += dstadd;
+            greendst += dstadd;
+            bluedst += dstadd;
+        }
+    }
+    else {
+        while (rows-- > 0) {
+            unsigned int i;
+            ppm_readppmrow(scaled, row, cols, maxval, format);
+            for (i = 0, rowp = row; i < cols; i++, rowp++) {
+                dst_c.red = *reddst;
+                dst_c.green = *greendst;
+                dst_c.blue = *bluedst;
+                src_c.red = PPM_GETR(*rowp) * 65536L / (maxval + 1); 
+                src_c.green = PPM_GETG(*rowp) * 65536L / (maxval + 1); 
+                src_c.blue = PPM_GETB(*rowp) * 65536L / (maxval + 1); 
+                (*trf)(&src_c, &dst_c);
+                *reddst++ = dst_c.red;
+                *greendst++ = dst_c.green;
+                *bluedst++ = dst_c.blue;
+            }
+            reddst += dstadd;
+            greendst += dstadd;
+            bluedst += dstadd;
+        }
+    }
+
+    pm_close(scaled);
+    ppm_freerow(row);
+    strfree(tempFilename);
+    unlink(tempFilename);
+}
+
+
+
 static int
 blit(struct Rect *      const srcRect, 
      struct Rect *      const srcBounds, 
@@ -996,167 +1166,10 @@ blit(struct Rect *      const srcRect,
         trf = transfer(mode & ~64);
 
     if (!rectsamesize(&clipsrc, &clipdst)) {
-        FILE*   pnmscale;
-#define TMPFILE_TEMPLATE "/tmp/picttoppm.XXXXXX"
-        char    tmpfile[sizeof(TMPFILE_TEMPLATE)];
-        int     fd;
-        char    command[1024];
-        register byte* redsrc;
-        register byte* greensrc;
-        register byte* bluesrc;
-        int greenpix;
-        FILE*   scaled;
-        int cols, rows, format;
-        pixval maxval;
-        pixel* row;
-        pixel* rowp;
-
-#if (defined(AMIGA) || defined(VMS))
-                char ami_tmpfile[L_tmpnam];
-                int ami_result;
-                tmpnam(ami_tmpfile);
-                if (!(pnmscale = fopen(ami_tmpfile, "wb")))
-                    pm_error("cannot create temporary file '%s'", ami_tmpfile);
-#else /* AMIGA or VMS */
-        strcpy(tmpfile, TMPFILE_TEMPLATE);
-        fd = mkstemp(tmpfile);
-        if (fd == -1) 
-            pm_error("Unable to create temporary file.");
-        close(fd);
-
-        sprintf(command, "pnmscale -xsize %d -ysize %d > %s",
-            rectwidth(&clipdst), rectheight(&clipdst), tmpfile);
-        
-        pm_message("running 'pnmscale -xsize %d -ysize %d' on a %d x %d image",
-            rectwidth(&clipdst), rectheight(&clipdst),
-            rectwidth(&clipsrc), rectheight(&clipsrc));
-
-        if (!(pnmscale = popen(command, "w"))) {
-            pm_error("cannot execute command '%s'  popen() errno = %s (%d)",
-                     command, strerror(errno), errno);
-        }
-#endif /* AMIGA or VMS */
-
-        fprintf(pnmscale, "P6\n%d %d\n%d\n",
-            rectwidth(&clipsrc), rectheight(&clipsrc), PPM_MAXMAXVAL);
-
-#define REDEPTH(c, oldmax)  ((c) * ((PPM_MAXMAXVAL) + 1) / (oldmax + 1))
-
-        switch (pixSize) {
-        case 8:
-            for (i = 0; i < ysize; ++i) {
-                for (j = 0; j < xsize; ++j) {
-                    ct = colour_map + *src++;
-                    fputc(REDEPTH(ct->red, 65535L), pnmscale);
-                    fputc(REDEPTH(ct->green, 65535L), pnmscale);
-                    fputc(REDEPTH(ct->blue, 65535L), pnmscale);
-                }
-                src += srcadd;
-            }
-            break;
-        case 16:
-            for (i = 0; i < ysize; ++i) {
-                for (j = 0; j < xsize; ++j) {
-                    fputc(REDEPTH((*src & 0x7c) >> 2, 32), pnmscale);
-                    greenpix = (*src++ & 3) << 3;
-                    greenpix |= (*src & 0xe0) >> 5;
-                    fputc(REDEPTH(greenpix, 32), pnmscale);
-                    fputc(REDEPTH((*src++ & 0x1f) << 11, 32), pnmscale);
-                }
-                src += srcadd;
-            }
-            break;
-        case 32:
-            srcadd = srcwid - xsize;
-            redsrc = src;
-            greensrc = src + (srcwid / 3);
-            bluesrc = greensrc + (srcwid / 3);
-
-            for (i = 0; i < ysize; ++i) {
-                for (j = 0; j < xsize; ++j) {
-                    fputc(REDEPTH(*redsrc++, 256), pnmscale);
-                    fputc(REDEPTH(*greensrc++, 256), pnmscale);
-                    fputc(REDEPTH(*bluesrc++, 256), pnmscale);
-                }
-                redsrc += srcadd;
-                greensrc += srcadd;
-                bluesrc += srcadd;
-            }
-        }
-
-#if (defined(AMIGA) || defined(VMS))
-  #ifndef VMS
-  #define AMI_OP != 
-  #else
-  #define AMI_OP ==
-  #endif
-
-        if( fclose(pnmscale) != 0 ) {
-            unlink(ami_tmpfile);
-            pm_error("Error closing file.  fclose() returns "
-                     "errno %s (%d).", strerror(errno), errno);
-        }
-        sprintf(command, "pnmscale -xsize %d -ysize %d %s > %s",
-                rectwidth(&clipdst), rectheight(&clipdst), 
-                ami_tmpfile, tmpfile);
-        pm_message("running 'pnmscale -xsize %d -ysize %d' "
-                   "on a %d x %d image",
-                   rectwidth(&clipdst), rectheight(&clipdst),
-                   rectwidth(&clipsrc), rectheight(&clipsrc), 0);
-        ami_result = system(command);
-        unlink(ami_tmpfile);
-        if( ami_result AMI_OP 0 ) {
-            pm_error("pnmscale failed");
-        }
-#else /* AMIGA or VMS */
-        if (pclose(pnmscale)) {
-            pm_error("pnmscale failed.  pclose() returned Errno %s (%d)",
-                     strerror(errno), errno);
-        }
-#endif /* AMIGA or VMS */
-        ppm_readppminit(scaled = pm_openr(tmpfile), &cols, &rows,
-            &maxval, &format);
-        row = ppm_allocrow(cols);
-        /* couldn't hurt to assert cols, rows and maxval... */  
-
-        if (trf == 0) {
-            while (rows-- > 0) {
-                ppm_readppmrow(scaled, row, cols, maxval, format);
-                for (i = 0, rowp = row; i < cols; ++i, ++rowp) {
-                    *reddst++ = PPM_GETR(*rowp) * 65536L / (maxval + 1); 
-                    *greendst++ = PPM_GETG(*rowp) * 65536L / (maxval + 1); 
-                    *bluedst++ = PPM_GETB(*rowp) * 65536L / (maxval + 1); 
-                }
-                reddst += dstadd;
-                greendst += dstadd;
-                bluedst += dstadd;
-            }
-        }
-        else {
-            while (rows-- > 0) {
-                ppm_readppmrow(scaled, row, cols, maxval, format);
-                for (i = 0, rowp = row; i < cols; i++, rowp++) {
-                    dst_c.red = *reddst;
-                    dst_c.green = *greendst;
-                    dst_c.blue = *bluedst;
-                    src_c.red = PPM_GETR(*rowp) * 65536L / (maxval + 1); 
-                    src_c.green = PPM_GETG(*rowp) * 65536L / (maxval + 1); 
-                    src_c.blue = PPM_GETB(*rowp) * 65536L / (maxval + 1); 
-                    (*trf)(&src_c, &dst_c);
-                    *reddst++ = dst_c.red;
-                    *greendst++ = dst_c.green;
-                    *bluedst++ = dst_c.blue;
-                }
-                reddst += dstadd;
-                greendst += dstadd;
-                bluedst += dstadd;
-            }
-        }
-
-        pm_close(scaled);
-        ppm_freerow(row);
-        unlink(tmpfile);
-        return(1);
+        doDiffSize(clipsrc, clipdst, pixSize, xsize, ysize, ct,
+                   srcadd, dstadd, srcwid, trf, colour_map, src,
+                   reddst, greendst, bluedst);
+        return 1;
     }
 
     if (trf == 0) {
@@ -1700,14 +1713,13 @@ unpackbits(struct Rect* const bounds,
         rowBytes = rowBytesArg & 0x7fff;
     else
         rowBytes = rowBytesArg;
+    if (rowBytes == 0)
+        rowBytes = pixwidth;
 
     stage = "unpacking packbits";
 
     pixwidth = bounds->right - bounds->left;
 
-    if (rowBytes == 0)
-        rowBytes = pixwidth;
-
     pkpixsize = 1;
     if (pixelSize == 16) {
         pkpixsize = 2;
@@ -3187,7 +3199,7 @@ interpret_pict(void) {
             if (verbose)
                 pm_message("reserved 0x%x", opcode);
         }
-        else if (opcode >= 8100 && opcode <= 0xffff) {
+        else if (opcode >= 8100) {
             stage = "skipping reserved";
             if (verbose)
                 pm_message("%s 0x%x", stage, opcode);
