@@ -47,13 +47,13 @@ MasterServer(struct inputSource * const inputSourceP,
              const char *         const outputFileName);
 
 void
-NotifyMasterDone(const char * const masterHostName, 
-                 int          const masterPortNum, 
-                 int          const childNum,
+NotifyMasterDone(const char * const hostName, 
+                 int          const portNum, 
+                 int          const machineNumber, 
                  unsigned int const seconds, 
                  boolean *    const moreWorkToDoP,
-                 int *        const nextFrameStartP,
-                 int *        const nextFrameEndP);
+                 int *        const frameStartP,
+                 int *        const frameEndP);
 
 void
 IoServer(struct inputSource * const inputSourceP,
