@@ -30,11 +30,14 @@
  * HEADER FILES *
  *==============*/
 
+#define _BSD_SOURCE   /* Make sure strdup() is in string.h */
+
 #include "all.h"
 #include <time.h>
 #include <errno.h>
 #include <unistd.h>
 #include <assert.h>
+#include <string.h>
 #ifdef MIPS
 #include <sys/types.h>
 #endif
