@@ -53,11 +53,12 @@
 #include <assert.h>
 #include <sys/types.h>
 #include <sys/socket.h>
-#include <sys/utsname.h>
 
 #include "pm.h"
 #include "nstring.h"
 
+#include "gethostname.h"
+
 #include "psocket.h"
 
 
@@ -79,29 +80,10 @@ typedef unsigned int socklenx_t;
 
 
 
-static const char *
-getHostName(void) {
-/*----------------------------------------------------------------------------
-   Return the host name of this system.
------------------------------------------------------------------------------*/
-    struct utsname utsname;
-    int rc;
-
-    rc = uname(&utsname);
-
-    if (rc < 0)
-        pm_error("Unable to find out host name.  "
-                 "uname() failed with errno %d (%s)", errno, strerror(errno));
-
-    return strdup(utsname.nodename);
-}
-
-
-
 static void GNU_PRINTF_ATTR
 errorExit(const char format[], ...) {
 
-    const char * const hostname = getHostName();
+    const char * const hostname = GetHostName();
 
     va_list args;
 
