@@ -72,27 +72,28 @@ static const char * const printable =
 ASDFGHJKLPIUYTREWQ!~^/()_`'][{}|";
 
 
-struct cmdline_info {
+struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    const char *input_filespec;  /* Filespecs of input files */
+    const char *inputFilename;
     const char *name;
     const char *rgb;
     const char *alpha_filename;
     unsigned int hexonly;
+    unsigned int verbose;
 };
 
 
 
 static void
 parseCommandLine(int argc, char ** argv,
-                 struct cmdline_info * const cmdlineP) {
+                 struct cmdlineInfo * const cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc(100*sizeof(optStruct));
+    optEntry *option_def;
         /* Instructions to OptParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
@@ -101,6 +102,8 @@ parseCommandLine(int argc, char ** argv,
     const char * nameOpt;
     unsigned int nameSpec;
 
+    MALLOCARRAY(option_def, 100);
+
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3(0,   "alphamask",   OPT_STRING, &cmdlineP->alpha_filename, 
             NULL, 0);
@@ -110,6 +113,8 @@ parseCommandLine(int argc, char ** argv,
             NULL, 0);
     OPTENT3(0,   "hexonly",     OPT_FLAG, NULL,
             &cmdlineP->hexonly, 0);
+    OPTENT3(0,   "verbose",     OPT_FLAG, NULL,
+            &cmdlineP->verbose, 0);
 
     /* Set the defaults */
     cmdlineP->alpha_filename = NULL;  /* no transparency */
@@ -123,23 +128,23 @@ parseCommandLine(int argc, char ** argv,
         /* Uses and sets argc, argv, and some of *cmdlineP and others. */
 
     if (argc-1 == 0) 
-        cmdlineP->input_filespec = "-";
+        cmdlineP->inputFilename = "-";
     else if (argc-1 != 1)
         pm_error("Program takes zero or one argument (filename).  You "
                  "specified %d", argc-1);
     else
-        cmdlineP->input_filespec = argv[1];
+        cmdlineP->inputFilename = argv[1];
 
     /* If output filename not specified, use input filename as default. */
     if (nameSpec)
         cmdlineP->name = nameOpt;
-    else if (strcmp(cmdlineP->input_filespec, "-") == 0)
+    else if (strcmp(cmdlineP->inputFilename, "-") == 0)
         cmdlineP->name = "noname";
     else {
         static char name[80+1];
         char *cp;
 
-        STRSCPY(name, cmdlineP->input_filespec);
+        STRSCPY(name, cmdlineP->inputFilename);
         cp = strchr(name, '.');
         if (cp)
             *cp = '\0';     /* remove extension */
@@ -165,19 +170,29 @@ typedef struct {
 
 
 static char *
-genNumstr(int const input, int const digits) {
+genNumstr(unsigned int const input, int const digits) {
 /*---------------------------------------------------------------------------
    Given a number and a base (MAXPRINTABLE), this routine prints the
    number into a malloc'ed string and returns it.  The length of the
-   string is specified by "digits".  The ascii characters of the
-   printed number range from printable[0] to printable[MAXPRINTABLE].
-   The string is printable[0] filled, (e.g. if printable[0]==0,
-   printable[1]==1, MAXPRINTABLE==2, digits==5, i=3, routine would
-   return the malloc'ed string "00011").
+   string is specified by "digits".  It is not printed in decimal or
+   any other number system you're used to.  Rather, each digit is from
+   the set printable[], which contains MAXPRINTABLE characters; the
+   character printable[n] has value n.
+
+   The string is printable[0] filled with high order zeroes.
+
+   Example:
+     Assume: 
+       printable[0] == 'q'
+       printable[1] == '%'
+       MAXPRINTABLE == 2 
+       digits == 5
+       input == 3 
+     Result is the malloc'ed string "qqq%%"
 ---------------------------------------------------------------------------*/
     char *str, *p;
     int d;
-    int i;
+    unsigned int i;
 
     /* Allocate memory for printed number.  Abort if error. */
     if (!(str = (char *) malloc(digits + 1)))
@@ -202,27 +217,77 @@ genNumstr(int const input, int const digits) {
 
 
 
+static unsigned int
+xppMaxvalFromMaxval(pixval const maxval) {
+
+    unsigned int retval;
+
+    /*
+     * Determine how many hex digits we'll be normalizing to if the rgb
+     * value doesn't match a color mnemonic. 
+     */
+    if (maxval <= 0x000F)
+        retval = 0x000F;
+    else if (maxval <= 0x00FF)
+        retval = 0x00FF;
+    else if (maxval <= 0x0FFF)
+        retval = 0x0FFF;
+    else if (maxval <= 0xFFFF)
+        retval = 0xFFFF;
+    else
+        pm_error("Internal error - impossible maxval %x", maxval);
+
+    return retval;
+}
+    
+
+
+static unsigned int
+charsPerPixelForSize(unsigned int const cmapSize) { 
+/*----------------------------------------------------------------------------
+   Return the number of characters it will take to represent a pixel in
+   an XPP that has a colormap of size 'cmapSize'.  Each pixel in an XPP
+   represents an index into the colormap with a base-92 scheme where each
+   character is one of 92 printable characters.  Ergo, if the colormap
+   has fewer than 92 characters, each pixel will be represented by a
+   single character.  If it has more than 92, but fewer than 92*92, it
+   will take 2, etc.
+-----------------------------------------------------------------------------*/
+    unsigned int charsPerPixel;
+    unsigned int j;
+
+    for (charsPerPixel = 0, j = cmapSize; j; charsPerPixel++)
+        j /= MAXPRINTABLE;
+
+    return charsPerPixel;
+} 
+
+
+
 static void
 genCmap(colorhist_vector const chv, 
         int              const ncolors, 
         pixval           const maxval, 
         colorhash_table  const colornameHash,
-        const char *           colornames[],
+        const char *     const colornames[],
+        bool             const includeTransparent,
         cixel_map **     const cmapP, 
-        int              const charsPerPixel) {
+        unsigned int *   const transIndexP,
+        unsigned int *   const cmapSizeP,
+        unsigned int *   const charsPerPixelP) {
 /*----------------------------------------------------------------------------
    Generate the XPM color map in cixel_map format (which is just a step
    away from the actual text that needs to be written the XPM file).  The
    color map is defined by 'chv', which contains 'ncolors' colors which
-   have maxval 'maxval'.
+   have maxval 'maxval'.  Iff 
 
    Output is in newly malloc'ed storage, with its address returned as
-   *cmapP.
+   *cmapP.  We return the number of entries in it as *cmapSizeP.
 
    This map includes an entry for transparency, whether the raster uses
-   it or not.  Its index is ncolors (so cmap's dimension is ncolors +1).
+   it or not.  We return its index as *transIndexP.
 
-   In the map, identify colors by the names give by 'colornameHash' and
+   In the map, identify colors by the names given by 'colornameHash' and
    colornames[].  'colornameHash' maps a color in 'pixel' form to an
    index into colornames[]; colornames[] contains the text to identify the
    color in the XPM format.  The colors in 'colornameHash' have maxval 255.
@@ -231,29 +296,26 @@ genCmap(colorhist_vector const chv,
 
    But if 'colornameHash' is null, don't use color names at all.  Just use
    hexadecimal notation.
+
+   Return as *charsPerPixel the number of characters, or digits, that
+   will be needed in the XPM raster to form an index into this color map.
 -----------------------------------------------------------------------------*/
-    int cmapIndex;
-    int mval;
+    unsigned int const cmapSize = ncolors + (includeTransparent ? 1 : 0);
+
     cixel_map * cmap;  /* malloc'ed */
+    unsigned int cmapIndex;
+    unsigned int charsPerPixel;
+    unsigned int xppMaxval;
     
-    MALLOCARRAY(cmap, ncolors + 1);
+    MALLOCARRAY(cmap, cmapSize);
     if (cmapP == NULL)
-        pm_error("Out of memory allocating %d bytes for a color map.",
+        pm_error("Out of memory allocating %u bytes for a color map.",
                  sizeof(cixel_map) * (ncolors+1));
-    /*
-     * Determine how many hex digits we'll be normalizing to if the rgb
-     * value doesn't match a color mnemonic. 
-     */
-    mval = (unsigned int) maxval;
-    if (mval <= 0x000F)
-        mval = 0x000F;
-    else if (mval <= 0x00FF)
-        mval = 0x00FF;
-    else if (mval <= 0x0FFF)
-        mval = 0x0FFF;
-    else
-        mval = 0xFFFF;
-    
+
+    xppMaxval = xppMaxvalFromMaxval(maxval);
+
+    charsPerPixel = charsPerPixelForSize(cmapSize);
+
     /*
      * Generate the character-pixel string and the rgb name for each
      * colormap entry. 
@@ -292,11 +354,11 @@ genCmap(colorhist_vector const chv,
             pixel scaledColor;
             const char * hexString;  /* malloc'ed */
 
-            PPM_DEPTH(scaledColor, color, maxval, mval);
+            PPM_DEPTH(scaledColor, color, maxval, xppMaxval);
 
-            asprintfN(&hexString, mval == 0x000F ? "#%X%X%X" :
-                      mval == 0x00FF ? "#%02X%02X%02X" :
-                      mval == 0x0FFF ? "#%03X%03X%03X" :
+            asprintfN(&hexString, xppMaxval == 0x000F ? "#%X%X%X" :
+                      xppMaxval == 0x00FF ? "#%02X%02X%02X" :
+                      xppMaxval == 0x0FFF ? "#%03X%03X%03X" :
                       "#%04X%04X%04X", 
                       PPM_GETR(scaledColor),
                       PPM_GETG(scaledColor),
@@ -309,58 +371,69 @@ genCmap(colorhist_vector const chv,
         }
     }
 
-    /* Add the special transparency entry to the colormap */
-    cmap[ncolors].rgbname = strdup("None");
-    cmap[ncolors].cixel = genNumstr(ncolors, charsPerPixel);
-
-    *cmapP = cmap;
+    if (includeTransparent) {
+        /* Add the special transparency entry to the colormap */
+        unsigned int const transIndex = ncolors;
+        cmap[transIndex].rgbname = strdup("None");
+        cmap[transIndex].cixel = genNumstr(transIndex, charsPerPixel);
+        *transIndexP = transIndex;
+    }
+    *cmapP          = cmap;
+    *cmapSizeP      = cmapSize;
+    *charsPerPixelP = charsPerPixel;
 }
 
 
 
 static void
-destroyCmap(cixel_map * const cmap, int const ncolors) {
+destroyCmap(cixel_map *  const cmap, 
+            unsigned int const cmapSize) {
 
     int i;
     /* Free the real color entries */
-    for (i = 0; i < ncolors; i++) {
+    for (i = 0; i < cmapSize; i++) {
         strfree(cmap[i].rgbname);
         free(cmap[i].cixel);
     }
-    strfree(cmap[ncolors].rgbname);  /* The transparency entry */
-    free(cmap[ncolors].cixel);  /* The transparency entry */
-
     free(cmap);
 }
 
 
 
 static void
-writeXpmFile(FILE * const outfile, pixel ** const pixels, 
-             gray ** const alpha, pixval const alphamaxval,
-             const char name[], 
-             int const cols, int const rows, int const colormapsize,
-             int charsPerPixel, 
-             cixel_map cmap[],
-             colorhash_table const cht) {
+writeXpmFile(FILE *          const outfile, 
+             pixel **        const pixels, 
+             gray **         const alpha, 
+             pixval          const alphamaxval,
+             char            const name[], 
+             int             const cols, 
+             int             const rows, 
+             unsigned int    const cmapSize,
+             unsigned int    const charsPerPixel, 
+             cixel_map       const cmap[],
+             colorhash_table const cht,
+             unsigned int    const transIndex) {
 /*----------------------------------------------------------------------------
    Write the whole XPM file to the open stream 'outfile'.
 
-   colormapsize is the total number of entries in the colormap -- one for
-   each color, plus one for transparency.
+   'cmap' is the colormap to be placed in the XPM.  'cmapSize' is the
+   number of entries in it.  'cht' is a hash table that gives you an 
+   index into 'cmap' given a color.  'transIndex' is the index into cmap
+   of the transparent color, and is valid only if 'alpha' is non-null
+   (otherwise, cmap might not contain a transparent color).
 -----------------------------------------------------------------------------*/
     /* First the header */
     printf("/* XPM */\n");
     fprintf(outfile, "static char *%s[] = {\n", name);
     fprintf(outfile, "/* width height ncolors chars_per_pixel */\n");
     fprintf(outfile, "\"%d %d %d %d\",\n", cols, rows, 
-            colormapsize, charsPerPixel);
+            cmapSize, charsPerPixel);
 
     {
         int i;
         /* Write out the colormap (part of header) */
         fprintf(outfile, "/* colors */\n");
-        for (i = 0; i < colormapsize; i++) { 
+        for (i = 0; i < cmapSize; i++) { 
             fprintf(outfile, "\"%s c %s\",\n", cmap[i].cixel, cmap[i].rgbname);
         }
     }
@@ -375,7 +448,7 @@ writeXpmFile(FILE * const outfile, pixel ** const pixels,
             for (col = 0; col < cols; col++) {
                 if (alpha && alpha[row][col] <= alphamaxval/2)
                     /* It's a transparent pixel */
-                    fprintf(outfile, "%s", cmap[colormapsize-1].cixel);
+                    fprintf(outfile, "%s", cmap[transIndex].cixel);
                 else 
                     fprintf(outfile, "%s", 
                             cmap[ppm_lookupcolor(cht, 
@@ -408,31 +481,98 @@ readAlpha(const char filespec[], gray *** const alphaP,
 }
     
 
+
 static void
-computeColormap(pixel ** const pixels, int const cols, int const rows,
+computecolorhash(pixel **          const pixels,
+                 gray **           const alpha,
+                 int               const cols,
+                 int               const rows,
+                 gray              const alphaMaxval,
+                 colorhash_table * const chtP,
+                 unsigned int *    const ncolorsP,
+                 bool *            const transparentSomewhereP) {
+/*----------------------------------------------------------------------------
+   Compute a colorhash_table with one entry for each color in 'pixels' that
+   is not mostly transparent according to alpha mask 'alpha' (which has
+   maxval 'alphaMaxval').  alpha == NULL means all pixels are opaque.
+
+   The value associated with the color in the hash we build is meaningless.
+
+   Return the colorhash_table as *chtP, and the number of colors in it
+   as *ncolorsP.  Return *transparentSomewhereP == TRUE iff the image has
+   at least one pixel that is mostly transparent.
+-----------------------------------------------------------------------------*/
+    colorhash_table cht;
+    int row;
+    
+    cht = ppm_alloccolorhash( );
+    *ncolorsP = 0;   /* initial value */
+    *transparentSomewhereP = FALSE;  /* initial assumption */
+
+    /* Go through the entire image, building a hash table of colors. */
+    for (row = 0; row < rows; ++row) {
+        int col;
+
+        for (col = 0; col < cols; ++col) {
+            if (!alpha || alpha[row][col] > alphaMaxval/2) {
+                /* It's mostly opaque, so add this color to the hash
+                   if it's not already there.  
+                */
+                pixel const color = pixels[row][col];
+                int const lookupRc = ppm_lookupcolor(cht, &color);
+ 
+                if (lookupRc < 0) {
+                    /* It's not in the hash yet, so add it */
+                    ppm_addtocolorhash(cht, &color, 0);
+                    ++(*ncolorsP);
+                }
+            } else
+                *transparentSomewhereP = TRUE;
+        }
+    }
+    *chtP = cht;
+}
+
+
+
+static void
+computeColormap(pixel **           const pixels, 
+                gray **            const alpha,
+                int                const cols, 
+                int                const rows,
+                gray               const alphaMaxval,
                 colorhist_vector * const chvP, 
-                colorhash_table * const chtP,
-                int * const ncolorsP) {
+                colorhash_table *  const chtP,
+                unsigned int *     const ncolorsP,
+                bool *             const transparentSomewhereP) {
 /*----------------------------------------------------------------------------
    Compute the color map for the image 'pixels', which is 'cols' by 'rows',
    in Netpbm data structures (a colorhist_vector for index-to-color lookups
    and a colorhash_table for color-to-index lookups).
+
+   Exclude pixels that alpha mask 'alpha' (which has maxval
+   'alphaMaxval') says are mostly transparent.  alpha == NULL means all
+   pixels are opaque.
+
+   We return as *chvP an array of the colors present in 'pixels',
+   excluding those that are mostly transparent.  We return as
+   *ncolorsP the number of such colors.  We return
+   *transparentSomewhereP == TRUE iff the image has at least one
+   pixel that is mostly transparent.
 -----------------------------------------------------------------------------*/
-    colorhash_table hist_cht;
+    colorhash_table histCht;
 
     pm_message("(Computing colormap...");
-    hist_cht = ppm_computecolorhash(pixels, cols, rows, 0, ncolorsP);
-    /* Old versions of the library don't have the special 0 value for 
-       maxcolors -- they take it literally.
+    computecolorhash(pixels, alpha, cols, rows, alphaMaxval, 
+                     &histCht, ncolorsP, transparentSomewhereP);
+    pm_message("...Done.  %d colors found.)", *ncolorsP);
+    
+    *chvP = ppm_colorhashtocolorhist(histCht, *ncolorsP);
+    ppm_freecolorhash(histCht);
+    /* Despite the name, the following generates an index on *chvP,
+       with which given a color you can quickly find the entry number
+       in *chvP that contains that color.
     */
-    if (hist_cht == NULL)
-        pm_error("You need a newer version of the Netpbm library 'libppm' "
-                 "to run this program.");
-    pm_message("...Done.  %d colors found.)\n", *ncolorsP);
-
-    *chvP = ppm_colorhashtocolorhist(hist_cht, *ncolorsP);
-    ppm_freecolorhash(hist_cht);
-    /* Despite the name, the following generates an index on cht */
     *chtP = ppm_colorhisttocolorhash(*chvP, *ncolorsP);
 }
 
@@ -442,8 +582,10 @@ int
 main(int argc, char *argv[]) {
 
     FILE *ifp;
-    int rows, cols, ncolors;
-    pixval maxval, alphamaxval;
+    int rows, cols;
+    unsigned int ncolors;
+    bool transparentSomewhere;
+    pixval maxval, alphaMaxval;
     colorhash_table cht;
     colorhist_vector chv;
 
@@ -459,51 +601,54 @@ main(int argc, char *argv[]) {
 
     /* Used for rgb value -> character-pixel string mapping */
     cixel_map *cmap;  /* malloc'ed */
-    int charsPerPixel;  
+        /* The XPM colormap */
+    unsigned int cmapSize;
+        /* Number of entries in 'cmap' */
+    unsigned int transIndex;
+        /* Index into 'cmap' of the transparent color, if there is one */
 
-    struct cmdline_info cmdline;
+    unsigned int charsPerPixel;  
+
+    struct cmdlineInfo cmdline;
 
     ppm_init(&argc, argv);
 
     parseCommandLine(argc, argv, &cmdline);
 
-    ifp = pm_openr(cmdline.input_filespec);
+    ifp = pm_openr(cmdline.inputFilename);
     pixels = ppm_readppm(ifp, &cols, &rows, &maxval);
     pm_close(ifp);
 
     if (cmdline.alpha_filename) 
-        readAlpha(cmdline.alpha_filename, &alpha, cols, rows, &alphamaxval);
+        readAlpha(cmdline.alpha_filename, &alpha, cols, rows, &alphaMaxval);
     else
         alpha = NULL;
 
-    computeColormap(pixels, cols, rows, &chv, &cht, &ncolors);
+    computeColormap(pixels, alpha, cols, rows, alphaMaxval,
+                    &chv, &cht, &ncolors, &transparentSomewhere);
 
     if (cmdline.hexonly)
         colornameHash = NULL;
     else
         ppm_readcolornamefile(cmdline.rgb, FALSE, &colornameHash, &colornames);
 
-    { 
-        /* Compute characters per pixel in the XPM file */
-        int j;
-        for (charsPerPixel = 0, j = ncolors-1; j > 0; ++charsPerPixel)
-            j /= MAXPRINTABLE;
-    }
     /* Now generate the character-pixel colormap table. */
-    genCmap(chv, ncolors, maxval, colornameHash, colornames,
-            &cmap, charsPerPixel);
+    genCmap(chv, ncolors, maxval, 
+            colornameHash, colornames, transparentSomewhere, 
+            &cmap, &transIndex, &cmapSize, &charsPerPixel);
 
-    writeXpmFile(stdout, pixels, alpha, alphamaxval,
-                 cmdline.name, cols, rows, ncolors + 1, 
-                 charsPerPixel, cmap, cht);
+    writeXpmFile(stdout, pixels, alpha, alphaMaxval,
+                 cmdline.name, cols, rows, cmapSize,
+                 charsPerPixel, cmap, cht, transIndex);
     
     if (colornameHash) {
         ppm_freecolorhash(colornameHash);
         ppm_freecolornames(colornames);
     }
-    destroyCmap(cmap, ncolors);
+    destroyCmap(cmap, cmapSize);
     ppm_freearray(pixels, rows);
     if (alpha) pgm_freearray(alpha, rows);
 
     return 0;
 }
+
