@@ -18,27 +18,20 @@
 #include "xim.h"
 #include "shhopt.h"
 
-static struct cmdline_info {
+struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    char *input_filename;
-    char *alpha_filename;
+    const char *input_filename;
+    const char *alpha_filename;
     bool alpha_stdout;
-} cmdline;
-
-
-static int ReadXim ARGS(( FILE *in_fp, XimImage *xim ));
-static int ReadXimHeader ARGS(( FILE *in_fp, XimImage *header ));
-static int ReadXimImage ARGS(( FILE *in_fp, XimImage *xim ));
-static int ReadImageChannel ARGS(( FILE *infp, byte *buf, unsigned int *bufsize, int encoded ));
-
+};
 
 
 
 static void
-parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdlineP) {
+parseCommandLine(int argc, char ** argv,
+                 struct cmdlineInfo *cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that many of the strings that this function returns in the
    *cmdlineP structure are actually in the supplied argv array.  And
@@ -63,7 +56,7 @@ parse_command_line(int argc, char ** argv,
         cmdlineP->alpha_filename = NULL;
 
     if (argc - 1 == 0)
-        cmdlineP->input_filename = NULL;  /* he wants stdin */
+        cmdlineP->input_filename = "-";  /* he wants stdin */
     else if (argc - 1 == 1)
         cmdlineP->input_filename = strdup(argv[1]);
     else 
@@ -79,187 +72,21 @@ parse_command_line(int argc, char ** argv,
 
 
 
-int
-main( argc, argv )
-    int argc;
-    char *argv[];
-    {
-    FILE *ifp, *imageout_file, *alpha_file;
-    XimImage xim;
-    pixel *pixelrow, colormap[256];
-    gray *alpharow;
-       /* The alpha channel of the row we're currently converting, in pgm fmt*/
-    register pixel *pP;
-    gray *aP;
-    int rows, cols, row, mapped;
-    register int col;
-    pixval maxval;
-
-
-    ppm_init( &argc, argv );
-
-    parse_command_line(argc, argv, &cmdline);
-
-    if ( cmdline.input_filename != NULL ) 
-        ifp = pm_openr( cmdline.input_filename );
-    else
-        ifp = stdin;
-
-    if (cmdline.alpha_stdout)
-        alpha_file = stdout;
-    else if (cmdline.alpha_filename == NULL) 
-        alpha_file = NULL;
-    else {
-        alpha_file = pm_openw(cmdline.alpha_filename);
-    }
-
-    if (cmdline.alpha_stdout) 
-        imageout_file = NULL;
-    else
-        imageout_file = stdout;
-
-
-    if ( ! ReadXim( ifp, &xim ) )
-	pm_error( "can't read Xim file" );
-    rows = xim.height;
-    cols = xim.width;
-    if ( xim.nchannels == 1 && xim.bits_channel == 8 )
-	{
-	int i;
-
-	mapped = 1;
-	maxval = 255;
-	for ( i = 0; i < xim.ncolors; ++i )
-	    {
-	    PPM_ASSIGN(
-		colormap[i], xim.colors[i].red,
-		xim.colors[i].grn, xim.colors[i].blu );
-	    /* Should be colormap[xim.colors[i].pixel], but Xim is broken. */
-	    }
-	}
-    else if ( xim.nchannels == 3 || xim.nchannels == 4 )
-	{
-	mapped = 0;
-	maxval = pm_bitstomaxval( xim.bits_channel );
-	}
-    else
-	pm_error(
-	    "unknown Xim file type, nchannels == %d, bits_channel == %d",
-	    xim.nchannels, xim.bits_channel );
-
-    if (imageout_file) 
-        ppm_writeppminit( imageout_file, cols, rows, maxval, 0 );
-    if (alpha_file)
-        pgm_writepgminit( alpha_file, cols, rows, maxval, 0 );
-
-    pixelrow = ppm_allocrow( cols );
-    alpharow = pgm_allocrow( cols );
-
-    for ( row = 0; row < rows; ++row )
-	{
-	if ( mapped )
-	    {
-	    register byte *xbp;
-
-	    for ( col = 0, pP = pixelrow,
-		      xbp = xim.data + row * xim.bytes_per_line;
-              col < cols;
-              ++col, ++pP, ++xbp )
-            *pP = colormap[*xbp];
-            *aP = 0;
-	    }
-	else
-	    {
-	    register byte *xrbp, *xgbp, *xbbp;
-
-	    for ( col = 0, pP = pixelrow, aP = alpharow,
-		      xrbp = xim.data + row * xim.bytes_per_line,
-		      xgbp = xim.grn_data + row * xim.bytes_per_line,
-		      xbbp = xim.blu_data + row * xim.bytes_per_line;
-              col < cols;
-              ++col, ++pP, ++xrbp, ++xgbp, ++xbbp ) {
-            PPM_ASSIGN( *pP, *xrbp, *xgbp, *xbbp );
-            if (xim.nchannels > 3)
-                *aP = *(xim.other + row * xim.bytes_per_line + col);
-            else 
-                *aP = 0;
-        }
-	    }
-    if (imageout_file) 
-        ppm_writeppmrow( imageout_file, pixelrow, cols, maxval, 0 );
-    if (alpha_file)
-        pgm_writepgmrow( alpha_file, alpharow, cols, maxval, 0 );
-	}
-    pm_close( ifp );
-    if (imageout_file)
-        pm_close( imageout_file );
-    if (alpha_file)
-        pm_close( alpha_file );
-    exit( 0 );
-    }
-
-/* The rest is excerpted and slightly modified from the X.V11R4 version
-** of xim_io.c.
+/* The subroutines are excerpted and slightly modified from the
+   X.V11R4 version of xim_io.c. 
 */
 
-/***********************************************************************
-*  File:   xlib.c
-*  Author: Philip Thompson
-*  $Date: 89/11/01 10:14:23 $
-*  $Revision: 1.14 $
-*  Purpose: General xim libray of utililities
-*  Copyright (c) 1988  Philip R. Thompson
-*                Computer Resource Laboratory (CRL)
-*                Dept. of Architecture and Planning
-*                M.I.T., Rm 9-526
-*                Cambridge, MA  02139
-*   This  software and its documentation may be used, copied, modified,
-*   and distributed for any purpose without fee, provided:
-*       --  The above copyright notice appears in all copies.
-*       --  This disclaimer appears in all source code copies.
-*       --  The names of M.I.T. and the CRL are not used in advertising
-*           or publicity pertaining to distribution of the software
-*           without prior specific written permission from me or CRL.
-*   I provide this software freely as a public service.  It is NOT a
-*   commercial product, and therefore is not subject to an an implied
-*   warranty of merchantability or fitness for a particular purpose.  I
-*   provide it as is, without warranty.
-*   This software is furnished  only on the basis that any party who
-*   receives it indemnifies and holds harmless the parties who furnish
-*   it against any claims, demands, or liabilities connected with using
-*   it, furnishing it to others, or providing it to a third party.
-*
-*   Philip R. Thompson (phils@athena.mit.edu)
-***********************************************************************/
-
 static int
-ReadXim(in_fp, xim)
-    FILE *in_fp;
-    XimImage *xim;
-{
-    if (!ReadXimHeader(in_fp, xim)) {
-        pm_message("can't read xim header" );
-	return(0);
-    }
-    if (!ReadXimImage(in_fp, xim)) {
-        pm_message("can't read xim data" );
-	return(0);
-    }
-    return(1);
-}
+ReadXimHeader(FILE *     const in_fp,
+              XimImage * const header) {
 
-static int
-ReadXimHeader(in_fp, header)
-    FILE *in_fp;
-    XimImage  *header;
-{
     int  i;
     char *cp;
     XimAsciiHeader  a_head;
 
     cp = (char *) header;
     for (i = 0; i < sizeof(XimImage); ++i )
-	*cp++ = 0;
+    *cp++ = 0;
     /* Read header and verify image file formats */
     if (fread((char *)&a_head, sizeof(ImageHeader), 1, in_fp) != 1) {
         pm_message("ReadXimHeader: unable to read file header" );
@@ -341,11 +168,60 @@ ReadXimHeader(in_fp, header)
     return(1);
 }
 
+
+
 static int
-ReadXimImage(in_fp, xim)
-    FILE *in_fp;
-    XimImage *xim;
-{
+ReadImageChannel(FILE *         const infp,
+                 byte *         const buf,
+                 unsigned int * const bufsize,
+                 int            const encoded) {
+
+    int  i, runlen, nbytes;
+    unsigned int  j;
+    byte *line;
+    long  marker;
+
+    if (!encoded)
+        j = fread((char *)buf, 1, (int)*bufsize, infp);
+    else {
+        if ((line=(byte *)malloc((unsigned int)BUFSIZ)) == NULL) {
+            pm_message("ReadImageChannel: can't malloc() fread string" );
+            return(0);
+        }
+        /* Unrunlength encode data */
+        marker = ftell(infp);
+        j = 0;
+        while (((nbytes=fread((char *)line, 1, BUFSIZ, infp)) > 0) &&
+            (j < *bufsize)) {
+            for (i=0; (i < nbytes) && (j < *bufsize); i++) {
+                runlen = (int)line[i]+1;
+                i++;
+                while (runlen--)
+                    buf[j++] = line[i];
+            }
+            marker += i;
+        }
+        /* return to the begining of the next image's bufffer */
+        if (fseek(infp, marker, 0) == -1) {
+            pm_message("ReadImageChannel: can't fseek to location in image buffer" );
+            return(0);
+        }
+        free((char *)line);
+    }
+    if (j != *bufsize) {
+        pm_message("unable to complete channel: %u / %u (%d%%)",
+            j, *bufsize, (int)(j*100.0 / *bufsize) );
+        *bufsize = j;
+    }
+    return(1);
+}
+
+
+
+static int
+ReadXimImage(FILE *     const in_fp,
+             XimImage * const xim) {
+
     if (xim->data) {
         free((char *)xim->data);
         xim->data = (byte *)0;
@@ -403,49 +279,159 @@ ReadXimImage(in_fp, xim)
     return(1);
 }
 
-static int
-ReadImageChannel(infp, buf, bufsize, encoded)
-FILE *infp;
-    byte  *buf;
-    unsigned int  *bufsize;
-    int  encoded;
-{
-    register int  i, runlen, nbytes;
-    register unsigned int  j;
-    register byte *line;
-    long  marker;
 
-    if (!encoded)
-        j = fread((char *)buf, 1, (int)*bufsize, infp);
-    else {
-        if ((line=(byte *)malloc((unsigned int)BUFSIZ)) == NULL) {
-            pm_message("ReadImageChannel: can't malloc() fread string" );
-            return(0);
-        }
-        /* Unrunlength encode data */
-        marker = ftell(infp);
-        j = 0;
-        while (((nbytes=fread((char *)line, 1, BUFSIZ, infp)) > 0) &&
-            (j < *bufsize)) {
-            for (i=0; (i < nbytes) && (j < *bufsize); i++) {
-                runlen = (int)line[i]+1;
-                i++;
-                while (runlen--)
-                    buf[j++] = line[i];
-            }
-            marker += i;
-        }
-        /* return to the begining of the next image's bufffer */
-        if (fseek(infp, marker, 0) == -1) {
-            pm_message("ReadImageChannel: can't fseek to location in image buffer" );
-            return(0);
-        }
-        free((char *)line);
+
+/***********************************************************************
+*  File:   xlib.c
+*  Author: Philip Thompson
+*  $Date: 89/11/01 10:14:23 $
+*  $Revision: 1.14 $
+*  Purpose: General xim libray of utililities
+*  Copyright (c) 1988  Philip R. Thompson
+*                Computer Resource Laboratory (CRL)
+*                Dept. of Architecture and Planning
+*                M.I.T., Rm 9-526
+*                Cambridge, MA  02139
+*   This  software and its documentation may be used, copied, modified,
+*   and distributed for any purpose without fee, provided:
+*       --  The above copyright notice appears in all copies.
+*       --  This disclaimer appears in all source code copies.
+*       --  The names of M.I.T. and the CRL are not used in advertising
+*           or publicity pertaining to distribution of the software
+*           without prior specific written permission from me or CRL.
+*   I provide this software freely as a public service.  It is NOT a
+*   commercial product, and therefore is not subject to an an implied
+*   warranty of merchantability or fitness for a particular purpose.  I
+*   provide it as is, without warranty.
+*   This software is furnished  only on the basis that any party who
+*   receives it indemnifies and holds harmless the parties who furnish
+*   it against any claims, demands, or liabilities connected with using
+*   it, furnishing it to others, or providing it to a third party.
+*
+*   Philip R. Thompson (phils@athena.mit.edu)
+***********************************************************************/
+
+static int
+ReadXim(in_fp, xim)
+    FILE *in_fp;
+    XimImage *xim;
+{
+    if (!ReadXimHeader(in_fp, xim)) {
+        pm_message("can't read xim header" );
+    return(0);
     }
-    if (j != *bufsize) {
-        pm_message("unable to complete channel: %u / %u (%d%%)",
-            j, *bufsize, (int)(j*100.0 / *bufsize) );
-        *bufsize = j;
+    if (!ReadXimImage(in_fp, xim)) {
+        pm_message("can't read xim data" );
+    return(0);
     }
     return(1);
 }
+
+
+
+int
+main(int argc,
+     char *argv[]) {
+
+    struct cmdlineInfo cmdline;
+    FILE *ifP, *imageout_file, *alpha_file;
+    XimImage xim;
+    pixel *pixelrow, colormap[256];
+    gray *alpharow;
+        /* The alpha channel of the row we're currently converting, in
+           pgm fmt
+        */
+    int rows, cols, row, mapped;
+    pixval maxval;
+    bool success;
+
+    ppm_init(&argc, argv);
+
+    parseCommandLine(argc, argv, &cmdline);
+    
+    ifP = pm_openr(cmdline.input_filename);
+    
+    if (cmdline.alpha_stdout)
+        alpha_file = stdout;
+    else if (cmdline.alpha_filename == NULL) 
+        alpha_file = NULL;
+    else
+        alpha_file = pm_openw(cmdline.alpha_filename);
+    
+    if (cmdline.alpha_stdout) 
+        imageout_file = NULL;
+    else
+        imageout_file = stdout;
+    
+    success = ReadXim(ifP, &xim);
+    if (!success)
+        pm_error("can't read Xim file");
+
+    rows = xim.height;
+    cols = xim.width;
+
+    if (xim.nchannels == 1 && xim.bits_channel == 8) {
+        int i;
+
+        mapped = true;
+        maxval = 255;
+        for (i = 0; i < xim.ncolors; ++i) {
+            PPM_ASSIGN(
+                colormap[i], xim.colors[i].red,
+                xim.colors[i].grn, xim.colors[i].blu );
+            /* Should be colormap[xim.colors[i].pixel], but Xim is broken. */
+        }
+    } else if (xim.nchannels == 3 || xim.nchannels == 4) {
+        mapped = false;
+        maxval = pm_bitstomaxval(xim.bits_channel);
+    } else
+        pm_error(
+            "unknown Xim file type, nchannels == %d, bits_channel == %d",
+            xim.nchannels, xim.bits_channel);
+
+    if (imageout_file) 
+        ppm_writeppminit(imageout_file, cols, rows, maxval, 0);
+    if (alpha_file)
+        pgm_writepgminit(alpha_file, cols, rows, maxval, 0);
+
+    pixelrow = ppm_allocrow(cols);
+    alpharow = pgm_allocrow(cols);
+
+    for (row = 0; row < rows; ++row) {
+        if (mapped) {
+            byte * const ximrow = xim.data + row * xim.bytes_per_line;
+            unsigned int col;
+            
+            for (col = 0; col < cols; ++col) 
+                pixelrow[col] = colormap[ximrow[col]];
+            alpharow[col] = 0;
+        } else {
+            byte * const redrow = xim.data     + row * xim.bytes_per_line;
+            byte * const grnrow = xim.grn_data + row * xim.bytes_per_line;
+            byte * const blurow = xim.blu_data + row * xim.bytes_per_line;
+            byte * const othrow = xim.other    + row * xim.bytes_per_line;
+
+            unsigned int col;
+
+            for (col = 0; col < cols; ++col) {
+                PPM_ASSIGN(pixelrow[col],
+                           redrow[col], grnrow[col], blurow[col]);
+                if (xim.nchannels > 3)
+                    alpharow[col] = othrow[col];
+                else 
+                    alpharow[col] = 0;
+            }
+        }
+        if (imageout_file) 
+            ppm_writeppmrow(imageout_file, pixelrow, cols, maxval, 0);
+        if (alpha_file)
+            pgm_writepgmrow(alpha_file, alpharow, cols, maxval, 0);
+    }
+    pm_close(ifP);
+    if (imageout_file)
+        pm_close(imageout_file);
+    if (alpha_file)
+        pm_close(alpha_file);
+
+    return 0;
+}
