@@ -516,7 +516,7 @@ readXpm1Header(FILE * const stream, int * const widthP, int * const heightP,
     if (*chars_per_pixelP == -1)
         pm_error("missing or invalid *chars_per_pixelP");
     if (*chars_per_pixelP > 2)
-        pm_message("warning, *chars_per_pixelP > 2 uses a lot of memory");
+        pm_message("WARNING: *chars_per_pixelP > 2 uses a lot of memory");
 
     /* If there's a monochrome color table, skip it. */
     if (!strncmp(t1, "mono", 4)) {
@@ -589,9 +589,12 @@ readXpm1Header(FILE * const stream, int * const widthP, int * const heightP,
 
 
 static void
-interpretXpmLine(const char line[], int const chars_per_pixel,
-                 int const ncolors, int * const ptab, 
-                 int ** const cursorP, int * const maxCursor) {
+interpretXpmLine(char   const line[],
+                 int    const chars_per_pixel,
+                 int    const ncolors,
+                 int *  const ptab, 
+                 int ** const cursorP,
+                 int *  const maxCursor) {
 /*----------------------------------------------------------------------------
    Interpret one line of XPM input.  The line is in 'line', and its
    format is 'chars_per_pixel' characters per pixel.  'ptab' is the
@@ -602,38 +605,46 @@ interpretXpmLine(const char line[], int const chars_per_pixel,
    *cursorP, lined up in the order they are in 'line', and return
    *cursorP positioned just after the last one.
 
+   If the line doesn't start with a quote (e.g. it is empty), we issue
+   a warning and just treat the line as one that describes no pixels.
+
    Stop processing the line either at the end of the line or when
    the output would go beyond maxCursor, whichever comes first.
 -----------------------------------------------------------------------------*/
     char *lineCursor;
 
     lineCursor = strchr(line, '"');  /* position to 1st quote in line */
-    if (lineCursor == NULL)
-        pm_error("No opening quotation mark in XPM input line which "
-                 "is supposed to be a line of raster data: '%s'", line);
+    if (lineCursor == NULL) {
+        /* We've seen a purported XPM that had a blank line in it.  Just
+           ignoring it was the right thing to do.  05.05.27.
+        */
+        pm_message("WARNING:  No opening quotation mark in XPM input "
+                   "line which is supposed to be a line of raster data: "
+                   "'%s'.  Ignoring this line.", line);
+    } else {
+        ++lineCursor; /* Skip to first character after quote */
 
-    ++lineCursor; /* Skip to first character after quote */
-
-    /* Handle pixels until a close quote, eol, or the end of the image. */
-    while (*lineCursor && *lineCursor != '"' && *cursorP <= maxCursor) {
-        int colorNumber;
-        int i;
-        colorNumber = 0;  /* initial value */
-        for (i = 0; i < chars_per_pixel; ++i)
-            colorNumber = (colorNumber << 8) + *(lineCursor++);
-        if (ptab == NULL)
-            /* colormap is indexed directly by XPM color number */
-            *(*cursorP)++ = colorNumber;
-        else {
-            /* colormap shadows ptab[].  Find this color number in ptab[] */
+        /* Handle pixels until a close quote, eol, or the end of the image. */
+        while (*lineCursor && *lineCursor != '"' && *cursorP <= maxCursor) {
+            int colorNumber;
             int i;
-            for (i = 0; i < ncolors && ptab[i] != colorNumber; ++i);
-            if (i < ncolors)
-                *(*cursorP)++ = i;
-            else
-                pm_error("Color number %d is in raster, but not in "
-                         "colormap.  Line it's in: '%s'",
-                         colorNumber, line);
+            colorNumber = 0;  /* initial value */
+            for (i = 0; i < chars_per_pixel; ++i)
+                colorNumber = (colorNumber << 8) + *(lineCursor++);
+            if (ptab == NULL)
+                /* colormap is indexed directly by XPM color number */
+                *(*cursorP)++ = colorNumber;
+            else {
+                /* colormap shadows ptab[].  Find this color # in ptab[] */
+                int i;
+                for (i = 0; i < ncolors && ptab[i] != colorNumber; ++i);
+                if (i < ncolors)
+                    *(*cursorP)++ = i;
+                else
+                    pm_error("Color number %d is in raster, but not in "
+                             "colormap.  Line it's in: '%s'",
+                             colorNumber, line);
+            }
         }
     }
 }
