@@ -12,6 +12,7 @@
 
 #include <assert.h>
 #include <string.h>
+
 #include "pam.h"
 #include "dithers.h"
 #include "mallocvar.h"
@@ -144,7 +145,6 @@ parseCommandLine(int argc, char ** argv,
 }
 
 
-
 static struct pam
 makeOutputPam(unsigned int const width,
               unsigned int const height) {
@@ -331,7 +331,7 @@ doHilbert(FILE *       const ifP,
     int *x,*y;
     int sum;
 
-    grays = pnm_readpam(ifP, &graypam, PAM_STRUCT_SIZE(tuple_type));
+    grays = pnm_readpam(ifP, &graypam, sizeof(graypam));
 
     bitpam = makeOutputPam(graypam.width, graypam.height);
 
@@ -570,13 +570,13 @@ clusterConvertRow(struct converter * const converterP,
                   tuple                    bitrow[]) {
 
     struct clusterState * const stateP = converterP->stateP;
-    unsigned int const diameter = 2 * stateP->radius;
+    unsigned int const radius = stateP->radius;
 
     unsigned int col;
 
     for (col = 0; col < converterP->cols; ++col) {
         float const threshold = 
-            stateP->clusterMatrix[row % diameter][col % diameter];
+            stateP->clusterMatrix[row % radius][col % radius];
         bitrow[col][0] = 
             grayrow[col][0] > threshold ? PAM_BW_WHITE : PAM_BLACK;
     }
@@ -588,11 +588,10 @@ static void
 clusterDestroy(struct converter * const converterP) {
 
     struct clusterState * const stateP = converterP->stateP;
-    unsigned int const diameter = 2 * stateP->radius;
 
     unsigned int row;
 
-    for (row = 0; row < diameter; ++row)
+    for (row = 0; row < stateP->radius; ++row)
         free(stateP->clusterMatrix[row]);
 
     free(stateP->clusterMatrix);
@@ -615,7 +614,6 @@ createClusterConverter(struct pam *    const graypamP,
     */
     
     int const clusterNormalizer = radius * radius * 2;
-    unsigned int const diameter = 2 * radius;
 
     struct converter converter;
     struct clusterState * stateP;
@@ -629,13 +627,13 @@ createClusterConverter(struct pam *    const graypamP,
 
     stateP->radius = radius;
 
-    MALLOCARRAY_NOFAIL(stateP->clusterMatrix, diameter);
-    for (row = 0; row < diameter; ++row) {
+    MALLOCARRAY_NOFAIL(stateP->clusterMatrix, radius);
+    for (row = 0; row < radius; ++row) {
         unsigned int col;
 
-        MALLOCARRAY_NOFAIL(stateP->clusterMatrix[row], diameter);
+        MALLOCARRAY_NOFAIL(stateP->clusterMatrix[row], radius);
         
-        for (col = 0; col < diameter; ++col) {
+        for (col = 0; col < radius; ++col) {
             switch (ditherType) {
             case DT_REGULAR: 
                 switch (radius) {
@@ -693,7 +691,7 @@ main(int argc, char *argv[]) {
         tuple * bitrow;
         int row;
 
-        pnm_readpaminit(ifP, &graypam, PAM_STRUCT_SIZE(tuple_type));
+        pnm_readpaminit(ifP, &graypam, sizeof(graypam));
 
         bitpam = makeOutputPam(graypam.width, graypam.height);
         
