@@ -18,13 +18,14 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#include "pam.h"
-#include "shhopt.h"
-#include "mallocvar.h"
+#define _BSD_SOURCE   /* Make sure strdup is int string.h */
 
 #include <math.h>
 #include <string.h>
 
+#include "pam.h"
+#include "shhopt.h"
+#include "mallocvar.h"
 
 typedef double number;
 
@@ -43,10 +44,7 @@ const char *const unit_token[3] = {"image", "pixel", NULL};
 typedef enum {lattice, pixel_s} coord_system;
 const char *const system_token[3] = {"lattice", "pixel", NULL};
 
-/* Note that 'nearest' is a function in AIX's math.h.  So don't use
-   that as a symbol.
-*/
-typedef enum {interp_nearest, interp_linear} interpolation;
+typedef enum {nearest, linear} interpolation;
 const char *const interpolation_token[3] = {"nearest", "linear", NULL};
 
 typedef enum {free_, fixed} proportion;
@@ -235,7 +233,7 @@ static void set_command_line_defaults (option *const options)
   options->enums[0] = lattice;          /* --input_system         */
   options->enums[1] = lattice;          /* --output_system        */
   options->enums[2] = pixel_u;          /* --input_unit           */
-  options->enums[3] = interp_nearest;   /* --interpolation        */
+  options->enums[3] = nearest;          /* --interpolation        */
   options->enums[4] = free_;            /* --proportion           */
   options->bools[0] = TRUE;             /* --frame_include        */
 }
@@ -1102,9 +1100,9 @@ static void init_buffer (buffer *const b, const world_data *const world,
                        diff (clean_y(yur,outpam), clean_y(y_min,outpam))))
     + 2;
   switch (options->enums[3]) {  /* --interpolation */
-  case interp_nearest:
+  case nearest:
     break;
-  case interp_linear:
+  case linear:
     num_rows += 1;
     break;
   };
@@ -1145,17 +1143,6 @@ static void free_buffer (buffer *const b)
 {
   int i;
 
-  /* We have to read through the end of the input image even if we
-     didn't use all the rows, because if the input is a pipe, the
-     guy writing into the pipe may require all the data to go
-     through.
-  */
-  
-  while (b->last_logical < b->inpam->height-1) {
-      pnm_readpamrow(b->inpam, b->rows[0]);
-      ++b->last_logical;
-  }
-
   for (i=0; i<b->num_rows; i++)
     pnm_freepamrow (b->rows[i]);
   free (b->rows);
@@ -1310,10 +1297,10 @@ int main (int argc, char* argv[])
   outrow = pnm_allocpamrow (&outpam);
   init_interpolation_global_vars (&inbuffer,&inpam,&outpam);
   switch (options.enums[3]) {   /* --interpolation */
-  case interp_nearest:
+  case nearest:
     interpolate = take_nearest;
     break;
-  case interp_linear:
+  case linear:
     interpolate = linear_interpolation;
     break;
   };
