@@ -22,6 +22,8 @@
    implied warranty.
 */
 
+#define _XOPEN_SOURCE   /* get M_PI in math.h */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
@@ -32,14 +34,6 @@
 #include "shhopt.h"
 #include "mallocvar.h"
 
-enum sampleScale {INTENSITY_SAMPLE, GAMMA_SAMPLE};
-/* This indicates a scale for a PAM sample value.  INTENSITY_SAMPLE means
-   the value is proportional to light intensity; GAMMA_SAMPLE means the 
-   value is gamma-adjusted as defined in the PGM/PPM spec.  In both
-   scales, the values are continuous and normalized to the range 0..1.
-
-   This scale has no meaning if the PAM is not a visual image.  
-*/
 
 /****************************/
 /****************************/
@@ -54,10 +48,6 @@ enum sampleScale {INTENSITY_SAMPLE, GAMMA_SAMPLE};
 ** For sinc and bessel the blackman window will be used per default.
 */
 
-#ifndef M_PI
-#define M_PI 3.14159265358979323846
-#endif
-
 #define EPSILON 1e-7
 
 
@@ -76,6 +66,7 @@ pow3 (double x)
 
 
 /* box, pulse, Fourier window, */
+/* box function also know as rectangle function */
 /* 1st order (constant) b-spline */
 
 #define radius_point (0.0)
@@ -91,6 +82,7 @@ filter_box (double x)
 
 
 /* triangle, Bartlett window, */
+/* triangle function also known as lambda function */
 /* 2nd order (linear) b-spline */
 
 #define radius_triangle (1.0)
@@ -194,7 +186,8 @@ filter_sinc(double x)
 {
     /* Note: Some people say sinc(x) is sin(x)/x.  Others say it's
        sin(PI*x)/(PI*x), a horizontal compression of the former which is
-       zero at integer values.  We use the latter.
+       zero at integer values.  We use the latter, whose Fourier transform
+       is a canonical rectangle function (edges at -1/2, +1/2, height 1).
     */
     if (x == 0.0) return 1.0;
     return sin(M_PI*x)/(M_PI*x);
@@ -360,7 +353,7 @@ static filter Filters[] = {
 };
 
 
-typedef double (*filterFunction_t)(double);
+typedef double (*basicFunction_t)(double);
 
 
 /****************************/
@@ -397,8 +390,8 @@ struct cmdlineInfo {
      */
     const char *input_filespec;  /* Filespec of input file */
     unsigned int nomix;
-    filterFunction_t filterFunction; /* NULL if not using resample method */
-    filterFunction_t windowFunction;
+    basicFunction_t filterFunction; /* NULL if not using resample method */
+    basicFunction_t windowFunction;
     double filterRadius;           
     enum scaleType scaleType;
     /* 'xsize' and 'ysize' are numbers of pixels.  Their meaning depends upon
@@ -793,32 +786,101 @@ computeOutputDimensions(struct cmdlineInfo  const cmdline,
 ** http://www.cs.cmu.edu/~ph/zoom
 */
 
+struct filterFunction {
+/*----------------------------------------------------------------------------
+   A function to convolve with the samples.
+-----------------------------------------------------------------------------*/
+    basicFunction_t basicFunction;
+        /* The basic shape of the function.  Its horizontal scale is
+           designed to filter out frequencies greater than 1.
+        */
+    basicFunction_t windowFunction;
+        /* A function to multiply by basicFunction().  NULL if none. */
+    double windowScaler;
+        /* Factor by which to compress windowFunction() horizontally */
+    double horizontalScaler;
+        /* Factor by which to compress basicFunction() *
+           windowFunction horizontally.  Note that compressing
+           horizontally in the sample domain is equivalent to
+           expanding horizontally (and shrinking vertically) in the
+           frequency domain.  I.e. values less than unity have the
+           effect of chopping out high frequencies.
+        */
+    double radius;
+        /* A final filter.  filterFunction(x) is zero for |x| > radius
+           regardless of what the rest of the members say.
+
+           Implementation note:  This is important because windowFunction(),
+           out of laziness, doesn't do the whole job of windowing.  It is
+           not zero beyond the cutoff points as it should be.  If not for
+           that, radius would only be a hint to describe what the other
+           members already do, so the convolver knows where to stop.
+        */
+};
+
 typedef struct {
-    int    index;
-    double weight;
+    /* A term of the linear combination of input rows that makes up an
+       output row.  I.e. an input row and its weight.
+
+       Alternatively, the analogous thing for a column.
+    */
+    int    position;    /* Row/column number in the input image */
+    double weight;      /* Weight to be given to that row/col.  In [0, 1]. */
 } WEIGHT;
 
 typedef struct {
-    int    nWeight;
+    /* A description of the linear combination of input rows that
+       generates a particular output row.  An output row is a weighted
+       average of some input rows.  E.g. Row 2 of the output might be
+       composed of 50% of Row 2 of the input and 50% of Row 3 of the
+       input.
+
+       Alternatively, the analogous thing for columns.
+    */
+    int    nWeight;  /* Number of elements in 'Weight' */
     WEIGHT *Weight;
+        /* The terms of the linear combination.  Has 'nWeight' elements. 
+           The coefficients (weights) of each add up to unity.
+        */
 } WLIST;
 
 typedef struct {
-    int y;
-    tuple *tuplerow;
+    /* This identifies a row of the input image. */
+    int rowNumber;
+        /* The row number in the input image of the row.
+           -1 means no row.
+        */
+    tuple *tuplerow;  
+        /* The tuples of the row.
+           If rowNumber = -1, these are arbitrary, but allocated, tuples.
+        */
 } SCANLINE;
 
 typedef struct {
-    int width;
-    int height;
+    /* A vertical window of a raster */
+    int width;    /* Width of the window, in columns */
+    int height;   /* Height of the window, in rows */
     SCANLINE *line;
+        /* An array of 'height' elements, malloced.
+           This identifies the lines of the input image that compose the
+           window.  The index order is NOT the order of the rows in the
+           image.  E.g. line[0] isn't always the topmost row of the window.
+           Rather, the rows are arranged in a cycle and you have to know
+           indpendently where the topmost one is.  E.g. the rows of a 5
+           line window with topmost row at index 3 might be:
+
+              line[0] = Row 24
+              line[1] = Row 25
+              line[2] = Row 26
+              line[3] = Row 22
+              line[4] = Row 23
+        */
 } SCAN;
 
 
 
 static int
-appendWeight (WLIST *WList, int index, double weight)
-{
+appendWeight (WLIST *WList, int index, double weight) {
     int n;
   
     if (weight == 0.0) 
@@ -831,8 +893,8 @@ appendWeight (WLIST *WList, int index, double weight)
         pm_error ("appendWeight: realloc(%d) failed", 
                   sizeof(*(WList->Weight))*(n+1));
   
-    WList->Weight[n].index  = index;
-    WList->Weight[n].weight = weight;
+    WList->Weight[n].position = index;
+    WList->Weight[n].weight   = weight;
     WList->nWeight++;
   
     return 0;
@@ -859,30 +921,29 @@ floatToSample(double const value,
 
 
 static void
-createWeightList(unsigned int const targetPos,
-                 unsigned int const sourceSize,
-                 double       const scale,
-                 double       const fscale,
-                 double       const radius,
-                 double       const filterRadius,
-                 filterFunction_t   filterFunction,
-                 filterFunction_t   windowFunction,
-                 WLIST *      const weightListP) {
+createWeightList(unsigned int          const targetPos,
+                 unsigned int          const sourceSize,
+                 double                const scale,
+                 struct filterFunction       filter,
+                 WLIST *               const weightListP) {
 /*----------------------------------------------------------------------------
    Create a weight list for computing target pixel number 'targetPos' from
-   a set of source pixels.  The weight list is a list of weights to give
+   a set of source pixels.  These pixels are a line of pixels either 
+   horizontally or vertically.  The weight list is a list of weights to give
    each source pixel in the set.
    
-   The source pixel set is a window of source pixels whose size is
-   determined by 'radius'.  The weights are defined by the function
-   'filterFunction' of the position within the window, and normalized
-   to add up to 1.0.
+   The source pixel set is a window of source pixels centered on some
+   point.  The weights are defined by the function 'filter' of
+   the position within the window, and normalized to add up to 1.0.
+   Technically, the window is infinite, but we know that the filter
+   function is zero beyond a certain distance from the center of the
+   window.
 
    For example, assume 'targetPos' is 5.  That means we're computing weights
    for either Column 5 or Row 5 of the target image.  Assume it's Column 5.
    Assume 'radius' is 1.  That means a window of two pixels' worth of a
    source row determines the color of the Column 5 pixel of a target
-   row.  Assume 'filterFunction' is a triangle function -- 1 at 0, sloping
+   row.  Assume 'filter' is a triangle function -- 1 at 0, sloping
    down to 0 at -1 and 1.
 
    Now assume that the scale factor is 2 -- the target image will be
@@ -917,21 +978,6 @@ createWeightList(unsigned int const targetPos,
 
       10*.022 + 20*.521 + 30*.457 = 24
 
-
-   Actually, the filter function isn't exactly 'filterFunction'.
-   That's the fundamental function, but we modify it with 'fscale' and
-   'windowFunction':
-
-   'fscale' is the amount by which we stretch the filter function
-   horizontally.  In the example above, it is 1.0.  If it were 2.0 instead,
-   then we would use the function f(x) = triangle(x/2.0), which is a
-   function that slopes down to 0 at -2 and 2 instead of -1 and 1.
-
-   'windowFunction' is a function we multiply by the fundamental
-   filter function, after scaling horizontally according to
-   'filterRadius' to fit it.  If 'windowFunction' is NULL, we don't
-   do this multiplication.
-
 -----------------------------------------------------------------------------*/
     /* 'windowCenter', is the continous position within the source of
        the center of the window that will influence target pixel
@@ -942,10 +988,10 @@ createWeightList(unsigned int const targetPos,
        bottom.  
     */
     double const windowCenter = ((double)targetPos + 0.5) / scale;
-    double left = MAX(0.0, windowCenter - radius - EPSILON);
+    double left = MAX(0.0, windowCenter - filter.radius - EPSILON);
     unsigned int const leftPixel = floor(left);
     double right = MIN((double)sourceSize - EPSILON, 
-                       windowCenter + radius + EPSILON);
+                       windowCenter + filter.radius + EPSILON);
     unsigned int const rightPixel = floor(right);
 
     double norm;
@@ -961,15 +1007,18 @@ createWeightList(unsigned int const targetPos,
         /* Calculate the weight that source pixel 'j' will have in the 
            value of target pixel 'targetPos'.
         */
-        double const regionLeft   = MIN(right, (double)(j + 1));
-        double const regionRight  = MAX(left, (double)j);
+        double const regionLeft   = MAX(left, (double)j);
+        double const regionRight  = MIN(right, (double)(j + 1));
         double const regionWidth  = regionRight - regionLeft;
         double const regionCenter = (regionRight + regionLeft) / 2;
         double const dist = regionCenter - windowCenter;
         double weight;
-        weight = filterFunction(dist/fscale);
-        if (windowFunction)
-            weight *= windowFunction(dist/fscale/filterRadius);
+
+        weight = filter.basicFunction(filter.horizontalScaler * dist);
+        if (filter.windowFunction)
+            weight *= filter.windowFunction(
+                filter.horizontalScaler * filter.windowScaler * dist);
+
         assert(regionWidth <= 1.0);
         weight *= regionWidth;
         norm += weight;
@@ -992,43 +1041,125 @@ createWeightList(unsigned int const targetPos,
 
 
 static void
-createWeightListSet(unsigned int const sourceSize,
-                    unsigned int const targetSize,
-                    filterFunction_t   filterFunction,
-                    double       const filterRadius,
-                    filterFunction_t   windowFunction,
-                    WLIST **     const weightListSetP) {
-    
-    double const scale = (double)targetSize / sourceSize;
-    double const radius = filterRadius;
-    double const fscale = 1.0;
-    
-    WLIST *weightListSet;  /* malloc'ed */
-    int i;
+createWeightListSet(unsigned int          const sourceSize,
+                    unsigned int          const targetSize,
+                    struct filterFunction const filterFunction,
+                    WLIST **              const weightListSetP) {
+/*----------------------------------------------------------------------------
+   Create the set of weight lists that will effect the resample.
 
-    /* The original code from Michael Reinelt inflated the radius and
-       fscale by 1/scale for scale < 1.0, thus changing this from a
-       simple resample into a combination of that and averaging of
-       pixels.  We don't do that because we want to keep things clean.
-       Resample is resample.
-       
-       We should probably add some kind of option to cleanly formalize 
-       and expose that mixture of resampling and averaging, because it
-       appears to be a useful function.  
-    */
+   This is where the actual work of resampling gets done.
+
+   The weight list set is a bunch of factors one can multiply by the
+   pixels in a region to effect a resampling.  Multiplying by these
+   factors effects all of the following transformations on the
+   original pixels:
+   
+   1) Filter out any frequencies that are artifacts of the
+      original sampling.  We assume a perfect sampling was done,
+      which means the original continuous dataset had a maximum
+      frequency of 1/2 of the original sample rate and anything
+      above that is an artifact of the sampling.  So we filter out
+      anything above 1/2 of the original sample rate (sample rate
+      == pixel resolution).
+      
+   2) Filter out any frequencies that are too high to be captured
+      by the new sampling -- i.e. frequencies above 1/2 the new
+      sample rate.  This is the information we must lose due to low
+      sample rate.
+      
+   3) Sample the result at the new sample rate.
+
+   We do all three of these steps in a single convolution of the
+   original pixels.  Steps (1) and (2) can be combined into a
+   single frequency domain rectangle function.  A frequency domain
+   rectangle function is a pixel domain sinc function, which is
+   what we assume 'filterFunction' is.  We get Step 3 by computing
+   the convolution only at the new sample points.
+   
+   I don't know what any of this means when 'filterFunction' is
+   not sinc.  Maybe it just means some approximation or additional
+   filtering steps are happening.
+-----------------------------------------------------------------------------*/
+    double const scale = (double)targetSize / sourceSize;
+    WLIST *weightListSet;  /* malloc'ed */
+    unsigned int targetPos;
 
     MALLOCARRAY_NOFAIL(weightListSet, targetSize);
-
-    for (i = 0; i < targetSize; ++i)
-        createWeightList(i, sourceSize, scale, fscale, radius, filterRadius,
-                         filterFunction, windowFunction,
-                         &weightListSet[i]);
+    
+    for (targetPos = 0; targetPos < targetSize; ++targetPos)
+        createWeightList(targetPos, sourceSize, scale, filterFunction,
+                         &weightListSet[targetPos]);
 
     *weightListSetP = weightListSet;
 }
 
 
 
+static struct filterFunction
+makeFilterFunction(double          const scale,
+                   basicFunction_t       basicFunction,
+                   double          const basicRadius,
+                   basicFunction_t       windowFunction) {
+/*----------------------------------------------------------------------------
+   Create a function to convolve with the samples (so it isn't actually
+   a filter function, but the Fourier transform of a filter function.
+   A filter function is something you multiply by in the frequency domain)
+   to create a function from which one can resample.
+
+   Convolving with this function will achieve two goals:
+
+   1) filter out high frequencies that are artifacts of the original
+      sampling (i.e. the turning of a continuous function into a staircase
+      function);
+   2) filter out frequencies higher than half the resample rate, so that
+      the resample will be a perfect sampling of it, and not have aliasing.
+
+
+   To make the calculation even more efficient, we take advantage
+   of the fact that the weight list doesn't depend on the
+   particular old and new sample rates at all except -- all that's
+   important is their ratio (which is 'scale').  So we assume the
+   original sample rate is 1 and the new sample rate is 'scale'.
+
+-----------------------------------------------------------------------------*/
+    double const freqLimit = MIN(1.0, scale);
+        /* We're going to cut out any frequencies above this, to accomplish
+           Steps (1) and (2) above.
+        */
+
+    struct filterFunction retval;
+
+    retval.basicFunction = basicFunction;
+    retval.windowFunction = windowFunction;
+    
+    retval.horizontalScaler = freqLimit;
+
+    /* Our 'windowFunction' argument is a function normalized to the
+       domain (-1, 1).  We need to scale it horizontally to fit the
+       basic filter function.  We assume the radius of the filter
+       function is the area to which the window should fit (i.e. zero
+       beyond the radius, nonzero inside the radius).  But that's
+       really a misuse of radius, because radius is supposed to be
+       just the distance beyond which we can assume for convenience
+       that the filter function is zero, possibly giving up some
+       precision.
+
+       But note that 'windowFunction' isn't zero outside (-1, 1), even
+       though the actual window function is supposed to be.  Hence,
+       scaling the window function exactly to the radius stops our
+       calculations from noticing the wrong values outside (-1, 1) --
+       we'll never use them.
+    */
+    retval.windowScaler = 1/basicRadius;
+
+    retval.radius = basicRadius / retval.horizontalScaler;
+
+    return retval;
+}
+                   
+
+                   
 static void
 destroyWeightListSet(WLIST *      const weightListSet, 
                      unsigned int const size) {
@@ -1045,7 +1176,7 @@ destroyWeightListSet(WLIST *      const weightListSet,
 
 static void
 createScanBuf(struct pam * const pamP,
-              double       const radius,
+              double       const maxRowWeights,
               bool         const verbose,
               SCAN *       const scanbufP) {
 
@@ -1053,17 +1184,17 @@ createScanBuf(struct pam * const pamP,
     unsigned int lineNumber;
 
     scanbuf.width = pamP->width;
-    scanbuf.height = ceil(2.0*(radius+EPSILON) + 1 + EPSILON);
+    scanbuf.height = maxRowWeights;
     MALLOCARRAY_NOFAIL(scanbuf.line, scanbuf.height);
   
     for (lineNumber = 0; lineNumber < scanbuf.height; ++lineNumber) {
-        scanbuf.line[lineNumber].y = -1;
+        scanbuf.line[lineNumber].rowNumber = -1;
         scanbuf.line[lineNumber].tuplerow = pnm_allocpamrow(pamP);
     }
   
     if (verbose)
-        pm_message ("scanline buffer: %d lines of %d pixels", 
-                    scanbuf.height, scanbuf.width);
+        pm_message("scanline buffer: %d lines of %d pixels", 
+                   scanbuf.height, scanbuf.width);
 
     *scanbufP = scanbuf;
 }
@@ -1200,14 +1331,14 @@ outputOneResampledRow(const struct pam * const outpamP,
         }
         
         for (i = 0; i < YW.nWeight; ++i) {
-            int   const yp   = YW.Weight[i].index;
+            int   const yp   = YW.Weight[i].position;
             float const yw   = YW.Weight[i].weight;
             int   const slot = yp % scanbuf.height;
 
             unsigned int j;
             
             for (j = 0; j < XW.nWeight; ++j) {
-                int   const xp    = XW.Weight[j].index;
+                int   const xp    = XW.Weight[j].position;
                 tuple const tuple = scanbuf.line[slot].tuplerow[xp];
                 
                 addInPixel(outpamP, tuple, yw * XW.Weight[j].weight, 
@@ -1223,12 +1354,81 @@ outputOneResampledRow(const struct pam * const outpamP,
 
 
 
+static bool
+scanbufContainsTheRows(SCAN  const scanbuf,
+                       WLIST const rowWeights) {
+/*----------------------------------------------------------------------------
+   Return TRUE iff scanbuf 'scanbuf' contains every row mentioned in
+   'rowWeights'.
+
+   It might contain additional rows besides.
+-----------------------------------------------------------------------------*/
+    bool missingRow;
+    unsigned int i;
+    
+    for (i = 0, missingRow = FALSE;
+         i < rowWeights.nWeight && !missingRow;
+        ++i) {
+        unsigned int const inputRow = rowWeights.Weight[i].position;
+        unsigned int const slot = inputRow % scanbuf.height;
+            /* This is the number of the slot in the scanbuf that would
+               have the input row in question if the scanbuf has the
+               row at all.
+            */
+        if (scanbuf.line[slot].rowNumber != inputRow) {
+            /* Nope, this slot has some other row or no row at all.
+               So the row we're looking for isn't in the scanbuf.
+            */
+            missingRow = TRUE;
+        }
+    }
+    return !missingRow;
+}
+
+
+
+static void
+createWeightLists(struct pam *     const inpamP,
+                  struct pam *     const outpamP,
+                  basicFunction_t  const filterFunction,
+                  double           const filterRadius,
+                  basicFunction_t  const windowFunction,
+                  WLIST **         const horizWeightP,
+                  WLIST **         const vertWeightP,
+                  unsigned int *   const maxRowWeightsP) {
+/*----------------------------------------------------------------------------
+   This is the function that actually does the resampling.  Note that it
+   does it without ever looking at the source or target pixels!  It produces
+   a simple set of numbers that Caller can blindly apply to the source 
+   pixels to get target pixels.
+-----------------------------------------------------------------------------*/
+    struct filterFunction horizFilter, vertFilter;
+
+    horizFilter = makeFilterFunction(
+        (double)outpamP->width/inpamP->width,
+        filterFunction, filterRadius, windowFunction);
+
+    createWeightListSet(inpamP->width, outpamP->width, horizFilter, 
+                        horizWeightP);
+    
+    vertFilter = makeFilterFunction(
+        (double)outpamP->height/inpamP->height,
+        filterFunction, filterRadius, windowFunction);
+
+    createWeightListSet(inpamP->height, outpamP->height, vertFilter, 
+                        vertWeightP);
+
+    *maxRowWeightsP = ceil(2.0*(vertFilter.radius+EPSILON) + 1 + EPSILON);
+}
+
+
+
 static void
 resample(struct pam *     const inpamP,
          struct pam *     const outpamP,
-         filterFunction_t const filterFunction,
+         basicFunction_t  const filterFunction,
          double           const filterRadius,
-         filterFunction_t const windowFunction,
+         basicFunction_t  const windowFunction,
          bool             const verbose,
          bool             const linear) {
 /*---------------------------------------------------------------------------
@@ -1243,10 +1443,11 @@ resample(struct pam *     const inpamP,
   The input file is positioned past the header, to the beginning of the
   raster.  The output file is too.
 ---------------------------------------------------------------------------*/
-    int row_in, row_out;
-    WLIST *XWeight;
-    WLIST *YWeight;
+    int inputRow, outputRow;
+    WLIST * horizWeight;
+    WLIST * vertWeight;
     SCAN scanbuf;
+    unsigned int maxRowWeights;
 
     tuple * line;
         /* This is just work space for outputOneSampleRow() */
@@ -1257,63 +1458,61 @@ resample(struct pam *     const inpamP,
         pm_error("You cannot use the resampling scaling method on "
                  "linear input.");
   
-    createWeightListSet(inpamP->width, outpamP->width, 
-                        filterFunction, filterRadius, 
-                        windowFunction, &XWeight);
-    
-    createWeightListSet(inpamP->height, outpamP->height, 
-                        filterFunction, filterRadius,
-                        windowFunction, &YWeight);
+    createWeightLists(inpamP, outpamP, filterFunction, filterRadius,
+                      windowFunction, &horizWeight, &vertWeight,
+                      &maxRowWeights);
 
-    createScanBuf(inpamP, filterRadius, verbose, &scanbuf);
+    createScanBuf(inpamP, maxRowWeights, verbose, &scanbuf);
 
     if (verbose)
         resampleDimensionMessage(inpamP, outpamP);
 
     line = pnm_allocpamrow(outpamP);
     MALLOCARRAY_NOFAIL(weight, outpamP->depth);
-    
-    row_out = 0;
-    for (row_in = 0; row_in < inpamP->height; ++row_in) {
-        bool targetRowsReady;
-            /* There is enough information in the buffers to output at least
-               one target row.
+
+    outputRow = 0;
+    for (inputRow = 0; inputRow < inpamP->height; ++inputRow) {
+        bool needMoreInput;
+            /* We've output as much as we can using the rows that are in
+               the scanbuf; it's time to move the window.  Or fill it in
+               the first place.
             */
-        int slot;
-    
-        /* read source row */
-        slot = row_in % scanbuf.height;
-        scanbuf.line[slot].y = row_in;
-        pnm_readpamrow(inpamP, scanbuf.line[slot].tuplerow);
+        unsigned int scanbufSlot;
 
-        targetRowsReady = TRUE; /* initial assumption */
-    
-        while (row_out < outpamP->height && targetRowsReady) {
-            WLIST const YW = YWeight[row_out];
+        /* Read source row; add it to the scanbuf */
+        scanbufSlot = inputRow % scanbuf.height;
+        scanbuf.line[scanbufSlot].rowNumber = inputRow;
+        pnm_readpamrow(inpamP, scanbuf.line[scanbufSlot].tuplerow);
 
-            unsigned int i;
+        /* Output all the rows we can make out of the current contents of
+           the scanbuf.  Might be none.
+        */
+        needMoreInput = FALSE;  /* initial assumption */
+        while (outputRow < outpamP->height && !needMoreInput) {
+            WLIST const rowWeights = vertWeight[outputRow];
+                /* The description of what makes up our current output row;
+                   i.e. what fractions of which input rows combine to create
+                   this output row.
+                */
+            assert(rowWeights.nWeight <= scanbuf.height); 
 
-            i = 0;
-
-            while (i < YW.nWeight && targetRowsReady) {
-                int const yp = YW.Weight[i].index;
-                int const slot = yp % scanbuf.height;
-                if (scanbuf.line[slot].y != yp)
-                    targetRowsReady = FALSE;
-                ++i;
-            }
-            if (targetRowsReady) {
-                outputOneResampledRow(outpamP, scanbuf, YW, XWeight, 
-                                      line, weight);
-                ++row_out;
-            }
+            if (scanbufContainsTheRows(scanbuf, rowWeights)) {
+                outputOneResampledRow(outpamP, scanbuf, rowWeights, 
+                                      horizWeight, line, weight);
+                ++outputRow;
+            } else
+                needMoreInput = TRUE;
         }
     }
 
+    if (outputRow != outpamP->height)
+        pm_error("INTERNAL ERROR: assembled only %u of the required %u "
+                 "output rows.", outputRow, outpamP->height);
+
     pnm_freepamrow(line);
     destroyScanbuf(scanbuf);
-    destroyWeightListSet(XWeight, outpamP->width);
-    destroyWeightListSet(YWeight, outpamP->height);
+    destroyWeightListSet(horizWeight, outpamP->width);
+    destroyWeightListSet(vertWeight, outpamP->height);
 }
 
 
@@ -1481,9 +1680,9 @@ accumOutputRow(struct pam * const pamP,
 
 
 static void
-readARow(struct pam *     const pamP,
-         tuplen *         const tuplenRow,
-         enum sampleScale const scale) {
+readARow(struct pam *             const pamP,
+         tuplen *                 const tuplenRow,
+         const pnm_transformMap * const transform) {
 /*----------------------------------------------------------------------------
   Read a row from the input file described by *pamP, as values (for the
   foreground color) proportional to light intensity, with opacity
@@ -1498,34 +1697,54 @@ readARow(struct pam *     const pamP,
 
   The opacity plane we return is the same as a simple libnetpbm read
   would return.
------------------------------------------------------------------------------*/
-    pnm_readpamrown(pamP, tuplenRow);
 
-    if (scale != INTENSITY_SAMPLE)
-        pnm_ungammarown(pamP, tuplenRow);
+  We ASSUME that the transform 'transform' is that necessary to effect
+  the conversion to intensity-linear values and normalize.  If it is
+  NULL, we ASSUME that they already are intensity-proportional and just
+  need to be normalized.
+-----------------------------------------------------------------------------*/
+    tuple * tupleRow;
+
+    tupleRow = pnm_allocpamrow(pamP);
+
+    pnm_readpamrow(pamP, tupleRow);
+
+    pnm_normalizeRow(pamP, tupleRow, transform, tuplenRow);
 
     pnm_applyopacityrown(pamP, tuplenRow);
+
+    pnm_freepamrow(tupleRow);
 }
 
 
 
 static void
-writeARow(struct pam *     const pamP,
-          tuplen *         const tuplenRow,
-          enum sampleScale const scale) {
+writeARow(struct pam *             const pamP,
+          tuplen *                 const tuplenRow,
+          const pnm_transformMap * const transform) {
 /*----------------------------------------------------------------------------
   Write a row to the output file described by *pamP, from values
   proportional to light intensity with opacity included (i.e. the same
   kind of number you would get form readARow()).
 
+  We ASSUME that the transform 'transform' is that necessary to effect
+  the conversion to brightness-linear unnormalized values.  If it is
+  NULL, we ASSUME that they already are brightness-proportional and just
+  need to be unnormalized.
+
   We destroy *tuplenRow in the process.
 -----------------------------------------------------------------------------*/
+    tuple * tupleRow;
+
+    tupleRow = pnm_allocpamrow(pamP);
+
     pnm_unapplyopacityrown(pamP, tuplenRow);
 
-    if (scale != INTENSITY_SAMPLE)
-        pnm_gammarown(pamP, tuplenRow);
+    pnm_unnormalizeRow(pamP, tuplenRow, transform, tupleRow);
 
-    pnm_writepamrown(pamP, tuplenRow);
+    pnm_writepamrow(pamP, tupleRow);
+
+    pnm_freepamrow(tupleRow);
 }
 
 
@@ -1551,14 +1770,14 @@ issueStretchWarning(bool   const verbose,
 
 
 static void
-scaleHorizontallyAndOutputRow(struct pam * const inpamP,
-                              struct pam * const outpamP,
-                              tuplen *     const rowAccumulator,
-                              enum sampleScale const sampleScale,
-                              tuplen *     const newtuplenrow,
-                              float        const xscale,
-                              unsigned int const row,
-                              bool         const verbose) {
+scaleHorizontallyAndOutputRow(struct pam *             const inpamP,
+                              struct pam *             const outpamP,
+                              tuplen *                 const rowAccumulator,
+                              const pnm_transformMap * const transform,
+                              tuplen *                 const newtuplenrow,
+                              float                    const xscale,
+                              unsigned int             const row,
+                              bool                     const verbose) {
 /*----------------------------------------------------------------------------
    Scale the row in 'rowAccumulator' horizontally by factor 'xscale'
    and output it.
@@ -1568,7 +1787,7 @@ scaleHorizontallyAndOutputRow(struct pam * const inpamP,
 -----------------------------------------------------------------------------*/
     if (outpamP->width == inpamP->width)    
         /* shortcut X scaling */
-        writeARow(outpamP, rowAccumulator, sampleScale);
+        writeARow(outpamP, rowAccumulator, transform);
             /* This destroys 'rowAccumulator' */
     else {
         float stretch;
@@ -1581,13 +1800,44 @@ scaleHorizontallyAndOutputRow(struct pam * const inpamP,
                        "arithmetic imprecision", 
                        stretch);
             
-        writeARow(outpamP, newtuplenrow, sampleScale);
+        writeARow(outpamP, newtuplenrow, transform);
             /* This destroys 'newtuplenrow' */
     }
 }
 
 
 
+static void
+createTransforms(struct pam *              const inpamP,
+                 struct pam *              const outpamP,
+                 bool                      const assumeLinear,
+                 const pnm_transformMap ** const inputTransformP,
+                 const pnm_transformMap ** const outputTransformP) {
+
+    if (assumeLinear) {
+        *inputTransformP  = NULL;
+        *outputTransformP = NULL;
+    } else {
+        *inputTransformP  = pnm_createungammatransform(inpamP);
+        *outputTransformP = pnm_creategammatransform(outpamP);
+    }
+}
+
+
+
+static void
+destroyTransforms(const pnm_transformMap * const inputTransform,
+                  const pnm_transformMap * const outputTransform) {
+
+    if (inputTransform)
+        free((void*)inputTransform);
+    
+    if (outputTransform)
+        free((void*)outputTransform);
+}
+
+
+
 static void
 scaleWithMixing(struct pam * const inpamP,
                 struct pam * const outpamP,
@@ -1656,10 +1906,8 @@ scaleWithMixing(struct pam * const inpamP,
        stretch the image vertically first (same process as above, but
        in place of a single-color pixels, we have a vector of colors).
        Then we take each row this vertical stretching generates and
-       stretch it horizontally.  */
-
-    enum sampleScale const sampleScale = 
-        assumeLinear ? INTENSITY_SAMPLE : GAMMA_SAMPLE;
+       stretch it horizontally.  
+    */
 
     tuplen * tuplenrow;     /* An input row */
     tuplen * newtuplenrow;  /* Working space */
@@ -1683,6 +1931,8 @@ scaleWithMixing(struct pam * const inpamP,
     int rowsread;
         /* Number of rows of the input file that have been read */
     int row;
+    const pnm_transformMap * inputTransform;
+    const pnm_transformMap * outputTransform;
     
     tuplenrow = pnm_allocpamrown(inpamP); 
     rowAccumulator = pnm_allocpamrown(inpamP);
@@ -1693,6 +1943,9 @@ scaleWithMixing(struct pam * const inpamP,
 
     newtuplenrow = pnm_allocpamrown(outpamP);
 
+    createTransforms(inpamP, outpamP, assumeLinear,
+                     &inputTransform, &outputTransform);
+
     for (row = 0; row < outpamP->height; ++row) {
         /* First scale Y from tuplerow[] into rowAccumulator[]. */
 
@@ -1700,12 +1953,12 @@ scaleWithMixing(struct pam * const inpamP,
 
         if (outpamP->height == inpamP->height) {
             /* shortcut Y scaling */
-            readARow(inpamP, rowAccumulator, sampleScale);
+            readARow(inpamP, rowAccumulator, inputTransform);
         } else {
             while (fracrowtofill > 0) {
                 if (rowsleft <= 0.0) {
                     if (rowsread < inpamP->height) {
-                        readARow(inpamP, tuplenrow, sampleScale);
+                        readARow(inpamP, tuplenrow, inputTransform);
                         ++rowsread;
                     } else
                         issueStretchWarning(verbose, fracrowtofill);
@@ -1730,11 +1983,12 @@ scaleWithMixing(struct pam * const inpamP,
            horizontally and write it out.
         */
         scaleHorizontallyAndOutputRow(inpamP, outpamP, rowAccumulator,
-                                      sampleScale, newtuplenrow, xscale,
+                                      outputTransform, newtuplenrow, xscale,
                                       row, verbose);
             /* Destroys rowAccumulator */
 
     }
+    destroyTransforms(inputTransform, outputTransform);
     pnm_freepamrown(rowAccumulator);
     pnm_freepamrown(newtuplenrow);
     pnm_freepamrown(tuplenrow);
@@ -1810,7 +2064,7 @@ main(int argc, char **argv ) {
     outpam = inpam;  /* initial value */
     outpam.file = stdout;
 
-    if (PNM_FORMAT_TYPE(inpam.format) == PBM_TYPE && !cmdline.nomix) {
+    if (PNM_FORMAT_TYPE(inpam.format) == PBM_TYPE) {
         outpam.format = PGM_TYPE;
         outpam.maxval = PGM_MAXMAXVAL;
         pm_message("promoting from PBM to PGM");
