@@ -31,7 +31,7 @@
 #include "shhopt.h"
 #include "mallocvar.h"
 
-enum brightMethod {BRIGHT_MAX, BRIGHT_LUMIN};
+enum brightMethod {BRIGHT_LUMINOSITY, BRIGHT_COLORVALUE, BRIGHT_SATURATION};
 
 struct cmdlineInfo {
     /* All the information the user supplied in the command line,
@@ -65,15 +65,17 @@ parseCommandLine (int argc, char ** argv,
    Note that the strings we return are stored in the storage that
    was passed to us as the argv array.  We also trash *argv.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc( 100*sizeof( optEntry ) );
+    optEntry *option_def;
         /* Instructions to optParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
 
-    unsigned int brightmax;
+    unsigned int luminosity, colorvalue, saturation;
 
     unsigned int option_def_index;
 
+    MALLOCARRAY_NOFAIL(option_def, 100);
+
     option_def_index = 0;   /* incremented by OPTENT3 */
     OPTENT3(0,   "bpercent",      OPT_FLOAT,   
             &cmdlineP->bpercent,   &cmdlineP->bpercentSpec, 0);
@@ -85,8 +87,19 @@ parseCommandLine (int argc, char ** argv,
             &cmdlineP->wvalue,     &cmdlineP->wvalueSpec, 0);
     OPTENT3(0,   "keephues",      OPT_FLAG,   
             NULL,                  &cmdlineP->keephues, 0);
-    OPTENT3(0,   "brightmax",      OPT_FLAG,   
-            NULL,                  &brightmax, 0);
+    OPTENT3(0,   "luminosity",    OPT_FLAG,   
+            NULL,                  &luminosity, 0);
+    OPTENT3(0,   "colorvalue",    OPT_FLAG,   
+            NULL,                  &colorvalue, 0);
+    OPTENT3(0,   "saturation",    OPT_FLAG,   
+            NULL,                  &saturation, 0);
+    OPTENT3(0,   "brightmax",     OPT_FLAG,   
+            NULL,                  &colorvalue, 0);
+
+    /* Note: -brightmax was documented and accepted long before it was
+       actually implemented.  By the time we implemented it, we
+       decided -colorvalue was a better name for it.
+    */
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
@@ -113,7 +126,17 @@ parseCommandLine (int argc, char ** argv,
         pm_error("You specified a per centage > 100 for bpercent: %f",
                  cmdlineP->bpercent);
 
-    cmdlineP->brightMethod = brightmax > 0 ? BRIGHT_MAX : BRIGHT_LUMIN;
+    if (luminosity + colorvalue + saturation > 0)
+        pm_error("You can specify only one of "
+                 "-luminosity, -colorvalue, and -saturation");
+    else {
+        if (colorvalue)
+            cmdlineP->brightMethod = BRIGHT_COLORVALUE;
+        else if (saturation)
+            cmdlineP->brightMethod = BRIGHT_SATURATION;
+        else
+            cmdlineP->brightMethod = BRIGHT_LUMINOSITY;
+    }
 
     if (argc-1 > 1)
         pm_error("Program takes at most one argument: the input file "
@@ -139,12 +162,13 @@ buildHistogram(FILE *   const ifp,
    Build the histogram of brightness values for the image that is in file
    'ifp', which is positioned just after the header (at the raster).
 
-   The histogram is the array hist[] such that hist[x] is the number of
-   xels in the image that have brightness x.  That brightness is either
-   the intensity of most intense component of the xel or it is the 
-   luminosity of the xel, depending on 'brightMethod'.  In either case,
-   it is based on the same maxval as the image, which is 'maxval'.  The
-   image is 'cols' columns wide by 'rows' rows high.
+   The histogram is the array hist[] such that hist[x] is the number
+   of xels in the image that have brightness x.  That brightness is
+   either the color value (intensity of most intense component) of the
+   xel or it is the luminosity of the xel, depending on
+   'brightMethod'.  In either case, it is based on the same maxval as
+   the image, which is 'maxval'.  The image is 'cols' columns wide by
+   'rows' rows high.
 
    Leave the file positioned arbitrarily.
 -----------------------------------------------------------------------------*/
@@ -165,11 +189,17 @@ buildHistogram(FILE *   const ifp,
             xelval brightness;
             xel const p = xelrow[col];
             if (PNM_FORMAT_TYPE(format) == PPM_TYPE) {
-                if (brightMethod == BRIGHT_MAX)
-                    brightness = MAX(PPM_GETR(p), 
-                                     MAX(PPM_GETR(p), PPM_GETB(p)));
-                else
+                switch(brightMethod) {
+                case BRIGHT_LUMINOSITY:
                     brightness = PPM_LUMIN(p);
+                    break;
+                case BRIGHT_COLORVALUE:
+                    brightness = ppm_colorvalue(p);
+                    break;
+                case BRIGHT_SATURATION:
+                    brightness = ppm_saturation(p, maxval);
+                    break;
+                }
             } else
                 brightness = PNM_GET1(p);
             ++hist[brightness];
@@ -264,7 +294,8 @@ computeEndValues(FILE *   const ifp,
     if (hist == NULL)
         pm_error("Unable to allocate storage for intensity histogram.");
     else {
-        buildHistogram(ifp, cols, rows, maxval, format, hist, BRIGHT_LUMIN);
+        buildHistogram(ifp, cols, rows, maxval, format, hist,
+                       cmdline.brightMethod);
 
         if (cmdline.bvalueSpec && !cmdline.bpercentSpec) {
             *bvalueP = cmdline.bvalue;
@@ -293,81 +324,95 @@ computeEndValues(FILE *   const ifp,
 
 
 static void
-computeTransferFunction(xelval const bvalue, 
-                        xelval const wvalue,
-                        xelval const maxval,
-                        xelval newIntensity[]) {
+computeTransferFunction(xelval            const bvalue, 
+                        xelval            const wvalue,
+                        xelval            const maxval,
+                        xelval **         const newBrightnessP) {
 /*----------------------------------------------------------------------------
-   Compute the transfer function, i.e. the array newIntensity[] such that
-   newIntensity[x] is the intensity of the xel that should replace a
-   xel with intensity x.
+   Compute the transfer function, i.e. the array *newBrightnessP such that
+   (*newBrightnessP)[x] is the brightness of the xel that should replace a
+   xel with brightness x.  Brightness in this case means either luminosity
+   or color value (and it doesn't matter to us which).
+
+   'bvalue' is the lowest brightness that should map to zero brightness;
+   'wvalue' is the highest brightness that should map to full brightness.
+   brightnesses in between should be stretched linearly.
 
    Define function only for values 0..maxval.
 -----------------------------------------------------------------------------*/
+    xelval * newBrightness;
     xelval i;
 
-    /* Map the lowest intensities to zero */
+    MALLOCARRAY(newBrightness, maxval+1);
+
+    if (newBrightness == NULL)
+        pm_error("Unable to allocate memory for transfer function.");
+
+    /* Map the lowest brightnesses to zero */
     if (bvalue > 0) 
         for (i = 0; i <= bvalue-1; ++i)
-            newIntensity[i] = 0;
+            newBrightness[i] = 0;
+
+    /* Map the middle brightnesses to 0..maxval */
     {
-        /* Map the middle intensities to 0..maxval */
         unsigned int const range = wvalue - bvalue;
         unsigned int val;
         /* The following for loop is a hand optimization of this one:
            for (i = bvalue; i <= wvalue; ++i)
-           newIntensity[i] = (i-bvalue)*maxval/range);
+             newBrightness[i] = (i-bvalue)*maxval/range);
            (with proper rounding)
         */
-        for (i = bvalue, val = range/2; 
+        for (i = bvalue, val = maxval/2; 
              i <= wvalue; 
              ++i, val += maxval)
-            newIntensity[i] = MIN(val / range, maxval);
+            newBrightness[i] = MIN(val / range, maxval);
         }
-    /* Map the highest intensities to maxval */
+
+    /* Map the highest brightnesses to maxval */
     for (i = wvalue+1; i <= maxval; ++i)
-        newIntensity[i] = maxval;
+        newBrightness[i] = maxval;
+
+    *newBrightnessP = newBrightness;
 }
             
 
 
 static float
-intensityScaler(xel               const p,
-                xelval            const newIntensity[]) {
-/*----------------------------------------------------------------------------
-  Return the multiple by which the intensity of pixel of color 'p'
-  (based on maxval 'maxval') should be changed according to the
-  transfer function newIntensity[].
+brightScaler(xel               const p,
+             pixval            const maxval,
+             xelval            const newBrightness[],
+             enum brightMethod const brightMethod) {
 
-  For example, if p is has luminosity 50, and newIntensity[50] is 75,
-  we would return 1.5.
------------------------------------------------------------------------------*/
-    xelval const oldIntensity = PPM_LUMIN(p);
-
-    float scaler;
-
-    if (oldIntensity == 0) {
-        /* Doesn't matter what we scale by.  zero times anything is zero. */
-        scaler = 1.0;
-    } else
-        scaler = (float)newIntensity[oldIntensity]/oldIntensity;
-
-    return scaler;
+    xelval oldBrightness;
+             
+    switch (brightMethod) {
+    case BRIGHT_LUMINOSITY:
+        oldBrightness = PPM_LUMIN(p);
+        break;
+    case BRIGHT_COLORVALUE:
+        oldBrightness = ppm_colorvalue(p);
+        break;
+    case BRIGHT_SATURATION:
+        oldBrightness = ppm_saturation(p, maxval);
+        break;
+    }
+    return (float)newBrightness[oldBrightness]/oldBrightness;
 }
             
 
 
 static void
-writeRowNormalized(xel *  const xelrow,
-                   int    const cols,
-                   xelval const maxval,
-                   int    const format,
-                   bool   const keephues,
-                   xelval const newIntensity[],
-                   xel *  const rowbuf) {
+writeRowNormalized(xel *             const xelrow,
+                   int               const cols,
+                   xelval            const maxval,
+                   int               const format,
+                   enum brightMethod const brightMethod,
+                   bool              const keephues,
+                   xelval            const newBrightness[],
+                   xel *             const rowbuf) {
 /*----------------------------------------------------------------------------
    Write to Standard Output a normalized version of the xel row 
-   'xelrow'.  Normalize it via the transfer function newIntensity[].
+   'xelrow'.  Normalize it via the transfer function newBrightness[].
 
    Use 'rowbuf' as a work buffer.  It is at least 'cols' columns wide.
 -----------------------------------------------------------------------------*/
@@ -379,19 +424,20 @@ writeRowNormalized(xel *  const xelrow,
 
         if (PPM_FORMAT_TYPE(format) == PPM_TYPE) {
             if (keephues) {
-                float const scaler = intensityScaler(p, newIntensity);
-            
+                float const scaler =
+                    brightScaler(p, maxval, newBrightness, brightMethod);
+
                 xelval const r = MIN((int)(PPM_GETR(p)*scaler+0.5), maxval);
                 xelval const g = MIN((int)(PPM_GETG(p)*scaler+0.5), maxval);
                 xelval const b = MIN((int)(PPM_GETB(p)*scaler+0.5), maxval);
                 PNM_ASSIGN(outrow[col], r, g, b);
             } else 
                 PNM_ASSIGN(outrow[col], 
-                           newIntensity[PPM_GETR(p)], 
-                           newIntensity[PPM_GETG(p)], 
-                           newIntensity[PPM_GETB(p)]);
+                           newBrightness[PPM_GETR(p)], 
+                           newBrightness[PPM_GETG(p)], 
+                           newBrightness[PPM_GETB(p)]);
         } else 
-            PNM_ASSIGN1(outrow[col], newIntensity[PNM_GET1(p)]);
+            PNM_ASSIGN1(outrow[col], newBrightness[PNM_GET1(p)]);
     }
     pnm_writepnmrow(stdout, outrow, cols, maxval, format, 0);
 }
@@ -402,7 +448,7 @@ int
 main(int argc, char *argv[]) {
 
     struct cmdlineInfo cmdline;
-    FILE *ifp;
+    FILE *ifP;
     pm_filepos imagePos;
     xelval maxval;
     int rows, cols, format;
@@ -412,20 +458,20 @@ main(int argc, char *argv[]) {
 
     parseCommandLine(argc, argv, &cmdline);
 
-    ifp = pm_openr_seekable(cmdline.inputFilespec);
+    ifP = pm_openr_seekable(cmdline.inputFilespec);
 
     /* Rescale so that bvalue maps to 0, wvalue maps to maxval. */
-    pnm_readpnminit(ifp, &cols, &rows, &maxval, &format);
-    pm_tell2(ifp, &imagePos, sizeof(imagePos));
+    pnm_readpnminit(ifP, &cols, &rows, &maxval, &format);
+    pm_tell2(ifP, &imagePos, sizeof(imagePos));
 
-    computeEndValues(ifp, cols, rows, maxval, format, cmdline, 
+    computeEndValues(ifP, cols, rows, maxval, format, cmdline, 
                      &bvalue, &wvalue);
         
-    if (wvalue <= bvalue)
+    if (wvalue < bvalue)
         pm_error("The colors which become black would overlap the "
                  "colors which become white.");
     else {
-        xelval newIntensity[PNM_OVERALLMAXVAL+1];
+        xelval * newBrightness;
         int row;
         xel * xelrow;
         xel * rowbuf;
@@ -434,21 +480,24 @@ main(int argc, char *argv[]) {
 
         pm_message("remapping %d..%d to %d..%d", bvalue, wvalue, 0, maxval);
 
-        computeTransferFunction(bvalue, wvalue, maxval, newIntensity);
+        computeTransferFunction(bvalue, wvalue, maxval, &newBrightness);
 
-        pm_seek2(ifp, &imagePos, sizeof(imagePos));
+        pm_seek2(ifP, &imagePos, sizeof(imagePos));
         pnm_writepnminit(stdout, cols, rows, maxval, format, 0);
 
         rowbuf = pnm_allocrow(cols);
 
         for (row = 0; row < rows; ++row) {
-            pnm_readpnmrow(ifp, xelrow, cols, maxval, format);
+            pnm_readpnmrow(ifP, xelrow, cols, maxval, format);
             writeRowNormalized(xelrow, cols, maxval, format,
-                               cmdline.keephues, newIntensity, rowbuf);
+                               cmdline.keephues, cmdline.brightMethod,
+                               newBrightness, rowbuf);
         }
+        free(newBrightness);
         pnm_freerow(rowbuf);
         pnm_freerow(xelrow);
     }
-    pm_close(ifp);
-    exit(0);
+    pm_close(ifP);
+    return 0;
 }
+
