@@ -10,16 +10,14 @@
 ** implied warranty.
 */
 
+#define _XOPEN_SOURCE   /* get M_PI in math.h */
+
 #include <math.h>
 
 #include "pnm.h"
 #include "shhopt.h"
 #include "mallocvar.h"
 
-#ifndef M_PI
-#define M_PI    3.14159265358979323846
-#endif /*M_PI*/
-
 #define SCALE 4096
 #define HALFSCALE 2048
 
@@ -138,27 +136,6 @@ computeNewFormat(bool const antialias,
 
 
 
-static bool
-isWhite(xel    const color,
-        xelval const maxval) {
-
-    return (PPM_GETR(color) == maxval &&
-            PPM_GETG(color) == maxval &&
-            PPM_GETB(color) == maxval);
-}
-
-
-
-static bool
-isBlack(xel const color) {
-
-    return (PPM_GETR(color) == 0 &&
-            PPM_GETG(color) == 0 &&
-            PPM_GETB(color) == 0);
-}
-
-
-
 static xel
 backgroundColor(const char * const backgroundColorName,
                 xel *        const topRow,
@@ -180,7 +157,8 @@ backgroundColor(const char * const backgroundColorName,
 
             break;
         case PBM_TYPE:
-            if (!isWhite(retval, maxval) && !isBlack(retval))
+            if (!PNM_EQUAL(retval, pnm_whitexel(maxval, format)) &&
+                !PNM_EQUAL(retval, pnm_blackxel(maxval, format)))
                 pm_error("Image is PBM (black and white), "
                          "but you specified '%s', which is neither black "
                          "nor white, as background color", 
@@ -306,16 +284,7 @@ shearYNoAntialias(xel **           const inxels,
    row, it would be one page per output row plus one page per input row.
 -----------------------------------------------------------------------------*/
     int inrow;
-    unsigned int outrow;
 
-    /* Fill the output with background */
-    for (outrow = 0; outrow < outrows; ++outrow) {
-        unsigned int col;
-        for (col = 0; col < cols; ++col)
-            outxels[outrow][col] = bgxel;
-    }
-
-    /* Overlay that background with sheared image */
     for (inrow = 0; inrow < inrows; ++inrow) {
         int col;
         for (col = 0; col < cols; ++col) {
@@ -324,6 +293,24 @@ shearYNoAntialias(xel **           const inxels,
                 outxels[outrow][col] = inxels[inrow][col];
         }
     }
+    {
+        /* Now fill in the pixels that we didn't fill above, with background */
+        int col;
+        for (col = 0; col < cols; ++col) {
+            int startBgRow, endBgRow;
+            int outrow;
+            
+            if (shearParm[col].intnew0 > 0 ) {
+                startBgRow = 0;
+                endBgRow   = shearParm[col].intnew0;
+            } else {
+                startBgRow = outrows + shearParm[col].intnew0;
+                endBgRow   = outrows;
+            }
+            for (outrow = startBgRow; outrow < endBgRow; ++outrow)
+                outxels[outrow][col] = bgxel;
+        }
+    }
 }
 
 
