@@ -58,180 +58,13 @@ static struct cmdline_info {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    char *input_filename;
+    const char * input_filename;
     unsigned int verbose;
     unsigned int debug;
     unsigned int pal;
     enum {ALL, LEGAL_ONLY, ILLEGAL_ONLY, CORRECTED_ONLY} output;
 } cmdline;
 
-static void
-parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdline_p);
-static void 
-report_mapping(const pixel old_pixel, const pixel new_pixel);
-
-static void 
-make_legal_yuv_b(const pixel input,
-                 pixel * const output_p,
-                 enum legalize * const action_p);
-static void 
-make_legal_yiq_b(const pixel input,
-                 pixel * const output_p,
-                 enum legalize * const action_p);
-
-
-static void
-convert_one_image(FILE * const ifp, struct cmdline_info const cmdline, 
-                  bool * const eofP, 
-                  int * const hicountP, int * const locountP) {
-
-    /* Parameters of input image: */
-    int rows, cols;
-    pixval maxval;
-    int format;
-
-    ppm_readppminit(ifp, &cols, &rows, &maxval, &format);
-    ppm_writeppminit(stdout, cols, rows, maxval, FALSE);
-    {
-        pixel* const input_row = ppm_allocrow(cols);
-        pixel* const output_row = ppm_allocrow(cols);
-        pixel last_illegal_pixel;
-        /* Value of the illegal pixel we most recently processed */
-        pixel black;
-        /* A constant - black pixel */
-
-        PPM_ASSIGN(black, 0, 0, 0);
-
-        PPM_ASSIGN(last_illegal_pixel, 0, 0, 0);  /* initial value */
-        {
-            int row;
-
-            *hicountP = 0; *locountP = 0;  /* initial values */
-
-            for (row = 0; row < rows; ++row) {
-                int col;
-                ppm_readppmrow(ifp, input_row, cols, maxval, format);
-                for (col = 0; col < cols; ++col) {
-                    pixel corrected;
-                    /* Corrected or would-be corrected value for pixel */
-                    enum legalize action;
-                    /* What action was used to make pixel legal */
-                    if (cmdline.pal)
-                        make_legal_yuv_b(input_row[col],
-                                         &corrected,
-                                         &action);
-                    else
-                        make_legal_yiq_b(input_row[col],
-                                         &corrected,
-                                         &action);
-                        
-                    if (action == LOWER_SAT) 
-                        (*hicountP)++;
-                    if (action == RAISE_SAT)
-                        (*locountP)++;
-                    if (cmdline.debug) report_mapping(input_row[col],
-                                                      corrected);
-                    switch (cmdline.output) {
-                    case ALL:
-                        output_row[col] = corrected;
-                        break;
-                    case LEGAL_ONLY:
-                        output_row[col] = (action == ALREADY_LEGAL) ?
-                            input_row[col] : black;
-                        break;
-                    case ILLEGAL_ONLY:
-                        output_row[col] = (action != ALREADY_LEGAL) ?
-                            input_row[col] : black;
-                        break;
-                    case CORRECTED_ONLY:
-                        output_row[col] = (action != ALREADY_LEGAL) ?
-                            corrected : black;
-                        break;
-                    }
-                }
-                ppm_writeppmrow(stdout, output_row, cols, maxval, FALSE);
-            }
-        }
-        ppm_freerow(output_row);
-        ppm_freerow(input_row);
-    }
-}
-
-
-int
-main(int argc, char **argv) {
-    
-    FILE* ifp;
-    int total_hicount, total_locount;
-    int image_count;
-
-    bool eof;
-
-    ppm_init(&argc, argv);
-
-    parse_command_line(argc, argv, &cmdline);
-
-    if (cmdline.input_filename != NULL) 
-        ifp = pm_openr(cmdline.input_filename);
-    else
-        ifp = stdin;
-
-    image_count = 0;    /* initial value */
-    total_hicount = 0;  /* initial value */
-    total_locount = 0;  /* initial value */
-
-    eof = FALSE;
-    while (!eof) {
-        int hicount, locount;
-        convert_one_image(ifp, cmdline, &eof, &hicount, &locount);
-        image_count++;
-        total_hicount += hicount;
-        total_locount += locount;
-        ppm_nextimage(ifp, &eof);
-    }
-
-
-	if (cmdline.verbose) {
-        pm_message("%d images processed.", image_count);
-        pm_message("%d pixels were above the saturation limit.", 
-                   total_hicount);
-        pm_message("%d pixels were below the saturation limit.", 
-                   total_locount);
-    }
-    
-    pm_close(ifp);
-
-    exit(0);
-}
-
-
-
-static void 
-report_mapping(const pixel old_pixel, const pixel new_pixel) {
-/*----------------------------------------------------------------------------
-  Assuming old_pixel and new_pixel are input and output pixels,
-  tell the user that we changed a pixel to make it legal, if in fact we
-  did and it isn't the same change that we just reported.
------------------------------------------------------------------------------*/
-    static pixel last_changed_pixel;
-    static int first_time = TRUE;
-
-    if (!PPM_EQUAL(old_pixel, new_pixel) && 
-        (first_time || PPM_EQUAL(old_pixel, last_changed_pixel))) {
-        pm_message("Mapping %d %d %d -> %d %d %d\n",
-                   PPM_GETR(old_pixel),
-                   PPM_GETG(old_pixel),
-                   PPM_GETB(old_pixel),
-                   PPM_GETR(new_pixel),
-                   PPM_GETG(new_pixel),
-                   PPM_GETB(new_pixel)
-            );
-
-        last_changed_pixel = old_pixel;
-        first_time = FALSE;
-    }    
-}
 
 
 
@@ -458,12 +291,118 @@ make_legal_yuv_b(const pixel input,
 
 
 
+static void 
+report_mapping(const pixel old_pixel, const pixel new_pixel) {
+/*----------------------------------------------------------------------------
+  Assuming old_pixel and new_pixel are input and output pixels,
+  tell the user that we changed a pixel to make it legal, if in fact we
+  did and it isn't the same change that we just reported.
+-----------------------------------------------------------------------------*/
+    static pixel last_changed_pixel;
+    static int first_time = TRUE;
+
+    if (!PPM_EQUAL(old_pixel, new_pixel) && 
+        (first_time || PPM_EQUAL(old_pixel, last_changed_pixel))) {
+        pm_message("Mapping %d %d %d -> %d %d %d\n",
+                   PPM_GETR(old_pixel),
+                   PPM_GETG(old_pixel),
+                   PPM_GETB(old_pixel),
+                   PPM_GETR(new_pixel),
+                   PPM_GETG(new_pixel),
+                   PPM_GETB(new_pixel)
+            );
+
+        last_changed_pixel = old_pixel;
+        first_time = FALSE;
+    }    
+}
+
+
+
+static void
+convert_one_image(FILE * const ifp, struct cmdline_info const cmdline, 
+                  bool * const eofP, 
+                  int * const hicountP, int * const locountP) {
+
+    /* Parameters of input image: */
+    int rows, cols;
+    pixval maxval;
+    int format;
+
+    ppm_readppminit(ifp, &cols, &rows, &maxval, &format);
+    ppm_writeppminit(stdout, cols, rows, maxval, FALSE);
+    {
+        pixel* const input_row = ppm_allocrow(cols);
+        pixel* const output_row = ppm_allocrow(cols);
+        pixel last_illegal_pixel;
+        /* Value of the illegal pixel we most recently processed */
+        pixel black;
+        /* A constant - black pixel */
+
+        PPM_ASSIGN(black, 0, 0, 0);
+
+        PPM_ASSIGN(last_illegal_pixel, 0, 0, 0);  /* initial value */
+        {
+            int row;
+
+            *hicountP = 0; *locountP = 0;  /* initial values */
+
+            for (row = 0; row < rows; ++row) {
+                int col;
+                ppm_readppmrow(ifp, input_row, cols, maxval, format);
+                for (col = 0; col < cols; ++col) {
+                    pixel corrected;
+                    /* Corrected or would-be corrected value for pixel */
+                    enum legalize action;
+                    /* What action was used to make pixel legal */
+                    if (cmdline.pal)
+                        make_legal_yuv_b(input_row[col],
+                                         &corrected,
+                                         &action);
+                    else
+                        make_legal_yiq_b(input_row[col],
+                                         &corrected,
+                                         &action);
+                        
+                    if (action == LOWER_SAT) 
+                        (*hicountP)++;
+                    if (action == RAISE_SAT)
+                        (*locountP)++;
+                    if (cmdline.debug) report_mapping(input_row[col],
+                                                      corrected);
+                    switch (cmdline.output) {
+                    case ALL:
+                        output_row[col] = corrected;
+                        break;
+                    case LEGAL_ONLY:
+                        output_row[col] = (action == ALREADY_LEGAL) ?
+                            input_row[col] : black;
+                        break;
+                    case ILLEGAL_ONLY:
+                        output_row[col] = (action != ALREADY_LEGAL) ?
+                            input_row[col] : black;
+                        break;
+                    case CORRECTED_ONLY:
+                        output_row[col] = (action != ALREADY_LEGAL) ?
+                            corrected : black;
+                        break;
+                    }
+                }
+                ppm_writeppmrow(stdout, output_row, cols, maxval, FALSE);
+            }
+        }
+        ppm_freerow(output_row);
+        ppm_freerow(input_row);
+    }
+}
+
+
 static void
 parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdline_p) {
+                   struct cmdline_info *cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that many of the strings that this function returns in the
-   *cmdline_p structure are actually in the supplied argv array.  And
+   *cmdlineP structure are actually in the supplied argv array.  And
    sometimes, one of these strings is actually just a suffix of an entry
    in argv!
 -----------------------------------------------------------------------------*/
@@ -475,9 +414,9 @@ parse_command_line(int argc, char ** argv,
     unsigned int legalonly, illegalonly, correctedonly;
 
     option_def_index = 0;   /* incremented by OPTENTRY */
-    OPTENT3('v', "verbose",        OPT_FLAG, NULL,  &cmdline_p->verbose,  0);
-    OPTENT3('V', "debug",          OPT_FLAG, NULL,  &cmdline_p->debug,    0);
-    OPTENT3('p', "pal",            OPT_FLAG, NULL,  &cmdline_p->pal,      0);
+    OPTENT3('v', "verbose",        OPT_FLAG, NULL,  &cmdlineP->verbose,  0);
+    OPTENT3('V', "debug",          OPT_FLAG, NULL,  &cmdlineP->debug,    0);
+    OPTENT3('p', "pal",            OPT_FLAG, NULL,  &cmdlineP->pal,      0);
     OPTENT3('l', "legalonly",      OPT_FLAG, NULL,  &legalonly,           0);
     OPTENT3('i', "illegalonly",    OPT_FLAG, NULL,  &illegalonly,         0);
     OPTENT3('c', "correctedonly",  OPT_FLAG, NULL,  &correctedonly,       0);
@@ -489,23 +428,68 @@ parse_command_line(int argc, char ** argv,
     optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
 
     if (argc - 1 == 0)
-        cmdline_p->input_filename = NULL;  /* he wants stdin */
-    else if (argc - 1 == 1) {
-        if (strcmp(argv[1], "-") == 0)
-            cmdline_p->input_filename = NULL;  /* he wants stdin */
-        else 
-            cmdline_p->input_filename = strdup(argv[1]);
-    } else 
-        pm_error("Too many arguments.  The only arguments accepted\n"
-                 "are the mask color and optional input file specificaton");
+        cmdlineP->input_filename = "-";  /* he wants stdin */
+    else if (argc - 1 == 1)
+        cmdlineP->input_filename = argv[1];
+    else 
+        pm_error("Too many arguments.  The only arguments accepted "
+                 "are the mask color and optional input file specification");
 
     if (legalonly + illegalonly + correctedonly > 1)
         pm_error("--legalonly, --illegalonly, and --correctedonly are "
                  "conflicting options.  Specify at most one of these.");
         
-    if (legalonly) cmdline_p->output = LEGAL_ONLY;
-    else if (illegalonly) cmdline_p->output = ILLEGAL_ONLY;
-    else if (correctedonly) cmdline_p->output = CORRECTED_ONLY;
-    else cmdline_p->output = ALL;
+    if (legalonly) 
+        cmdlineP->output = LEGAL_ONLY;
+    else if (illegalonly) 
+        cmdlineP->output = ILLEGAL_ONLY;
+    else if (correctedonly) 
+        cmdlineP->output = CORRECTED_ONLY;
+    else 
+        cmdlineP->output = ALL;
+}
 
+
+
+int
+main(int argc, char **argv) {
+    
+    FILE * ifP;
+    int total_hicount, total_locount;
+    int image_count;
+
+    bool eof;
+
+    ppm_init(&argc, argv);
+
+    parse_command_line(argc, argv, &cmdline);
+
+    ifP = pm_openr(cmdline.input_filename);
+
+    image_count = 0;    /* initial value */
+    total_hicount = 0;  /* initial value */
+    total_locount = 0;  /* initial value */
+
+    eof = FALSE;
+    while (!eof) {
+        int hicount, locount;
+        convert_one_image(ifP, cmdline, &eof, &hicount, &locount);
+        image_count++;
+        total_hicount += hicount;
+        total_locount += locount;
+        ppm_nextimage(ifP, &eof);
+    }
+
+
+	if (cmdline.verbose) {
+        pm_message("%d images processed.", image_count);
+        pm_message("%d pixels were above the saturation limit.", 
+                   total_hicount);
+        pm_message("%d pixels were below the saturation limit.", 
+                   total_locount);
+    }
+    
+    pm_close(ifP);
+
+    return 0;
 }
