@@ -92,8 +92,11 @@ parseCommandLine(int argc, char ** argv,
 
 
 static unsigned int
-exp(unsigned int base, unsigned int exponent) {
-
+powint(unsigned int base, unsigned int exponent) {
+/*----------------------------------------------------------------------------
+   This is standard pow(), but for integers and optimized for small
+   exponents.
+-----------------------------------------------------------------------------*/
     unsigned int result;
     unsigned int i;
 
@@ -171,7 +174,7 @@ main(int argc, char **argv) {
     pam.file = stdout;
     pam.format = PAM_FORMAT;
     pam.plainformat = 0;
-    pam.width = exp(cmdline.maxval+1, cmdline.depth);
+    pam.width = powint(cmdline.maxval+1, cmdline.depth);
     pam.height = 1;
     pam.depth = cmdline.depth;
     pam.maxval = cmdline.maxval;
