@@ -12,10 +12,11 @@
   which was also derived from the same package.
 ****************************************************************************/
 
-#include <math.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <math.h>
+#include <stdio.h>
+
 #include "pbm.h"
 
 /* Support both US and A4. */
@@ -30,26 +31,35 @@
 #define A4WIDTH  (4960)
 #define A4HEIGHT (7016)
 
-int Width;      /* width and height in 600ths of an inch */
-int Height;
-int Pwidth;     /* width in bytes */
-char *bitmap;
+
+struct bitmap {
+    int Width;      /* width and height in 600ths of an inch */
+    int Height;
+    int Pwidth;     /* width in bytes */
+    char *bitmap;
+};
+
+static struct bitmap bitmap;
 
 
 
 static void
-setpixel(int x,int y,int c)
-{
-  int charidx=y*Pwidth+x/8;
-  char bitmask=128>>(x%8);
+setpixel(int const x,
+         int const y,
+         int const c) {
 
-  if(x<0 || x>=Width) return;
-  if(y<0 || y>=Height) return;
+    int const charidx = y * bitmap.Pwidth + x/8;
+    char const bitmask = 128 >> (x % 8);
 
-  if(c)
-    bitmap[charidx] |= bitmask;
-  else
-    bitmap[charidx] &= ~bitmask;
+    if (x < 0 || x >= bitmap.Width)
+        return;
+    if (y < 0 || y >= bitmap.Height)
+        return;
+
+    if (c)
+        bitmap.bitmap[charidx] |= bitmask;
+    else
+        bitmap.bitmap[charidx] &= ~bitmask;
 }
 
 
@@ -147,40 +157,40 @@ setCG(int x,int y)
 
 
 static void
-output_pbm(FILE *file, const int Width, const int Height, 
-           const char bitmap[]) {
+outputPbm(FILE *        const file,
+          struct bitmap const bitmap) {
 /*----------------------------------------------------------------------------
   Create a pbm file containing the image from the global variable bitmap[].
 -----------------------------------------------------------------------------*/
-    const int forceplain = 0;
+    int const forceplain = 0;
     bit *pbmrow;
     int row;
     int bitmap_cursor;
     
-    pbm_writepbminit(file, Width, Height, forceplain);
+    pbm_writepbminit(file, bitmap.Width, bitmap.Height, forceplain);
   
     /* We round the allocated row space up to a multiple of 8 so the ugly
        fast code below can work.
        */
-    pbmrow = pbm_allocrow(((Width+7)/8)*8);
+    pbmrow = pbm_allocrow(((bitmap.Width+7)/8)*8);
     
     bitmap_cursor = 0;
-    for (row = 0; row < Height; row++) {
+    for (row = 0; row < bitmap.Height; row++) {
         int col;
-        for (col = 0; col < Width;) {
+        for (col = 0; col < bitmap.Width;) {
             /* A little ugliness makes a big speed difference here. */
-            pbmrow[col++] = bitmap[bitmap_cursor] & (1<<7);
-            pbmrow[col++] = bitmap[bitmap_cursor] & (1<<6);
-            pbmrow[col++] = bitmap[bitmap_cursor] & (1<<5);
-            pbmrow[col++] = bitmap[bitmap_cursor] & (1<<4);
-            pbmrow[col++] = bitmap[bitmap_cursor] & (1<<3);
-            pbmrow[col++] = bitmap[bitmap_cursor] & (1<<2);
-            pbmrow[col++] = bitmap[bitmap_cursor] & (1<<1);
-            pbmrow[col++] = bitmap[bitmap_cursor] & (1<<0);
+            pbmrow[col++] = bitmap.bitmap[bitmap_cursor] & (1<<7);
+            pbmrow[col++] = bitmap.bitmap[bitmap_cursor] & (1<<6);
+            pbmrow[col++] = bitmap.bitmap[bitmap_cursor] & (1<<5);
+            pbmrow[col++] = bitmap.bitmap[bitmap_cursor] & (1<<4);
+            pbmrow[col++] = bitmap.bitmap[bitmap_cursor] & (1<<3);
+            pbmrow[col++] = bitmap.bitmap[bitmap_cursor] & (1<<2);
+            pbmrow[col++] = bitmap.bitmap[bitmap_cursor] & (1<<1);
+            pbmrow[col++] = bitmap.bitmap[bitmap_cursor] & (1<<0);
                 
             bitmap_cursor++;
         }
-        pbm_writepbmrow(file, pbmrow, Width, forceplain); 
+        pbm_writepbmrow(file, pbmrow, bitmap.Width, forceplain); 
     }
     pbm_freerow(pbmrow);
     pm_close(file);
@@ -213,72 +223,71 @@ framePerimeter(unsigned int const Width,
 
 
 int 
-main(int argc,char** argv)
-{
-  int TP=1;
-  int x,y;
-  char buf[128];
-  char *progname = *argv;
+main(int argc,char** argv) {
 
-  pbm_init(&argc, argv);
+    int TP=1;
+    int x,y;
+    char buf[128];
+    int Width;      /* width and height in 600ths of an inch */
+    int Height;
 
-  Width = USWIDTH;
-  Height = USHEIGHT;
-  if(argc>1 && strcmp(argv[1], "-a4")==0)
-  {
-    Width = A4WIDTH;
-    Height = A4HEIGHT;
-    argc--;
-    argv++;
-  }
-  Pwidth = (Width+7)/8;
-  bitmap = (char *)malloc(Pwidth*Height);
+    pbm_init(&argc, argv);
 
-  if(argc>1)
-    TP=atoi(argv[1]);
-
-  switch(TP)
-  {
-  case 1:
-    framePerimeter(Width, Height);
-    for(x=0; x<Width; x+=100)
-      for(y=0; y<Height; y+=100)
-	setplus(x,y,4);
-    for(x=0; x<Width; x+=100)
-    {
-      sprintf(buf,"%d",x);
-      setstring(x+3,(Height/200)*100+3,buf);
+    if (argc > 1 && strcmp(argv[1], "-a4") == 0) {
+        Width = A4WIDTH;
+        Height = A4HEIGHT;
+        argc--;
+        argv++;
+    } else {
+        Width = USWIDTH;
+        Height = USHEIGHT;
     }
-    for(y=0; y<Height; y+=100)
-    {
-      sprintf(buf,"%d",y);
-      setstring((Width/200)*100+3,y+3,buf);
-    }
-    for(x=0; x<Width; x+=10)
-      for(y=0; y<Height; y+=100)
-	setplus(x,y,((x%100)==50) ? 2 : 1);
-    for(x=0; x<Width; x+=100)
-      for(y=0; y<Height; y+=10)
-	setplus(x,y,((y%100)==50) ? 2 : 1);
-    setCG(Width/2,Height/2);
-    break;
-  case 2:
-    for(y=0; y<300; y++)
-      setpixel(Width/2,Height/2-y,1);
-    break;
-  case 3:
-    for(y=0; y<300; y++)
-    {
-      setpixel(y,y,1);
-      setpixel(Width-1-y,Height-1-y,1);
-    }
-    break;
-  default:
-    fprintf(stderr,"%s: unknown test pattern (%d)\n",progname,TP);
-    return 1;
-  }
 
-  output_pbm(stdout, Width, Height, bitmap);
+    bitmap.Width = Width;
+    bitmap.Height = Height;
+    bitmap.Pwidth = (Width + 7) / 8;
+    bitmap.bitmap = malloc(bitmap.Pwidth * bitmap.Height);
 
-  return 0;
+    if (argc>1)
+        TP = atoi(argv[1]);
+
+    switch (TP) {
+    case 1:
+        framePerimeter(Width, Height);
+        for (x = 0; x < Width; x += 100)
+            for(y = 0; y < Height; y += 100)
+                setplus(x, y, 4);
+        for(x = 0; x < Width; x += 100) {
+            sprintf(buf,"%d", x);
+            setstring(x + 3, (Height/200) * 100 + 3, buf);
+        }
+        for (y=0; y < Height; y += 100) {
+            sprintf(buf, "%d", y);
+            setstring((Width/200) * 100 + 3, y + 3, buf);
+        }
+        for (x = 0; x < Width; x += 10)
+            for (y = 0; y < Height; y += 100)
+                setplus(x, y, ((x%100) == 50) ? 2 : 1);
+        for (x=0; x < Width; x += 100)
+            for (y = 0; y < Height; y += 10)
+                setplus(x, y, ((y%100) == 50) ? 2 : 1);
+        setCG(Width/2, Height/2);
+        break;
+    case 2:
+        for (y = 0; y < 300; ++y)
+            setpixel(Width/2, Height/2-y, 1);
+        break;
+    case 3:
+        for (y = 0; y < 300; ++y) {
+            setpixel(y, y, 1);
+            setpixel(Width-1-y, Height-1-y, 1);
+        }
+        break;
+    default:
+        pm_error("unknown test pattern (%d)", TP);
+    }
+
+    outputPbm(stdout, bitmap);
+
+    return 0;
 }
