@@ -49,17 +49,16 @@
  * edge. Make craters wrap around the image (enables tiling of image).
  */
 
+#define _XOPEN_SOURCE   /* get M_PI in math.h */
+
 #include <assert.h>
 #include <math.h>
 #include "pgm.h"
+#include "mallocvar.h"
 
 static void gencraters ARGS((void));
 static void initseed ARGS((void));
 
-#ifndef M_PI
-#define M_PI	3.14159265358979323846
-#endif
-
 /* Definitions for obtaining random numbers. */
 
 #define Cast(low, high) ((low)+((high)-(low)) * ((rand() & 0x7FFF) / arand))
@@ -130,10 +129,9 @@ static void gencraters()
     /* Acquire the elevation array and initialise it to mean
        surface elevation. */
 
-    aux = (unsigned short *) malloc(SCRX * SCRY * sizeof(short));
-    if (aux == (unsigned short *) 0) {
+    MALLOCARRAY(aux, SCRX * SCRY);
+    if (aux == NULL) 
         pm_error("out of memory allocating elevation array");
-    }
 
     /* Acquire the elevation buffer and initialise to mean
        initial elevation. */
@@ -257,10 +255,10 @@ static void gencraters()
     }
 
     i = MAX((slopemax - slopemin) + 1, 1);
-    slopemap = (unsigned char *) malloc(i * sizeof(unsigned char));
-    if (slopemap == (unsigned char *) 0) {
+    MALLOCARRAY(slopemap, i);
+    if (slopemap == NULL)
         pm_error("out of memory allocating slope map");
-    }
+
     for (i = slopemin; i <= slopemax; i++) {
 
         /* Confused?   OK,  we're using the  left-to-right slope to
