@@ -31,18 +31,14 @@
 
 */
 
+#define _XOPEN_SOURCE   /* get M_PI in math.h */
+
 #include <math.h>
 #include <assert.h>
+
 #include "ppm.h"
 #include "mallocvar.h"
 
-#ifndef M_PI
-#define M_PI    3.14159265358979323846
-#endif
-#ifndef M_E
-#define M_E 2.7182818284590452354
-#endif
-
 #ifdef VMS
 static double const hugeVal = HUGE_VAL;
 #else
