@@ -10,10 +10,12 @@
 */
 #define _FILE_OFFSET_BITS 64
 #define _LARGE_FILES  
+#define _XOPEN_SOURCE
 
 #include <string.h>
 #include <limits.h>
 #include <assert.h>
+#include <stdio.h>
 
 #include <math.h>
 
@@ -124,7 +126,7 @@ validateComputableSize(struct pam * const pamP) {
 
     if (depth > INT_MAX/sizeof(sample))
         pm_error("image depth (%u) too large to be processed", depth);
-    else if (pamP->width > 0 && depth * sizeof(sample) > INT_MAX/pamP->width)
+    else if (depth * sizeof(sample) > INT_MAX/pamP->width)
         pm_error("image width and depth (%u, %u) too large "
                  "to be processed.", pamP->width, depth);
     else if (pamP->width * (depth * sizeof(sample)) >
@@ -163,8 +165,6 @@ pnm_tupleequal(const struct pam * const pamP,
                tuple              const comparand, 
                tuple              const comparator) {
 
-    /* TODO: For speed reasons, we should probably make this an inline */
-
     unsigned int plane;
     bool equal;
 
@@ -184,8 +184,6 @@ pnm_assigntuple(const struct pam * const pamP,
                 tuple              const dest,
                 tuple              const source) {
 
-    /* TODO: For speed reasons, we should probably make this an inline */
-
     unsigned int plane;
     for (plane = 0; plane < pamP->depth; ++plane) {
         dest[plane] = source[plane];
@@ -258,41 +256,53 @@ createBlackTuple(const struct pam * const pamP,
 }
 
 
-tuple *
-pnm_allocpamrow(const struct pam * const pamP) {
+
+static tuple *
+allocPamRow(const struct pam * const pamP) {
 /*----------------------------------------------------------------------------
    We assume that the dimensions of the image are such that arithmetic
    overflow will not occur in our calculations.  NOTE: pnm_readpaminit()
    ensures this assumption is valid.
 -----------------------------------------------------------------------------*/
-    const int bytes_per_tuple = allocationDepth(pamP) * sizeof(sample);
-    tuple * tuplerow;
-
     /* The tuple row data structure starts with 'width' pointers to
        the tuples, immediately followed by the 'width' tuples
        themselves.  Each tuple consists of 'depth' samples.  
     */
 
-    tuplerow = malloc(pamP->width * (sizeof(tuple *) + bytes_per_tuple));
+    int const bytesPerTuple = allocationDepth(pamP) * sizeof(sample);
+    tuple * tuplerow;
+
+    tuplerow = malloc(pamP->width * (sizeof(tuple *) + bytesPerTuple));
+                      
+    if (tuplerow != NULL) {
+        /* Now we initialize the pointers to the individual tuples
+           to make this a regulation C two dimensional array.  
+        */
+        char * p;
+        unsigned int col;
+        
+        p = (char*) (tuplerow + pamP->width);  /* location of Tuple 0 */
+        for (col = 0; col < pamP->width; ++col) {
+                tuplerow[col] = (tuple) p;
+                p += bytesPerTuple;
+        }
+    }
+    return tuplerow;
+}
+
+
+
+tuple *
+pnm_allocpamrow(const struct pam * const pamP) {
+
+
+    tuple * const tuplerow = allocPamRow(pamP);
+
     if (tuplerow == NULL)
         pm_error("Out of memory allocating space for a tuple row of\n"
                  "%d tuples by %d samples per tuple by %d bytes per sample.",
                  pamP->width, allocationDepth(pamP), sizeof(sample));
 
-    {
-        /* Now we initialize the pointers to the individual tuples to make this
-           a regulation C two dimensional array.
-        */
-        
-        char *p;
-        int i;
-        
-        p = (char*) (tuplerow + pamP->width);  /* location of Tuple 0 */
-        for (i = 0; i < pamP->width; i++) {
-            tuplerow[i] = (tuple) p;
-            p += bytes_per_tuple;
-        }
-    }
     return tuplerow;
 }
 
@@ -350,9 +360,13 @@ pnm_scaletuplerow(const struct pam * const pamP,
                   tuple *            const sourceRow,
                   sample             const newMaxval) {
 
-    unsigned int col;
-    for (col = 0; col < pamP->width; ++col) 
-        scaleTuple(pamP, destRow[col], sourceRow[col], newMaxval);
+    if (pamP->maxval == newMaxval) {
+        /* Fast path for common case:  It's already what it needs to be. */
+    } else {        
+        unsigned int col;
+        for (col = 0; col < pamP->width; ++col) 
+            scaleTuple(pamP, destRow[col], sourceRow[col], newMaxval);
+    }
 }
 
  
@@ -361,7 +375,6 @@ tuple **
 pnm_allocpamarray(const struct pam * const pamP) {
     
     tuple **tuplearray;
-    int row;
 
     /* If the speed of this is ever an issue, it might be sped up a little
        by allocating one large chunk.
@@ -371,9 +384,28 @@ pnm_allocpamarray(const struct pam * const pamP) {
     if (tuplearray == NULL) 
         pm_error("Out of memory allocating the row pointer section of "
                  "a %u row array", pamP->height);
+    else {
+        int row;
+        bool outOfMemory;
 
-    for (row = 0; row < pamP->height; ++row) {
-        tuplearray[row] = pnm_allocpamrow(pamP);
+        outOfMemory = FALSE;
+        for (row = 0; row < pamP->height && !outOfMemory; ++row) {
+            tuplearray[row] = allocPamRow(pamP);
+            if (tuplearray[row] == NULL) {
+                unsigned int freerow;
+                outOfMemory = TRUE;
+                
+                for (freerow = 0; freerow < row; ++freerow)
+                    pnm_freepamrow(tuplearray[row]);
+            }
+        }
+        if (outOfMemory) {
+            free(tuplearray);
+            
+            pm_error("Out of memory allocating the %u rows %u columns wide by "
+                     "%u planes deep", pamP->height, pamP->width,
+                     allocationDepth(pamP));
+        }
     }
     return tuplearray;
 }
@@ -396,6 +428,12 @@ void
 pnm_setminallocationdepth(struct pam * const pamP,
                           unsigned int const allocationDepth) {
 
+    if (pamP->len < PAM_STRUCT_SIZE(allocation_depth))
+        pm_error("Can't set minimum allocation depth in pam structure, "
+                 "because the structure is only %u bytes long, and to "
+                 "have an allocation_depth field, it must bea at least %u",
+                 pamP->len, PAM_STRUCT_SIZE(allocation_depth));
+
     pamP->allocation_depth = MAX(allocationDepth, pamP->depth);
         
     validateComputableSize(pamP);
@@ -618,6 +656,11 @@ pnm_readpaminit(FILE *       const file,
     /* Get magic number. */
     pamP->format = pm_readmagicnumber(file);
 
+    pamP->plainformat =
+        (pamP->format == PBM_FORMAT || 
+         pamP->format == PGM_FORMAT ||
+         pamP->format == PPM_FORMAT);
+
     switch (PAM_FORMAT_TYPE(pamP->format)) {
     case PAM_TYPE: 
         readpaminitrest(pamP);
@@ -652,7 +695,6 @@ pnm_readpaminit(FILE *       const file,
     }
     
     pamP->bytes_per_sample = pnm_bytespersample(pamP->maxval);
-    pamP->plainformat = FALSE;
 
     validateComputableSize(pamP);
 }
@@ -691,9 +733,6 @@ pnm_writepaminit(struct pam * const pamP) {
     
     switch (PAM_FORMAT_TYPE(pamP->format)) {
     case PAM_TYPE:
-        if (pm_plain_output)
-            pm_error("There is no plain version of PAM.  -plain option "
-                     "is not allowed");
         fprintf(pamP->file, "P7\n");
         fprintf(pamP->file, "WIDTH %d\n",    pamP->width);
         fprintf(pamP->file, "HEIGHT %d\n",   pamP->height);
@@ -716,7 +755,7 @@ pnm_writepaminit(struct pam * const pamP) {
                      "which exceeds the maximum allowed for PPM: %d", 
                      pamP->maxval, PPM_OVERALLMAXVAL);
         ppm_writeppminit(pamP->file, pamP->width, pamP->height, 
-                         (pixval) pamP->maxval, 0);
+                         (pixval) pamP->maxval, pamP->plainformat);
         break;
 
     case PGM_TYPE:
@@ -729,7 +768,7 @@ pnm_writepaminit(struct pam * const pamP) {
                      "which exceeds the maximum allowed for PGM: %d", 
                      pamP->maxval, PGM_OVERALLMAXVAL);
         pgm_writepgminit(pamP->file, pamP->width, pamP->height, 
-                         (gray) pamP->maxval, 0);
+                         (gray) pamP->maxval, pamP->plainformat);
         break;
 
     case PBM_TYPE:
@@ -740,7 +779,8 @@ pnm_writepaminit(struct pam * const pamP) {
         if (pamP->maxval != 1) 
             pm_error("pnm_writepaminit() got PBM format, but maxval = %ld "
                      "instead of 1, as required for PBM.", pamP->maxval);
-        pbm_writepbminit(pamP->file, pamP->width, pamP->height, 0);
+        pbm_writepbminit(pamP->file, pamP->width, pamP->height, 
+                         pamP->plainformat);
         break;
 
     default:
@@ -879,7 +919,7 @@ pm_feed_from_pamtuples(int    const pipeToFeedFd,
     outpam.file = fdopen(pipeToFeedFd, "w");
 
     /* The following signals (and normally kills) the process with
-       SIGPIPE if the pipe does not take all the data.
+       SIGPIPE if the pipe does not take all 'inputLength' bytes.
     */
     pnm_writepam(&outpam, *inputTuplesP->tuplesP);
 
@@ -894,11 +934,10 @@ pm_accept_to_pamtuples(int    const pipeToSuckFd,
 
     struct pamtuples * const outputTuplesP = accepterParm;
 
-    struct pam * const inpamP = outputTuplesP->pamP;
+    struct pam inpam;
 
     *outputTuplesP->tuplesP =
-        pnm_readpam(fdopen(pipeToSuckFd, "r"), inpamP,
-                    PAM_STRUCT_SIZE(tuple_type));
+        pnm_readpam(fdopen(pipeToSuckFd, "r"), &inpam, sizeof(inpam));
 
-    pm_close(inpamP->file);
+    pm_close(inpam.file);
 }
