@@ -51,7 +51,11 @@ pnm_createtuplehash(void) {
     tuplehash retval;
     unsigned int i;
 
-    MALLOCARRAY_NOFAIL(retval, HASH_SIZE);
+    MALLOCARRAY(retval, HASH_SIZE);
+
+    if (retval == NULL)
+        pm_error("Out of memory allocating tuple hash of size %u",
+                 HASH_SIZE);
 
     for (i = 0; i < HASH_SIZE; ++i) 
         retval[i] = NULL;
@@ -64,7 +68,6 @@ pnm_createtuplehash(void) {
 void
 pnm_destroytuplehash(tuplehash const tuplehash) {
 
-
     int i;
 
     /* Free the chains */
@@ -205,6 +208,67 @@ addColorToHash(tuple          const color,
 
 
 
+static void
+computehashrecoverable(struct pam *   const pamP,
+                       tuple **       const tupleArray, 
+                       unsigned int   const maxsize, 
+                       sample         const newMaxval,
+                       unsigned int * const sizeP,
+                       tuplehash *    const tuplefreqhashP,
+                       tuple **       const rowbufferP,
+                       tuple *        const colorP) {
+/*----------------------------------------------------------------------------
+   This is computetuplefreqhash(), only it leaves a trail so that if it
+   happens to longjmp out because of a failed memory allocation, the
+   setjmp'er can cleanup whatever it had done so far.
+-----------------------------------------------------------------------------*/
+    unsigned int row;
+    struct pam freqPam;
+    bool full;
+
+    freqPam = *pamP;
+    freqPam.maxval = newMaxval;
+
+    *tuplefreqhashP = pnm_createtuplehash();
+    *sizeP = 0;   /* initial value */
+    
+    *rowbufferP = pnm_allocpamrow(pamP);
+    
+    *colorP = pnm_allocpamtuple(&freqPam);
+    
+    full = FALSE;  /* initial value */
+    
+    /* Go through the entire raster, building a hash table of
+           tuple values. 
+    */
+    for (row = 0; row < pamP->height && !full; ++row) {
+        int col;
+        const tuple * tuplerow;  /* The row of tuples we are processing */
+        
+        if (tupleArray)
+            tuplerow = tupleArray[row];
+        else {
+            pnm_readpamrow(pamP, *rowbufferP);
+            tuplerow = *rowbufferP;
+        }
+        for (col = 0; col < pamP->width && !full; ++col) {
+            pnm_scaletuple(pamP, *colorP, tuplerow[col], freqPam.maxval);
+            addColorToHash(*colorP, *tuplefreqhashP, &freqPam, maxsize, 
+                           sizeP, &full);
+        }
+    }
+
+    pnm_freepamtuple(*colorP); *colorP = NULL;
+    pnm_freepamrow(*rowbufferP); *rowbufferP = NULL;
+
+    if (full) {
+        pnm_destroytuplehash(*tuplefreqhashP);
+        *tuplefreqhashP = NULL;
+    }
+}
+
+
+
 static tuplehash
 computetuplefreqhash(struct pam *   const pamP,
                      tuple **       const tupleArray, 
@@ -233,53 +297,35 @@ computetuplefreqhash(struct pam *   const pamP,
 
 -----------------------------------------------------------------------------*/
     tuplehash tuplefreqhash;
-    int row;
     tuple * rowbuffer;  /* malloc'ed */
         /* Buffer for a row read from the input file; undefined (but still
            allocated) if input is not from a file.
         */
     tuple color;  
         /* The color being current being added, scaled to the new maxval */
-    struct pam freqPam;
-    bool full;
+    jmp_buf jmpbuf;
+    jmp_buf * origJmpbufP;
     
-    freqPam = *pamP;
-    freqPam.maxval = newMaxval;
+    /* Initialize to "none" for purposes of error recovery */
+    tuplefreqhash = NULL;
+    rowbuffer = NULL;
+    color = NULL;
 
-    tuplefreqhash = pnm_createtuplehash();
-    *sizeP = 0;   /* initial value */
-
-    rowbuffer = pnm_allocpamrow(pamP);
-
-    color = pnm_allocpamtuple(&freqPam);
-
-    full = FALSE;  /* initial value */
-
-    /* Go through the entire raster, building a hash table of tuple values. */
-    for (row = 0; row < pamP->height && !full; ++row) {
-        int col;
-        const tuple * tuplerow;  /* The row of tuples we are processing */
-
-        if (tupleArray)
-            tuplerow = tupleArray[row];
-        else {
-            pnm_readpamrow(pamP, rowbuffer);
-            tuplerow = rowbuffer;
-        }
-        for (col = 0; col < pamP->width && !full; ++col) {
-            pnm_scaletuple(pamP, color, tuplerow[col], freqPam.maxval);
-            addColorToHash(color, tuplefreqhash, &freqPam, maxsize, 
-                           sizeP, &full);
-        }
+    if (setjmp(jmpbuf) == 0) {
+        pm_setjmpbufsave(&jmpbuf, &origJmpbufP);
+        computehashrecoverable(pamP, tupleArray, maxsize, newMaxval, sizeP,
+                               &tuplefreqhash, &rowbuffer, &color);
+        pm_setjmpbuf(origJmpbufP);
+    } else {
+        if (color) 
+            pnm_freepamtuple(color);
+        if (rowbuffer)
+            pnm_freepamrow(rowbuffer);
+        if (tuplefreqhash)
+            pnm_destroytuplehash(tuplefreqhash);
+        longjmp(*origJmpbufP, -1);
     }
-    pnm_freepamtuple(color);
-    pnm_freepamrow(rowbuffer);
-
-    if (full) {
-        pnm_destroytuplehash(tuplefreqhash);
-        return NULL;
-    } else
-        return tuplefreqhash;
+    return tuplefreqhash;
 }
 
 
@@ -298,31 +344,39 @@ pnm_computetuplefreqhash(struct pam *   const pamP,
 
 
 
-
 tupletable
-pnm_alloctupletable(const struct pam * const pamP, unsigned int const size) {
+pnm_alloctupletable(const struct pam * const pamP, 
+                    unsigned int       const size) {
     
-    unsigned int const tupleIntSize = 
-        sizeof(struct tupleint) - sizeof(sample) 
-        + pamP->depth * sizeof(sample);
-    unsigned int const mainTableSize = size * sizeof(struct tupleint *);
-    void * pool;
     tupletable retval;
-    unsigned int i;
-    
-    /* To save the enormous amount of time it could take to allocate
-       each individual tuple, we do a trick here and allocate everything
-       as a single malloc block and suballocate internally.
-    */
-    
-    pool = malloc(mainTableSize + size * tupleIntSize);
-    
-    retval = (tupletable) pool;
 
-    for (i = 0; i < size; ++i)
-        retval[i] = (struct tupleint *)
-            ((char*)pool + mainTableSize + i * tupleIntSize);
+    if (UINT_MAX / sizeof(struct tupleint) < size)
+        pm_error("size %u is too big for arithmetic", size);
+    else {
+        unsigned int const mainTableSize = size * sizeof(struct tupleint *);
+        unsigned int const tupleIntSize = 
+            sizeof(struct tupleint) - sizeof(sample) 
+            + pamP->depth * sizeof(sample);
+
+        /* To save the enormous amount of time it could take to allocate
+           each individual tuple, we do a trick here and allocate everything
+           as a single malloc block and suballocate internally.
+        */
+        if ((UINT_MAX - mainTableSize) / tupleIntSize < size)
+            pm_error("size %u is too big for arithmetic", size);
+        else {
+            void * pool;
+            unsigned int i;
     
+            pool = malloc(mainTableSize + size * tupleIntSize);
+    
+            retval = (tupletable) pool;
+
+            for (i = 0; i < size; ++i)
+                retval[i] = (struct tupleint *)
+                    ((char*)pool + mainTableSize + i * tupleIntSize);
+        }
+    }
     return retval;
 }
 
@@ -342,10 +396,10 @@ pnm_freetupletable(struct pam * const pamP, tupletable const tupletable) {
 
 
 
-tupletable
-pnm_tuplehashtotable(const struct pam * const pamP,
-                     tuplehash          const tuplehash,
-                     unsigned int       const allocsize) {
+static tupletable
+tuplehashtotable(const struct pam * const pamP,
+                 tuplehash          const tuplehash,
+                 unsigned int       const allocsize) {
 /*----------------------------------------------------------------------------
    Create a tuple table containing the info from a tuple hash.  Allocate
    space in the table for 'allocsize' elements even if there aren't that
@@ -359,25 +413,41 @@ pnm_tuplehashtotable(const struct pam * const pamP,
    in the table to tuples or anything else in existing space.
 -----------------------------------------------------------------------------*/
     tupletable tupletable;
-    int i, j;
 
     tupletable = pnm_alloctupletable(pamP, allocsize);
 
+    if (tupletable != NULL) {
+        unsigned int i, j;
+        /* Loop through the hash table. */
+        j = 0;
+        for (i = 0; i < HASH_SIZE; ++i) {
+            /* Walk this hash chain */
+            struct tupleint_list_item * p;
+            for (p = tuplehash[i]; p; p = p->next) {
+                assert(j < allocsize);
+                tupletable[j]->value = p->tupleint.value;
+                pnm_assigntuple(pamP, tupletable[j]->tuple, p->tupleint.tuple);
+                ++j;
+            }
+        }
+    }
+    return tupletable;
+}
+
+
+
+tupletable
+pnm_tuplehashtotable(const struct pam * const pamP,
+                     tuplehash          const tuplehash,
+                     unsigned int       const allocsize) {
+
+    tupletable tupletable;
+
+    tupletable = tuplehashtotable(pamP, tuplehash, allocsize);
+
     if (tupletable == NULL)
         pm_error("out of memory generating tuple table");
 
-    /* Loop through the hash table. */
-    j = 0;
-    for (i = 0; i < HASH_SIZE; ++i) {
-        /* Walk this hash chain */
-        struct tupleint_list_item * p;
-        for (p = tuplehash[i]; p; p = p->next) {
-            assert(j < allocsize);
-            tupletable[j]->value = p->tupleint.value;
-            pnm_assigntuple(pamP, tupletable[j]->tuple, p->tupleint.tuple);
-            ++j;
-        }
-    }
     return tupletable;
 }
 
@@ -400,16 +470,18 @@ pnm_computetupletablehash(struct pam * const pamP,
 -----------------------------------------------------------------------------*/
     tuplehash tupletablehash;
     unsigned int i;
+    bool fits;
     
     tupletablehash = pnm_createtuplehash();
 
-    for (i = 0; i < tupletableSize; ++i) {
-        bool fits;
-        
+    fits = TRUE;  /* initial assumption */
+    for (i = 0; i < tupletableSize && fits; ++i) {
         pnm_addtotuplehash(pamP, tupletablehash, 
                            tupletable[i]->tuple, i, &fits);
-        if (!fits)
-            pm_error("Out of memory computing tuple hash from tuple table");
+    }
+    if (!fits) {
+        pnm_destroytuplehash(tupletablehash);
+        pm_error("Out of memory computing tuple hash from tuple table");
     }
     return tupletablehash;
 }
@@ -458,8 +530,10 @@ pnm_computetuplefreqtable2(struct pam *   const pamP,
     else {
         unsigned int tableSize = (maxsize == 0 ? uniqueCount : maxsize);
         assert(tableSize >= uniqueCount);
-        tuplefreqtable = pnm_tuplehashtotable(pamP, tuplefreqhash, tableSize);
+        tuplefreqtable = tuplehashtotable(pamP, tuplefreqhash, tableSize);
         pnm_destroytuplehash(tuplefreqhash);
+        if (tuplefreqtable == NULL)
+            pm_error("Out of memory generating tuple table");
     }
     *countP = uniqueCount;
 
