@@ -27,9 +27,9 @@ samplesPerPlainLine(sample       const maxval,
                     unsigned int const depth, 
                     unsigned int const lineLength) {
 /*----------------------------------------------------------------------------
-   Return the number of minimum number of samples that should go in a
-   line 'lineLength' characters long in a plain format non-PBM PNM
-   image with depth 'depth' and maxval 'maxval'.
+   Return the minimum number of samples that should go in a line
+   'lineLength' characters long in a plain format non-PBM PNM image
+   with depth 'depth' and maxval 'maxval'.
 
    Note that this number is just for aesthetics; the Netpbm formats allow
    any number of samples per line.
@@ -341,20 +341,22 @@ pnm_writepamrow(const struct pam * const pamP,
     */
     
     if (pm_plain_output || pamP->plainformat) {
-        if (PAM_FORMAT_TYPE(pamP->format) == PBM_TYPE)
+        switch (PAM_FORMAT_TYPE(pamP->format)) {
+        case PBM_TYPE:
             writePamPlainPbmRow(pamP, tuplerow);
-        else if (   PAM_FORMAT_TYPE(pamP->format) == PGM_TYPE 
-                    || PAM_FORMAT_TYPE(pamP->format) == PPM_TYPE  )
+            break;
+        case PGM_TYPE:
+        case PPM_TYPE:
             writePamPlainRow(pamP, tuplerow);
-        else if (PAM_FORMAT_TYPE(pamP->format) == PAM_TYPE)
-            /* pm_plain_output is impossible here due to assumption stated
-               above about pnm_writepaminit() having checked it.  The
-               pamP->plainformat is meaningless for PAM.
-            */
-            writePamRawRow(pamP, tuplerow, 1);
-        else
+            break;
+        case PAM_TYPE:
+            pm_error("There is no plain version of PAM.  -plain option "
+                     "is not allowed");
+            break;
+        default:
             pm_error("Invalid 'format' value %u in pam structure", 
                      pamP->format);
+        }
     } else
         writePamRawRow(pamP, tuplerow, 1);
 }
