@@ -654,7 +654,9 @@ static struct font default_bdffont = { 14, 15, -1, -3, {
 
 struct font *
 pbm_defaultfont(const char * const name) {
-
+/*----------------------------------------------------------------------------
+   Generate the built-in font with name 'name'.
+-----------------------------------------------------------------------------*/
     struct font * retval;
 
     if (strcmp(name, "bdf") == 0)
@@ -678,7 +680,7 @@ pbm_defaultfont(const char * const name) {
                 for (scol = MIN( col + 32, DEFAULTFONT_COLS) - 1;
                      scol >= (int)col; 
                      --scol) {
-                    if (l & 1)
+                    if (l & 0x01)
                         defaultfont[row][scol] = 1;
                     else
                         defaultfont[row][scol] = 0;
