@@ -14,7 +14,8 @@
 **************************************************************************/
 #define _SVID_SOURCE
     /* Make sure P_tmpdir is defined in GNU libc 2.0.7 (_XOPEN_SOURCE 500
-       does it in other libc's.
+       does it in other libc's).  pm_config.h defines TMPDIR as P_tmpdir
+       in some environments.
     */
 #define _XOPEN_SOURCE 500    /* Make sure ftello, fseeko are defined */
 #define _LARGEFILE_SOURCE 1  /* Make sure ftello, fseeko are defined */
@@ -35,19 +36,20 @@
     /* This makes the the x64() functions available on AIX */
 
 #include <stdio.h>
-#include "version.h"
-#include "compile.h"
-#include "nstring.h"
-#include "shhopt.h"
-#include "pm.h"
 #include <stdarg.h>
 #include <string.h>
 #include <errno.h>
+#include <setjmp.h>
 #ifdef __DJGPP__
   #include <io.h>
 #endif
 
+#include "version.h"
+#include "compile.h"
+#include "nstring.h"
+#include "shhopt.h"
 #include "mallocvar.h"
+#include "pm.h"
 
 /* The following are set by pm_init(), then used by subsequent calls to other
    pm_xxx() functions.
@@ -58,30 +60,57 @@ static bool pm_showmessages;
        specify the --quiet option).
     */
 
-/* Programs should produce output in plain format */
 int pm_plain_output;
+    /* Boolean: programs should produce output in plain format */
+
+static jmp_buf * pm_jmpbufP = NULL;
+    /* A description of the point to which the program should hyperjump
+       if a libnetpbm function encounters an error (libnetpbm functions
+       don't normally return in that case).
+
+       User sets this to something in his own extra-library context.
+       Libnetpbm routines that have something that needs to be cleaned up
+       preempt it.
+
+       NULL, which is the default value, means when a libnetpbm function
+       encounters an error, it causes the process to exit.
+    */
+
+
 
 void
-pm_usage( const char usage[] ) {
-    fprintf( stderr, "usage:  %s %s\n", pm_progname, usage );
-    exit( 1 );
-    }
+pm_setjmpbuf(jmp_buf * const jmpbufP) {
+    pm_jmpbufP = jmpbufP;
+}
+
+
+
+void
+pm_setjmpbufsave(jmp_buf *  const jmpbufP,
+                 jmp_buf ** const oldJmpbufPP) {
+
+    *oldJmpbufPP = pm_jmpbufP;
+    pm_jmpbufP = jmpbufP;
+}
+
+
+
+void
+pm_usage(const char usage[]) {
+    pm_error("usage:  %s %s", pm_progname, usage);
+}
+
+
 
 void
 pm_perror(const char reason[] ) {
-    const char* e;
 
-#ifdef A_STRERROR
-    e = strerror(errno);
-#else /* A_STRERROR */
-    e = sys_errlist[errno];
-#endif /* A_STRERROR */
-
-    if ( reason != 0 && reason[0] != '\0' )
-        pm_error( "%s - %s", reason, e );
+    if (reason != NULL && strlen(reason) != 0)
+        pm_error("%s - errno=%d (%s)", reason, errno, strerror(errno));
     else
-        pm_error( "%s", e );
-    }
+        pm_error("Something failed with errno=%d (%s)", 
+                 errno, strerror(errno));
+}
 
 
 
@@ -112,7 +141,11 @@ pm_error(const char format[], ...) {
     vfprintf(stderr, format, args);
     fputc('\n', stderr);
     va_end(args);
-    exit(1);
+
+    if (pm_jmpbufP)
+        longjmp(*pm_jmpbufP, 1);
+    else
+        exit(1);
 }
 
 
@@ -159,11 +192,11 @@ pm_allocarray(int const cols, int const rows, int const size )  {
     char * rowheap;
 
     MALLOCARRAY(rowIndex, rows + 1);
-    if ( rowIndex == NULL )
+    if (rowIndex == NULL)
         pm_error("out of memory allocating row index (%u rows) for an array",
                  rows);
-    rowheap = malloc( rows * cols * size );
-    if ( rowheap == NULL ) {
+    rowheap = malloc(rows * cols * size);
+    if (rowheap == NULL) {
         /* We couldn't get the whole heap in one block, so try fragmented
            format.
         */
@@ -289,11 +322,11 @@ pm_bitstomaxval(int const bits) {
 }
 
 
-unsigned int 
-pm_lcm(const unsigned int x, 
-       const unsigned int y, 
-       const unsigned int z, 
-       const unsigned int limit) {
+unsigned int PURE_FN_ATTR
+pm_lcm(unsigned int const x, 
+       unsigned int const y,
+       unsigned int const z,
+       unsigned int const limit) {
 /*----------------------------------------------------------------------------
   Compute the least common multiple of 'x', 'y', and 'z'.  If it's bigger than
   'limit', though, just return 'limit'.
@@ -365,7 +398,7 @@ pm_init(const char * const progname, unsigned int const flags) {
     pm_setMessage(FALSE, NULL);
 
     pm_progname = progname;
-    
+
 #ifdef O_BINARY
 #ifdef HAVE_SETMODE
     /* Set the stdin and stdout mode to binary.  This means nothing on Unix,
@@ -384,7 +417,6 @@ pm_init(const char * const progname, unsigned int const flags) {
     if (!isatty(1)) setmode(1,O_BINARY);  /* Standard Output */
 #endif /* HAVE_SETMODE */
 #endif /* O_BINARY */
-
 }
 
 
@@ -657,53 +689,74 @@ pm_openw(const char * const name) {
 
 
 
-FILE * 
-pm_tmpfile(void) {
+void
+pm_make_tmpfile(FILE **       const filePP,
+                const char ** const filenameP) {
 
     int fd;
-    FILE * retval;
-    char buf[FILENAME_MAX];
+    FILE * fileP;
+    const char * filenameTemplate;
+    char * filenameBuffer;  /* malloc'ed */
     const char * tbuf;
     unsigned int fnamelen;
+    const char * tmpdir;
+    const char * dirseparator;
 
     fnamelen = strlen (pm_progname) + 10; /* "/" + "_XXXXXX\0" */
 
     tbuf = getenv("TMPDIR");
 
-    if ((tbuf != NULL) && (strlen(tbuf) > FILENAME_MAX - fnamelen))
-        /* length of TMPDIR value too big for buf */
-        tbuf = NULL;
-
-    buf[FILENAME_MAX - fnamelen -1] = 0;
-
-    if ((tbuf == NULL) || (tbuf[0] == 0))
-        /* ENV string not suitable for evaluation of tmp-file */
-        strncpy(buf, P_tmpdir, FILENAME_MAX - fnamelen);
+    if ((tbuf == NULL) || (strlen(tbuf) == 0))
+        /* environment variable not suitable to construct file name.
+           Use default.
+        */
+        tmpdir = TMPDIR;
     else
-        strncpy(buf, tbuf, FILENAME_MAX - fnamelen);
+        tmpdir = tbuf;
 
-    if (buf[strlen(buf) - 1] != '/')
-        strcat (buf, "/");
+    if (tmpdir[strlen(tmpdir) - 1] == '/')
+        dirseparator = "";
+    else
+        dirseparator = "/";
     
-    strcat(buf, pm_progname);
-    strcat(buf, "_XXXXXX");
+    asprintfN(&filenameTemplate, "%s%s%s%s", 
+              tmpdir, dirseparator, pm_progname, "_XXXXXX");
 
-    fd = mkstemp(buf);
+    if (filenameTemplate == NULL)
+        pm_error("Unable to allocate storage for temporary file name");
+
+    filenameBuffer = (char*)filenameTemplate;
+    fd = mkstemp(filenameBuffer);
 
     if (fd < 0)
         pm_error("Unable to create temporary file.  mkstemp() failed with "
                  "errno %d (%s)", errno, strerror(errno));
     else {
-        unlink(buf);
+        fileP = fdopen(fd, "w+b");
 
-        retval = fdopen(fd, "w+b");
-
-        if (retval == NULL)
+        if (fileP == NULL)
             pm_error("Unable to create temporary file.  fdopen() failed "
                      "with errno %d (%s)", errno, strerror(errno));
     }
+    *filenameP = filenameBuffer;
+    *filePP = fileP;
+}
 
-    return retval;
+
+
+FILE * 
+pm_tmpfile(void) {
+
+    FILE * fileP;
+    const char * tmpfile;
+
+    pm_make_tmpfile(&fileP, &tmpfile);
+
+    unlink(tmpfile);
+
+    strfree(tmpfile);
+
+    return fileP;
 }
 
 
@@ -719,7 +772,7 @@ pm_openr_seekable(const char name[]) {
   and return that file's stream descriptor.
 
   We use a file that the operating system recognizes as temporary, so
-  it picks the filename and deletes the file when we close it.
+  it picks the filename and deletes the file when Caller closes it.
 -----------------------------------------------------------------------------*/
     int stat_rc;
     int seekable;  /* logical: file is seekable */
@@ -813,107 +866,218 @@ pm_closew(FILE * const f) {
 
 
 /* Endian I/O.
+
+   Before Netpbm 10.27 (March 2005), these would return failure on EOF
+   or I/O failure.  For backward compatibility, they still have the return
+   code, but it is always zero and the routines abort the program in case
+   of EOF or I/O failure.  A program that wants to handle failure differently
+   must use lower level (C library) interfaces.  But that level of detail
+   is uncharacteristic of a Netpbm program; the ease of programming that
+   comes with not checking a return code is more Netpbm.
+
+   It is also for historical reasons that these return signed values,
+   when clearly unsigned would make more sense.
 */
 
-int
-pm_readbigshort(FILE * const in, short * const sP) {
+
+
+static void
+abortWithReadError(FILE * const ifP) {
+
+    if (feof(ifP))
+        pm_error("Unexpected end of input file");
+    else
+        pm_error("Error (not EOF) reading file.");
+}
+
+
+
+void
+pm_readchar(FILE * const ifP,
+            char * const cP) {
+    
     int c;
 
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *sP = ( c & 0xff ) << 8;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *sP |= c & 0xff;
-    return 0;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+
+    *cP = c;
 }
 
-int
-pm_writebigshort(FILE* const out, short const s) {
-    (void) putc( ( s >> 8 ) & 0xff, out );
-    (void) putc( s & 0xff, out );
-    return 0;
+
+
+void
+pm_writechar(FILE * const ofP,
+             char   const c) {
+
+    putc(c, ofP);
 }
 
+
+
 int
-pm_readbiglong(FILE * const in, long * const lP) {
+pm_readbigshort(FILE *  const ifP, 
+                short * const sP) {
     int c;
 
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP = ( c & 0xff ) << 24;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP |= ( c & 0xff ) << 16;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP |= ( c & 0xff ) << 8;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP |= c & 0xff;
+    unsigned short s;
+
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    s = (c & 0xff) << 8;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    s |= c & 0xff;
+
+    *sP = s;
+
     return 0;
 }
 
+
+
 int
-pm_writebiglong(FILE * const out, long const l) {
-    (void) putc( ( l >> 24 ) & 0xff, out );
-    (void) putc( ( l >> 16 ) & 0xff, out );
-    (void) putc( ( l >> 8 ) & 0xff, out );
-    (void) putc( l & 0xff, out );
+pm_writebigshort(FILE * const ofP, 
+                 short  const s) {
+
+    putc((s >> 8) & 0xff, ofP);
+    putc(s & 0xff, ofP);
+
     return 0;
 }
 
+
+
 int
-pm_readlittleshort(FILE * const in, short * const sP) {
+pm_readbiglong(FILE * const ifP, 
+               long * const lP) {
+
     int c;
+    unsigned long l;
+
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l = c << 24;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l |= c << 16;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l |= c << 8;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l |= c;
+
+    *lP = l;
 
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *sP = c & 0xff;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *sP |= ( c & 0xff ) << 8;
     return 0;
 }
 
+
+
 int
-pm_writelittleshort(FILE* const out, short const s) {
-    (void) putc( s & 0xff, out );
-    (void) putc( ( s >> 8 ) & 0xff, out );
+pm_writebiglong(FILE * const ofP, 
+                long   const l) {
+
+    putc((l >> 24) & 0xff, ofP);
+    putc((l >> 16) & 0xff, ofP);
+    putc((l >>  8) & 0xff, ofP);
+    putc((l >>  0) & 0xff, ofP);
+
     return 0;
 }
 
+
+
 int
-pm_readlittlelong(FILE * const in, long * const lP) {
+pm_readlittleshort(FILE *  const ifP, 
+                   short * const sP) {
     int c;
+    unsigned short s;
+
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    s = c & 0xff;
+
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    s |= (c & 0xff) << 8;
+
+    *sP = s;
 
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP = c & 0xff;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP |= ( c & 0xff ) << 8;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP |= ( c & 0xff ) << 16;
-    if ( (c = getc( in )) == EOF )
-        return -1;
-    *lP |= ( c & 0xff ) << 24;
     return 0;
 }
 
+
+
 int
-pm_writelittlelong(FILE * const out, long const l) {
-    (void) putc( l & 0xff, out );
-    (void) putc( ( l >> 8 ) & 0xff, out );
-    (void) putc( ( l >> 16 ) & 0xff, out );
-    (void) putc( ( l >> 24 ) & 0xff, out );
+pm_writelittleshort(FILE * const ofP, 
+                    short  const s) {
+
+    putc((s >> 0) & 0xff, ofP);
+    putc((s >> 8) & 0xff, ofP);
+
     return 0;
 }
 
 
+
+int
+pm_readlittlelong(FILE * const ifP, 
+                  long * const lP) {
+    int c;
+    unsigned long l;
+
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l = c;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l |= c << 8;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l |= c << 16;
+    c = getc(ifP);
+    if (c == EOF)
+        abortWithReadError(ifP);
+    l |= c << 24;
+
+    *lP = l;
+
+    return 0;
+}
+
+
+
+int
+pm_writelittlelong(FILE * const ofP, 
+                   long   const l) {
+
+    putc((l >>  0) & 0xff, ofP);
+    putc((l >>  8) & 0xff, ofP);
+    putc((l >> 16) & 0xff, ofP);
+    putc((l >> 24) & 0xff, ofP);
+
+    return 0;
+}
+
+
+
 int 
 pm_readmagicnumber(FILE * const ifP) {
+
     int ich1, ich2;
 
     ich1 = getc(ifP);
@@ -926,6 +1090,8 @@ pm_readmagicnumber(FILE * const ifP) {
     return ich1 * 256 + ich2;
 }
 
+
+
 /* Read a file of unknown size to a buffer. Return the number of bytes
    read. Allocate more memory as we need it. The calling routine has
    to free() the buffer.
@@ -941,25 +1107,24 @@ char *
 pm_read_unknown_size(FILE * const file, 
                      long * const nread) {
     long nalloc;
-    int val;
-    char* buf;
+    char * buf;
     bool eof;
 
     *nread = 0;
-    if ((buf=malloc(PM_BUF_SIZE)) == NULL)
-        pm_error("Cannot allocate memory");
     nalloc = PM_BUF_SIZE;
+    MALLOCARRAY(buf, nalloc);
 
     eof = FALSE;  /* initial value */
 
     while(!eof) {
+        int val;
+
         if (*nread >= nalloc) { /* We need a larger buffer */
             if (nalloc > PM_MAX_BUF_INC)
                 nalloc += PM_MAX_BUF_INC;
             else
                 nalloc += nalloc;
-            if ((buf=realloc(buf, nalloc)) == NULL)
-                pm_error("Cannot allocate %ld bytes of memory", nalloc);
+            REALLOCARRAY_NOFAIL(buf, nalloc);
         }
 
         val = getc(file);
