@@ -12,6 +12,7 @@
 #define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
 #define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
 
+#include <stdlib.h>
 #include <string.h>
 #include <math.h>
 #include "ppm.h"
@@ -65,7 +66,8 @@ invRgbnorm(pixval       const rgb,
         retval = (long) ((double) rgb * 65535UL / maxval + 0.5);
         break;
     default:
-        pm_error("Internal error in invRgbnorm()");
+        pm_message("Internal error in invRgbnorm()");
+        abort();
     }
     return retval;
 }
@@ -533,11 +535,11 @@ nonnegative(unsigned int const arg) {
 
 
 pixel
-color_from_ycbcr(unsigned int const y, 
-                 int          const cb, 
-                 int          const cr) {
+ppm_color_from_ycbcr(unsigned int const y, 
+                     int          const cb, 
+                     int          const cr) {
 /*----------------------------------------------------------------------------
-   Return the color that has luminence 'y', blue chrominance 'cb', and
+   Return the color that has luminance 'y', blue chrominance 'cb', and
    red chrominance 'cr'.
 
    The 3 input values can be on any scale (as long as it's the same
@@ -556,3 +558,122 @@ color_from_ycbcr(unsigned int const y,
     
     return retval;
 }
+
+
+
+pixel
+ppm_color_from_hsv(struct hsv const hsv,
+                   pixval     const maxval) {
+
+    pixel retval;
+    double R, G, B;
+
+    if (hsv.s == 0) {
+        R = hsv.v;
+        G = hsv.v;
+        B = hsv.v;
+    } else {
+        unsigned int const sectorSize = 60;
+            /* Color wheel is divided into six 60 degree sectors. */
+        unsigned int const sector = (hsv.h/sectorSize);
+            /* The sector in which our color resides.  Value is in 0..5 */
+        double const f = (hsv.h - sector*sectorSize)/60;
+            /* The fraction of the way the color is from one side of
+               our sector to the other side, going clockwise.  Value is
+               in [0, 1).
+            */
+        double const m = (hsv.v * (1 - hsv.s));
+        double const n = (hsv.v * (1 - (hsv.s * f)));
+        double const k = (hsv.v * (1 - (hsv.s * (1 - f))));
+
+        switch (sector) {
+        case 0:
+            R = hsv.v;
+            G = k;
+            B = m;
+            break;
+        case 1:
+            R = n;
+            G = hsv.v;
+            B = m;
+            break;
+        case 2:
+            R = m;
+            G = hsv.v;
+            B = k;
+            break;
+        case 3:
+            R = m;
+            G = n;
+            B = hsv.v;
+            break;
+        case 4:
+            R = k;
+            G = m;
+            B = hsv.v;
+            break;
+        case 5:
+            R = hsv.v;
+            G = m;
+            B = n;
+            break;
+        default:
+            pm_error("Invalid H value passed to color_from_HSV: %f", hsv.h);
+        }
+    }
+    PPM_ASSIGN(retval, 
+               ROUNDU(R * maxval),
+               ROUNDU(G * maxval),
+               ROUNDU(B * maxval));
+
+    return retval;
+}
+
+
+
+struct hsv
+ppm_hsv_from_color(pixel  const color,
+                   pixval const maxval) {
+
+    double const epsilon = 1e-5;
+
+    double const R = (double)(PPM_GETR(color) + maxval - 1) / maxval;
+    double const G = (double)(PPM_GETG(color) + maxval - 1) / maxval;
+    double const B = (double)(PPM_GETB(color) + maxval - 1) / maxval;
+
+    struct hsv retval;
+    double range;
+
+    retval.v = MAX(R, MAX(G, B));
+
+    range = retval.v - MIN(R, MIN(G, B));
+
+    if (retval.v < epsilon)
+        retval.s = 0.0;
+    else
+        retval.s = range/retval.v;
+
+    if (range < epsilon)
+        /* It's gray, so hue really has no meaning.  We arbitrarily pick 0 */
+        retval.h = 0.0;
+    else {
+        double const cr = (retval.v - R) / range;
+        double const cg = (retval.v - G) / range;
+        double const cb = (retval.v - B) / range;
+
+        double angle;  /* hue angle, in range -30 - +330 */
+
+        if (R == retval.v)
+            angle = 60.0 * (cb - cg);
+        else if (G == retval.v)
+            angle = 120.0 + 60.0 * (cr - cb);
+        else if (B == retval.v)
+            angle = 240.0 + 60.0 * (cg - cr);
+        else
+            pm_error("Internal error: neither r, g, nor b is maximum");
+
+        retval.h = angle >= 0.0 ? angle : 360 + angle;
+    }
+
+    return retval;
+}
