@@ -9,6 +9,13 @@
 #include "pm.h"
 #include "pnm.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+#if 0
+} /* to fake out automatic code indenters */
+#endif
+
 typedef unsigned long sample;
 
 struct pam {
@@ -56,22 +63,47 @@ struct pam {
         /* Number of bytes used to represent each sample in the image file.
            Note that this is strictly a function of 'maxval'.  It is in a
            a separate member for computational speed.
-           */
+        */
     char tuple_type[256];
-        /* The tuple type string from the image header.  Netpbm does 
-           not define any values for this except the following, which are
-           used for a PAM image which is really a view of a PBM, PGM,
-           or PPM image:  PAM_PBM_TUPLETYPE, PAM_PGM_TUPLETYPE, 
-           PAM_PPM_TUPLETYPE.
-           */
+        /* The tuple type string from the image header.  If the PAM image
+           is really a view of a PBM, PGM, or PPM image, the value is
+           PAM_PBM_TUPLETYPE, PAM_PGM_TUPLETYPE, or PAM_PPM_TUPLETYPE,
+           respectively.
+        */
+    unsigned int allocation_depth;
+        /* The number of samples for which memory is allocated for any
+           'tuple' type associated with this PAM structure.  This must
+           be at least as great as 'depth'.  Only the first 'depth' of
+           the samples of a tuple are meaningful.
+
+           The purpose of this is to make it possible for a program to
+           change the type of a tuple to one with more or fewer
+           planes.  
+
+           0 means the allocation depth is the same as the image depth.
+        */
 };
 
+/* PAM_STRUCT_SIZE(x) tells you how big a struct pam is up through the 
+   member named x.  This is useful in conjunction with the 'len' value
+   to determine which fields are present in the structure.
+*/
+#define PAM_MEMBER_OFFSET(mbrname) \
+  ((unsigned int)(char*)&((struct pam *)0)->mbrname)
+#define PAM_MEMBER_SIZE(mbrname) \
+  sizeof(((struct pam *)0)->mbrname)
+#define PAM_STRUCT_SIZE(mbrname) \
+  (PAM_MEMBER_OFFSET(mbrname) + PAM_MEMBER_SIZE(mbrname))
+
+#define PAM_BLACK 0
+#define PAM_BW_WHITE 1
+
 #define PAM_PBM_TUPLETYPE "BLACKANDWHITE"
 #define PAM_PGM_TUPLETYPE "GRAYSCALE"
 #define PAM_PPM_TUPLETYPE "RGB"
 
-#define PAM_PBM_BLACK 0
-#define PAM_PBM_WHITE 1
+#define PAM_PBM_BLACK PAM_BLACK
+#define PAM_PBM_WHITE PAM_BW_WHITE
     /* These are values of samples in a PAM image that represents a black
        and white bitmap image.  They are the values of black and white,
        respectively.  For example, if you use pnm_readpamrow() to read a
@@ -83,15 +115,17 @@ struct pam {
 #define PAM_GRN_PLANE 1
 #define PAM_BLU_PLANE 2
     /* These are plane numbers for the 3 planes of a PAM image that
-       represents an RGB image (tuple type is PAM_PPM_TUPLETYPE).  So
+       represents an RGB image (tuple type is "RGB").  So
        if 'pixel' is a tuple returned by pnmreadpamrow(), then
        pixel[PAM_GRN_PLANE] is the value of the green sample in that
        pixel.
        */
 #define PAM_TRN_PLANE 3
-    /* Some PAMs with "RGB" or "RGBA" tuple types have this 4th plane
+    /* A PAM with "RGB_ALPHA" tuple type has this 4th plane
        for transparency.  0 = transparent, maxval = opaque.
     */
+#define PAM_GRAY_TRN_PLANE 1
+    /* For a "GRAYSCALE" tuple type, this is the transparency plane */
 
 typedef sample *tuple;  
     /* A tuple in a PAM.  This is an array such that tuple[i-1] is the
@@ -121,6 +155,33 @@ struct pamtuples {
 };
 
 
+typedef float * pnm_transformMap;
+    /* This is an array of transform maps.  transform[N] is the 
+       array that is the map for Plane N.
+   
+       Transform maps define a transformation between PAM sample value
+       to normalized libnetpbm "samplen" value, i.e. what you get back
+       from pnm_readpamrown() or pass to pnm_writepamrown().
+       Typically, it's a gamma transfer function generated by
+       pnm_creategammatransform() or pnm_createungammatransform().
+
+       NULL for any transform means just plain normalization -- divide
+       the PAM sample value by the maxval to get the samplen, multiply
+       samplen by the maxval and round to get PAM sample value.
+
+       NULL for map table, or 'transform' member not present (pam
+       structure is too small to contain it) means ALL transforms
+       are plain normalization.
+
+       Each transform map is an array indexed by a PAM sample
+       value, containing 'float' values.  So it must have 'maxval'
+       entries.  The sample -> samplen tranformation is just the
+       obvious table lookup.  The samplen -> sample transformation is
+       more complicated -- if the samplen value is between map[N]
+       and map[N+1], then the sample value is N.  And only transforms
+       where map[N+1] > map[N] are allowed.  
+    */
+
 /* Declarations of library functions. */
 
 /* We don't have a specific PAM function for init and nextimage, because
@@ -130,6 +191,9 @@ struct pamtuples {
 char
 stripeq(const char * const comparand, const char * const comparator);
 
+unsigned int
+pnm_bytespersample(sample const maxval);
+
 int
 pnm_tupleequal(const struct pam * const pamP, 
                tuple              const comparand, 
@@ -166,6 +230,23 @@ pnm_scaletuplerow(const struct pam * const pamP,
                   tuple *            const sourceRow,
                   sample             const newMaxval);
 
+void 
+pnm_maketuplergb(const struct pam * const pamP,
+                 tuple              const tuple);
+
+void 
+pnm_makerowrgb(const struct pam * const pamP,
+               tuple *            const tuplerow);
+
+void 
+pnm_makearrayrgb(const struct pam * const pamP,
+                 tuple **           const tuples);
+
+void
+pnm_getopacity(const struct pam * const pamP,
+               bool *             const haveOpacityP,
+               unsigned int *     const opacityPlaneP);
+
 void
 pnm_createBlackTuple(const struct pam * const pamP, tuple * const blackTupleP);
 
@@ -173,8 +254,8 @@ void
 createBlackTuple(const struct pam * const pamP, tuple * const blackTupleP);
 
 
-#define pnm_allocpamtuple(pamP) \
-    ((tuple) pm_allocrow(1, (pamP)->depth*sizeof(sample)))
+tuple
+pnm_allocpamtuple(const struct pam * const pamP);
 
 #define pnm_freepamtuple(tuple) pm_freerow((char*) tuple)
 
@@ -189,10 +270,20 @@ pnm_allocpamarray(const struct pam * const pamP);
 void
 pnm_freepamarray(tuple ** const tuplearray, const struct pam * const pamP);
 
+void 
+pnm_setminallocationdepth(struct pam * const pamP,
+                          unsigned int const allocationDepth);
+
 void
-pnm_setpamrow(struct pam const pam, 
-              tuple *    const tuplerow, 
-              sample     const value);
+pnm_setpamrow(const struct pam * const pam, 
+              tuple *            const tuplerow, 
+              sample             const value);
+
+unsigned char *
+pnm_allocrowimage(const struct pam * const pamP);
+
+void
+pnm_freerowimage(unsigned char * const rowimage);
 
 void 
 pnm_readpaminit(FILE *file, struct pam * const pamP, const int size);
@@ -206,9 +297,20 @@ pnm_readpam(FILE *file, struct pam * const pamP, const int size);
 void 
 pnm_writepaminit(struct pam * const pamP);
 
+void
+pnm_formatpamrow(const struct pam * const pamP,
+                 const tuple *      const tuplerow,
+                 unsigned char *    const outbuf,
+                 unsigned int *     const rowSizeP);
+
 void 
 pnm_writepamrow(const struct pam * const pamP, const tuple * const tuplerow);
 
+void
+pnm_writepamrowmult(const struct pam * const pamP, 
+                    const tuple *      const tuplerow,
+                    unsigned int       const rptcnt);
+
 void 
 pnm_writepam(struct pam * const pamP, tuple ** const tuplearray);
 
@@ -245,7 +347,81 @@ void
 pnm_writepamrown(const struct pam * const pamP, 
                  const tuplen *     const tuplenrow);
 
-/*---------------------------------------------------------------------------*/
+tuplen **
+pnm_allocpamarrayn(const struct pam * const pamP);
+
+void
+pnm_freepamarrayn(tuplen **          const tuplenarray, 
+                  const struct pam * const pamP);
+
+tuplen** 
+pnm_readpamn(FILE *       const file, 
+             struct pam * const pamP, 
+             int          const size);
+
+void 
+pnm_writepamn(struct pam * const pamP, 
+              tuplen **    const tuplenarray);
+
+
+void
+pnm_normalizetuple(struct pam * const pamP,
+                   tuple        const tuple,
+                   tuplen       const tuplen);
+
+void
+pnm_unnormalizetuple(struct pam * const pamP,
+                     tuplen       const tuplen,
+                     tuple        const tuple);
+
+void
+pnm_normalizeRow(struct pam *             const pamP,
+                 const tuple *            const tuplerow,
+                 const pnm_transformMap * const transform,
+                 tuplen *                 const tuplenrow);
+
+void
+pnm_unnormalizeRow(struct pam *             const pamP,
+                   const tuplen *           const tuplenrow,
+                   const pnm_transformMap * const transform,
+                   tuple *                  const tuplerow);
+
+/*----------------------------------------------------------------------------
+   Facilities for working with visual images in particular
+-----------------------------------------------------------------------------*/
+
+
+void
+pnm_gammarown(struct pam * const pamP,
+              tuplen *     const row);
+
+void
+pnm_ungammarown(struct pam * const pamP,
+                tuplen *     const row);
+
+void
+pnm_applyopacityrown(struct pam * const pamP,
+                     tuplen *     const tuplenrow);
+
+void
+pnm_unapplyopacityrown(struct pam * const pamP,
+                       tuplen *     const tuplenrow);
+
+pnm_transformMap *
+pnm_creategammatransform(const struct pam * const pamP);
+
+void
+pnm_freegammatransform(const pnm_transformMap * const transform,
+                       const struct pam *       const pamP);
+
+pnm_transformMap *
+pnm_createungammatransform(const struct pam * const pamP);
+
+#define pnm_freeungammatransform pnm_freegammatransform;
+
+tuple
+pnm_parsecolor(const char * const colorname,
+               sample       const maxval);
 
 extern double 
 pnm_lumin_factor[3];
@@ -263,18 +439,20 @@ pnm_YCbCr_to_rgbtuple(const struct pam * const pamP,
                       int *              const overflowP);
 
 
-/* The following are meant for passing to pm_system() as Standard Input
-   feeder and Standard Output accepter
-*/
+/*----------------------------------------------------------------------------
+   These are meant for passing to pm_system() as Standard Input feeder
+   and Standard Output accepter.
+-----------------------------------------------------------------------------*/
+
 void
 pm_feed_from_pamtuples(int    const pipeToFeedFd,
                        void * const feederParm);
 
 void
 pm_accept_to_pamtuples(int    const pipeToSuckFd,
-                       void * const accepterParm );
-
+                       void * const accepterParm);
 
+#ifdef __cplusplus
+}
+#endif
 #endif
-
-
