@@ -788,19 +788,10 @@ int portable_vsnprintf(char *str, size_t str_m, const char *fmt, va_list ap) {
       }
       default: /* unrecognized conversion specifier, keep format string as-is*/
         zero_padding = 0;  /* turn zero padding off for non-numeric convers. */
-#ifndef DIGITAL_UNIX_COMPATIBLE
         justify_left = 1; min_field_width = 0;                /* reset flags */
-#endif
-#if defined(PERL_COMPATIBLE) || defined(LINUX_COMPATIBLE)
-     /* keep the entire format string unchanged */
-        str_arg = starting_p; str_arg_l = p - starting_p;
-     /* well, not exactly so for Linux, which does something inbetween,
-      * and I don't feel an urge to imitate it: "%+++++hy" -> "%+y"  */
-#else
      /* discard the unrecognized conversion, just keep *
       * the unrecognized conversion character          */
         str_arg = p; str_arg_l = 0;
-#endif
         if (*p) str_arg_l++;  /* include invalid conversion specifier unchanged
                                  if not at end-of-string */
         break;
@@ -998,10 +989,43 @@ int vasnprintfN (const char **ptr, size_t str_m, const char *fmt, va_list ap) {
   *ptr = str;
   return str_l;
 }
+#endif
+
+
 
 void
 strfree(const char * const string) {
     free((void *) string);
 }
-#endif
 
+
+
+const char *
+strsepN(char ** const stringP, const char * const delim) {
+    const char * retval;   
+
+    if (stringP == NULL || *stringP == NULL)
+        retval = NULL;
+    else {
+        char * p;
+
+        retval = *stringP;
+
+        for (p = *stringP; *p && strchr(delim, *p) == NULL; ++p);
+ 
+        if (*p) {
+            /* We hit a delimiter, not end-of-string.  So null out the 
+               delimiter and advance user's pointer to the next token
+            */
+            *p++ = '\0';
+            *stringP = p;
+        } else {
+            /* We ran out of string.  So the end-of-string delimiter is 
+               already there, and we set the user's pointer to NULL to 
+               indicate there are no more tokens.
+            */
+            *stringP = NULL;
+        }
+    }
+    return retval;
+}
