@@ -271,16 +271,12 @@ optExecute(optEntry  const opt, char *arg, int lng)
 
     switch (opt.type) {
     case OPT_FLAG:
-        if (opt.flags & OPT_CALLFUNC)
-            ((void (*)(void)) opt.arg)();
-        else if (opt.arg)
+        if (opt.arg)
             *((int *) opt.arg) = 1;
         break;
 
     case OPT_STRING:
-        if (opt.flags & OPT_CALLFUNC)
-            ((void (*)(char *)) opt.arg)(arg);
-        else if (opt.arg)
+        if (opt.arg)
             *((char **) opt.arg) = arg;
         break;
 
@@ -300,14 +296,9 @@ optExecute(optEntry  const opt, char *arg, int lng)
             optFatal("number `%s' to `%s' out of range\n",
                      arg, optString(opt, lng));
         if (opt.type == OPT_INT) {
-            if (opt.flags & OPT_CALLFUNC)
-                ((void (*)(int)) opt.arg)((int) tmp);
-            else
-                *((int *) opt.arg) = (int) tmp;
+            *((int *) opt.arg) = (int) tmp;
         } else /* OPT_LONG */ {
-            if (opt.flags & OPT_CALLFUNC)
-                ((void (*)(long)) opt.arg)(tmp);
-            else if (opt.arg)
+            if (opt.arg)
                 *((long *) opt.arg) = tmp;
         }
         break;
@@ -329,14 +320,10 @@ optExecute(optEntry  const opt, char *arg, int lng)
             optFatal("number `%s' to `%s' out of range\n",
                      arg, optString(opt, lng));
         if (opt.type == OPT_UINT) {
-            if (opt.flags & OPT_CALLFUNC)
-                ((void (*)(unsigned)) opt.arg)((unsigned) tmp);
-            else if (opt.arg)
-                *((unsigned *) opt.arg) = (unsigned) tmp;
+           if (opt.arg)
+               *((unsigned *) opt.arg) = (unsigned) tmp;
         } else /* OPT_ULONG */ {
-            if (opt.flags & OPT_CALLFUNC)
-                ((void (*)(unsigned long)) opt.arg)(tmp);
-            else if (opt.arg)
+            if (opt.arg)
                 *((unsigned long *) opt.arg) = tmp;
         }
         break;
@@ -354,9 +341,7 @@ optExecute(optEntry  const opt, char *arg, int lng)
         if (errno == ERANGE)
             optFatal("floating point number `%s' to `%s' out of range\n",
                      arg, optString(opt, lng));
-        if (opt.flags & OPT_CALLFUNC)
-            ((void (*)(float)) opt.arg)(tmp);
-        else if (opt.arg)
+        if (opt.arg)
             *((float *) opt.arg) = tmp;
         break;
     }
