@@ -13,13 +13,17 @@
 ** implied warranty.
 */
 
+#include <assert.h>
 #include <string.h>
 
 #include "pam.h"
 #include "shhopt.h"
 
 enum function {FN_ADD, FN_SUBTRACT, FN_MULTIPLY, FN_DIFFERENCE,
-               FN_MINIMUM, FN_MAXIMUM, FN_MEAN, FN_COMPARE};
+               FN_MINIMUM, FN_MAXIMUM, FN_MEAN, FN_COMPARE,
+               FN_AND, FN_OR, FN_NAND, FN_NOR, FN_XOR,
+               FN_SHIFTLEFT, FN_SHIFTRIGHT
+              };
 
 struct cmdlineInfo {
     /* All the information the user supplied in the command line,
@@ -47,7 +51,9 @@ parseCommandLine(int argc, char ** const argv,
     unsigned int option_def_index;
     
     unsigned int addSpec, subtractSpec, multiplySpec, differenceSpec,
-        minimumSpec, maximumSpec, meanSpec, compareSpec;
+        minimumSpec, maximumSpec, meanSpec, compareSpec,
+    andSpec, orSpec, nandSpec, norSpec, xorSpec,
+    shiftleftSpec, shiftrightSpec;
 
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3(0, "add",         OPT_FLAG,   NULL, &addSpec,        0);
@@ -58,6 +64,13 @@ parseCommandLine(int argc, char ** const argv,
     OPTENT3(0, "maximum",     OPT_FLAG,   NULL, &maximumSpec,    0);
     OPTENT3(0, "mean",        OPT_FLAG,   NULL, &meanSpec,       0);
     OPTENT3(0, "compare",     OPT_FLAG,   NULL, &compareSpec,    0);
+    OPTENT3(0, "and",         OPT_FLAG,   NULL, &andSpec,        0);
+    OPTENT3(0, "or",          OPT_FLAG,   NULL, &orSpec,         0);
+    OPTENT3(0, "nand",        OPT_FLAG,   NULL, &nandSpec,       0);
+    OPTENT3(0, "nor",         OPT_FLAG,   NULL, &norSpec,        0);
+    OPTENT3(0, "xor",         OPT_FLAG,   NULL, &xorSpec,        0);
+    OPTENT3(0, "shiftleft",   OPT_FLAG,   NULL, &shiftleftSpec,  0);
+    OPTENT3(0, "shiftright",  OPT_FLAG,   NULL, &shiftrightSpec, 0);
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
@@ -67,7 +80,9 @@ parseCommandLine(int argc, char ** const argv,
         /* Uses and sets argc, argv, and some of *cmdlineP and others. */
 
     if (addSpec + subtractSpec + multiplySpec + differenceSpec +
-        minimumSpec + maximumSpec + meanSpec + compareSpec > 1)
+        minimumSpec + maximumSpec + meanSpec + compareSpec +
+        andSpec + orSpec + nandSpec + norSpec + xorSpec +
+        shiftleftSpec + shiftrightSpec > 1)
         pm_error("You may specify only one function");
 
     if (argc-1 != 2)
@@ -95,12 +110,60 @@ parseCommandLine(int argc, char ** const argv,
         cmdlineP->function = FN_MEAN;
     else if (compareSpec)
         cmdlineP->function = FN_COMPARE;
+    else if (andSpec)
+        cmdlineP->function = FN_AND;
+    else if (orSpec)
+        cmdlineP->function = FN_OR;
+    else if (nandSpec)
+        cmdlineP->function = FN_NAND;
+    else if (norSpec)
+        cmdlineP->function = FN_NOR;
+    else if (xorSpec)
+        cmdlineP->function = FN_XOR;
+    else if (shiftleftSpec)
+        cmdlineP->function = FN_SHIFTLEFT;
+    else if (shiftrightSpec)
+        cmdlineP->function = FN_SHIFTRIGHT;
     else
         pm_error("You must specify a function (e.g. '-add')");
 }        
 
 
 
+enum category {CATEGORY_FRACTIONAL_ARITH, CATEGORY_BITSTRING, CATEGORY_SHIFT};
+
+static enum category
+functionCategory(enum function const function) {
+
+    enum category retval;
+    
+    switch (function) {
+    case FN_ADD:
+    case FN_SUBTRACT:
+    case FN_MULTIPLY:
+    case FN_DIFFERENCE:
+    case FN_MINIMUM:
+    case FN_MAXIMUM:
+    case FN_MEAN:
+    case FN_COMPARE:
+        retval = CATEGORY_FRACTIONAL_ARITH;
+        break;
+    case FN_AND:
+    case FN_OR:
+    case FN_NAND:
+    case FN_NOR:
+    case FN_XOR:
+        retval = CATEGORY_BITSTRING;
+        break;
+    case FN_SHIFTLEFT:
+    case FN_SHIFTRIGHT:
+        retval = CATEGORY_SHIFT;
+        break;
+    }
+    return retval;
+}
+
+
 static void
 computeOutputType(struct pam *  const outpamP,
                   struct pam    const inpam1,
@@ -115,15 +178,113 @@ computeOutputType(struct pam *  const outpamP,
     outpamP->height      = inpam1.height;
     outpamP->width       = inpam1.width;
     outpamP->depth       = MAX(inpam1.depth, inpam2.depth);
-    outpamP->maxval      = MAX(inpam1.maxval, inpam2.maxval);
-    if (function == FN_COMPARE)
-        outpamP->maxval = MAX(outpamP->maxval, 2);
+
+    switch (functionCategory(function)) {    
+    case CATEGORY_FRACTIONAL_ARITH:
+        if (function == FN_COMPARE)
+            outpamP->maxval = 2;
+        else
+            outpamP->maxval = MAX(inpam1.maxval, inpam2.maxval);
+        break;
+    case CATEGORY_BITSTRING:
+        if (inpam2.maxval != inpam1.maxval)
+            pm_error("For a bit string operation, the maxvals of the "
+                     "left and right image must be the same.  You have "
+                     "left=%u and right=%u", 
+                     (unsigned)inpam1.maxval, (unsigned)inpam2.maxval);
+
+        if (pm_bitstomaxval(pm_maxvaltobits(inpam1.maxval)) != inpam1.maxval)
+            pm_error("For a bit string operation, the maxvals of the inputs "
+                     "must be a full binary count, i.e. a power of two "
+                     "minus one such as 0xff.  You have 0x%x",
+                     (unsigned)inpam1.maxval);
+
+        outpamP->maxval = inpam1.maxval;
+        break;
+    case CATEGORY_SHIFT:
+        if (pm_bitstomaxval(pm_maxvaltobits(inpam1.maxval)) != inpam1.maxval)
+            pm_error("For a bit shift operation, the maxval of the left "
+                     "input image "
+                     "must be a full binary count, i.e. a power of two "
+                     "minus one such as 0xff.  You have 0x%x",
+                     (unsigned)inpam1.maxval);
+        outpamP->maxval = inpam1.maxval;
+    }
     outpamP->bytes_per_sample = (pm_maxvaltobits(outpamP->maxval)+7)/8;
     strcpy(outpamP->tuple_type, inpam1.tuple_type);
 }
 
 
 
+static sample
+applyFunction(enum function const function,
+              sample        const leftArg,
+              sample        const rightArg,
+              sample        const resultMaxval) {
+
+    sample retval;
+    double resultArith;
+    unsigned int resultBit;
+
+    switch (function) {
+    case FN_ADD:
+        resultArith = MIN(resultMaxval, leftArg + rightArg);
+        break;
+    case FN_SUBTRACT:
+        resultArith = MAX(0, (int)leftArg - (int)rightArg);
+        break;
+    case FN_MULTIPLY:
+        resultArith = leftArg * rightArg / resultMaxval;
+        break;
+    case FN_DIFFERENCE:
+        resultArith = leftArg > rightArg ? 
+            leftArg - rightArg : rightArg - leftArg;
+        break;
+    case FN_MINIMUM:
+        resultArith = MIN(leftArg, rightArg);
+        break;
+    case FN_MAXIMUM:
+        resultArith = MAX(leftArg, rightArg);
+        break;
+    case FN_MEAN:
+        resultArith = (leftArg + rightArg) / 2.0;
+        break;
+    case FN_COMPARE:
+        resultArith = 
+            leftArg > rightArg ? 2 : leftArg < rightArg ? 0 : 1;
+        break;
+    case FN_AND:
+        resultBit = leftArg & rightArg;
+        break;
+    case FN_OR:
+        resultBit = leftArg | rightArg;
+        break;
+    case FN_NAND:
+        resultBit = ~(leftArg & rightArg) & resultMaxval;
+        break;
+    case FN_NOR:
+        resultBit = ~(leftArg | rightArg) & resultMaxval;
+        break;
+    case FN_XOR:
+        resultBit = leftArg ^ rightArg;
+        break;
+    case FN_SHIFTLEFT:
+        resultBit = (leftArg << rightArg) & resultMaxval;
+        break;
+    case FN_SHIFTRIGHT:
+        resultBit = leftArg >> rightArg;
+        break;
+    }
+    if (functionCategory(function) == CATEGORY_FRACTIONAL_ARITH)
+        retval = (sample) (resultArith + 0.5);
+    else
+        retval = resultBit;
+
+    return retval;
+}
+
+
+
 int
 main(int argc, char *argv[]) {
 
@@ -184,49 +345,14 @@ main(int argc, char *argv[]) {
                 unsigned int const plane1 = MIN(outplane, inpam1.depth-1);
                 unsigned int const plane2 = MIN(outplane, inpam2.depth-1);
 
-                double result;
+                tuplerowOut[col][outplane] = 
+                    applyFunction(cmdline.function, 
+                                  tuplerow1[col][plane1], 
+                                  tuplerow2[col][plane2],
+                                  outpam.maxval);
 
-                switch (cmdline.function) {
-                case FN_ADD:
-                    result = tuplerow1[col][plane1] + tuplerow2[col][plane2];
-                    break;
-                case FN_SUBTRACT:
-                    result = (int)tuplerow1[col][plane1] - 
-                        (int)tuplerow2[col][plane2];
-                    break;
-                case FN_MULTIPLY:
-                    result = (double)tuplerow1[col][plane1] *
-                        tuplerow2[col][plane2] /
-                        outpam.maxval;
-                    break;
-                case FN_DIFFERENCE:
-                    result =
-                        tuplerow1[col][plane1] > tuplerow2[col][plane2] ?
-                        tuplerow1[col][plane1] - tuplerow2[col][plane2] :
-                            tuplerow2[col][plane2] - tuplerow1[col][plane1];
-                    break;
-                case FN_MINIMUM:
-                    result = 
-                        MIN(tuplerow1[col][plane1], tuplerow2[col][plane2]);
-                    break;
-                case FN_MAXIMUM:
-                    result = 
-                        MAX(tuplerow1[col][plane1], tuplerow2[col][plane2]);
-                    break;
-                case FN_MEAN:
-                    result = 
-                        (tuplerow1[col][plane1] + tuplerow2[col][plane2])/2.0;
-                    break;
-                case FN_COMPARE:
-                    result = 
-                        tuplerow1[col][plane1] > tuplerow2[col][plane2] ? 
-                        2 : tuplerow1[col][plane1] < tuplerow2[col][plane2] ?
-                        0 : 1;
-                    break;
-                }
-                result = MAX(0, result);
-                result = MIN(outpam.maxval, result);
-                tuplerowOut[col][outplane] = (unsigned int) (result + 0.5);
+                assert(tuplerowOut[col][outplane] >= 0);
+                assert(tuplerowOut[col][outplane] <= outpam.maxval);
             }
         }
         pnm_writepamrow(&outpam, tuplerowOut);
