@@ -24,7 +24,6 @@
 ******************************************************************************/
 
 #include <math.h>
-#include <assert.h>
 
 #include "pam.h"
 #include "pammap.h"
@@ -209,8 +208,6 @@ initSampleCompare(void) {
     samplecompare[0].fn = compareplane0;
     samplecompare[1].fn = compareplane1;
     samplecompare[2].fn = compareplane2;
-
-    assert(MAXDEPTH == 3);
 }
 
 
@@ -524,10 +521,6 @@ splitBox(boxVector             const bv,
     int lowersum;
     /* Number of pixels whose value is "less than" the median */
 
-    if (depth > MAXDEPTH)
-        pm_error("Depth of image (%u) is too high.  Maximum depth "
-                 "this program can process is %u", depth, MAXDEPTH);
-
     findBoxBoundaries(colorfreqtable, depth, boxStart, boxSize, 
                       minval, maxval);
 
@@ -934,5 +927,4 @@ main(int argc, char * argv[] ) {
     pm_close(stdout);
 
     return 0;
-    
 }
